searchData={"content_type":"text/markdown","items":[{"doc":"Documentation for the Absinthe package, a toolkit for building GraphQL\nAPIs with Elixir.\n\nFor usage information, see [the documentation](https://hexdocs.pm/absinthe), which\nincludes guides, API information for important modules, and links to useful resources.","ref":"Absinthe.html","title":"Absinthe","type":"module"},{"doc":"Evaluates a query document against a schema, with options.","ref":"Absinthe.html#run/3","title":"Absinthe.run/3","type":"function"},{"doc":"* `:adapter` - The name of the adapter to use. See the `Absinthe.Adapter`\n  behaviour and the `Absinthe.Adapter.Passthrough` and\n  `Absinthe.Adapter.LanguageConventions` modules that implement it.\n  (`Absinthe.Adapter.LanguageConventions` is the default value for this option.)\n* `:operation_name` - If more than one operation is present in the provided\n  query document, this must be provided to select which operation to execute.\n* `:variables` - A map of provided variable values to be used when filling in\n  arguments in the provided query document.\n* `:context` -> A map of the execution context.\n* `:root_value` -> A root value to use as the source for toplevel fields.\n* `:analyze_complexity` -> Whether to analyze the complexity before\nexecuting an operation.\n* `:max_complexity` -> An integer (or `:infinity`) for the maximum allowed\ncomplexity for the operation being executed.","ref":"Absinthe.html#run/3-options","title":"Options - Absinthe.run/3","type":"function"},{"doc":"```\n\"\"\"\nquery GetItemById($id: ID) {\n  item(id: $id) {\n    name\n  }\n}\n\"\"\"\n|> Absinthe.run(App.Schema, variables: %{\"id\" => params[:item_id]})\n```\n\nSee the `Absinthe` module documentation for more examples.","ref":"Absinthe.html#run/3-examples","title":"Examples - Absinthe.run/3","type":"function"},{"doc":"Evaluates a query document against a schema, without options.","ref":"Absinthe.html#run!/3","title":"Absinthe.run!/3","type":"function"},{"doc":"See `run/3` for the available options.","ref":"Absinthe.html#run!/3-options","title":"Options - Absinthe.run!/3","type":"function"},{"doc":"","ref":"Absinthe.html#t:pipeline_modifier_fun/0","title":"Absinthe.pipeline_modifier_fun/0","type":"type"},{"doc":"","ref":"Absinthe.html#t:result_error_t/0","title":"Absinthe.result_error_t/0","type":"type"},{"doc":"","ref":"Absinthe.html#t:result_selection_t/0","title":"Absinthe.result_selection_t/0","type":"type"},{"doc":"","ref":"Absinthe.html#t:result_t/0","title":"Absinthe.result_t/0","type":"type"},{"doc":"","ref":"Absinthe.html#t:run_opts/0","title":"Absinthe.run_opts/0","type":"type"},{"doc":"","ref":"Absinthe.html#t:run_result/0","title":"Absinthe.run_result/0","type":"type"},{"doc":"Formatter task for graphql\n\nWill format files with the extensions .graphql or .gql","ref":"Absinthe.Formatter.html","title":"Absinthe.Formatter","type":"module"},{"doc":"```elixir\nAbsinthe.Formatter.format(\"{ version }\")\n\"{\n  version\n}\n\"\n```\n\n\nFrom Elixir 1.13 onwards the Absinthe.Formatter can be added to\nthe formatter as a plugin:\n\n```elixir\n# .formatter.exs\n[\n  # Define the desired plugins\n  plugins: [Absinthe.Formatter],\n  # Remember to update the inputs list to include the new extensions\n  inputs: [\"{mix,.formatter}.exs\", \"{config,lib,test}/**/*.{ex,exs}\", \"{lib,priv}/**/*.{gql,graphql}\"]\n]\n```","ref":"Absinthe.Formatter.html#module-example","title":"Example - Absinthe.Formatter","type":"module"},{"doc":"","ref":"Absinthe.Formatter.html#features/1","title":"Absinthe.Formatter.features/1","type":"function"},{"doc":"","ref":"Absinthe.Formatter.html#format/2","title":"Absinthe.Formatter.format/2","type":"function"},{"doc":"","ref":"Absinthe.Lexer.html","title":"Absinthe.Lexer","type":"module"},{"doc":"Parses the given `binary` as do_tokenize.\n\nReturns `{:ok, [token], rest, context, position, byte_offset}` or\n`{:error, reason, rest, context, line, byte_offset}` where `position`\ndescribes the location of the do_tokenize (start position) as `{line, offset_to_start_of_line}`.\n\nTo column where the error occurred can be inferred from `byte_offset - offset_to_start_of_line`.","ref":"Absinthe.Lexer.html#do_tokenize/2","title":"Absinthe.Lexer.do_tokenize/2","type":"function"},{"doc":"* `:byte_offset` - the byte offset for the whole binary, defaults to 0\n  * `:line` - the line and the byte offset into that line, defaults to `{1, byte_offset}`\n  * `:context` - the initial context value. It will be converted to a map","ref":"Absinthe.Lexer.html#do_tokenize/2-options","title":"Options - Absinthe.Lexer.do_tokenize/2","type":"function"},{"doc":"","ref":"Absinthe.Lexer.html#line_and_column/3","title":"Absinthe.Lexer.line_and_column/3","type":"function"},{"doc":"","ref":"Absinthe.Lexer.html#tokenize/2","title":"Absinthe.Lexer.tokenize/2","type":"function"},{"doc":"","ref":"Absinthe.Schema.Manager.html","title":"Absinthe.Schema.Manager","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Absinthe.Schema.Manager.html#child_spec/1","title":"Absinthe.Schema.Manager.child_spec/1","type":"function"},{"doc":"","ref":"Absinthe.Schema.Manager.html#init/1","title":"Absinthe.Schema.Manager.init/1","type":"function"},{"doc":"","ref":"Absinthe.Schema.Manager.html#start_link/1","title":"Absinthe.Schema.Manager.start_link/1","type":"function"},{"doc":"Experimental: Persistent Term based Schema Backend\n\nBy default, Absinthe schemas are stored in a generated module. If your schema\nis called `MyAppWeb.Schema`, Absinthe creates a `MyAppWeb.Schema.Compiled`\nmodule containing the structs and other data that Absinthe needs at runtime\nto execute GraphQL operations against that schema.\n\nOTP introduced the `:persistent_term` module to provide many of the same\nperformance benefits of using compiled modules, without the downsides associated\nwith manipulating complex structures at compile time.\n\nThis module is an experimental effort at using the `:persistent_term` module\nas an Absinthe schema backend. This module has been tested against against\nthe entire Absinthe test suite and shown to perform perhaps even better\nthan the compiled module.\n\nTo use:\n\nIn your schema module:\n```\nuse Absinthe.Schema\n@schema_provider Absinthe.Schema.PersistentTerm\n```\n\nIn your application's supervision tree, prior to anywhere where you'd use\nthe schema:\n```\n{Absinthe.Schema, MyAppWeb.Schema}\n```\n\nwhere MyAppWeb.Schema is the name of your schema.","ref":"Absinthe.Schema.PersistentTerm.html","title":"Absinthe.Schema.PersistentTerm","type":"module"},{"doc":"","ref":"Absinthe.Schema.PersistentTerm.html#pipeline/1","title":"Absinthe.Schema.PersistentTerm.pipeline/1","type":"function"},{"doc":"Experimental: Behaviour for providing schema data\n\nThis behaviour is experimental and may change significantly in patch releases.","ref":"Absinthe.Schema.Provider.html","title":"Absinthe.Schema.Provider","type":"behaviour"},{"doc":"","ref":"Absinthe.Schema.Provider.html#c:__absinthe_directive__/2","title":"Absinthe.Schema.Provider.__absinthe_directive__/2","type":"callback"},{"doc":"","ref":"Absinthe.Schema.Provider.html#c:__absinthe_directives__/1","title":"Absinthe.Schema.Provider.__absinthe_directives__/1","type":"callback"},{"doc":"","ref":"Absinthe.Schema.Provider.html#c:__absinthe_interface_implementors__/1","title":"Absinthe.Schema.Provider.__absinthe_interface_implementors__/1","type":"callback"},{"doc":"","ref":"Absinthe.Schema.Provider.html#c:__absinthe_type__/2","title":"Absinthe.Schema.Provider.__absinthe_type__/2","type":"callback"},{"doc":"","ref":"Absinthe.Schema.Provider.html#c:__absinthe_types__/1","title":"Absinthe.Schema.Provider.__absinthe_types__/1","type":"callback"},{"doc":"","ref":"Absinthe.Schema.Provider.html#c:__absinthe_types__/2","title":"Absinthe.Schema.Provider.__absinthe_types__/2","type":"callback"},{"doc":"","ref":"Absinthe.Schema.Provider.html#c:pipeline/1","title":"Absinthe.Schema.Provider.pipeline/1","type":"callback"},{"doc":"","ref":"Absinthe.Schema.Provider.html#t:schema_identifier/0","title":"Absinthe.Schema.Provider.schema_identifier/0","type":"type"},{"doc":"","ref":"Absinthe.Schema.Provider.html#t:type_group/0","title":"Absinthe.Schema.Provider.type_group/0","type":"type"},{"doc":"Generate a `schema.json` file","ref":"Mix.Tasks.Absinthe.Schema.Json.html","title":"mix absinthe.schema.json","type":"task"},{"doc":"mix absinthe.schema.json [OPTIONS] [FILENAME]\n\nThe JSON codec to be used needs to be included in your `mix.exs` dependencies. If using the default codec,\nsee the Jason [installation instructions](https://hexdocs.pm/jason).","ref":"Mix.Tasks.Absinthe.Schema.Json.html#module-usage","title":"Usage - mix absinthe.schema.json","type":"task"},{"doc":"* `--schema` - The name of the `Absinthe.Schema` module defining the schema to be generated.\n   Default: As [configured](https://hexdocs.pm/mix/Mix.Config.html) for `:absinthe` `:schema`\n* `--json-codec` - Codec to use to generate the JSON file (see [Custom Codecs](#module-custom-codecs)).\n   Default: [`Jason`](https://hexdocs.pm/jason/)\n* `--pretty` - Whether to pretty-print.\n   Default: `false`","ref":"Mix.Tasks.Absinthe.Schema.Json.html#module-options","title":"Options - mix absinthe.schema.json","type":"task"},{"doc":"Write to default path `./schema.json` using the `:schema` configured for the `:absinthe` application:\n\n    mix absinthe.schema.json\n\nWrite to default path `./schema.json` using the `MySchema` schema:\n\n    mix absinthe.schema.json --schema MySchema\n\nWrite to path `/path/to/schema.json` using the `MySchema` schema, with pretty-printing:\n\n    mix absinthe.schema.json --schema MySchema --pretty /path/to/schema.json\n\nWrite to default path `./schema.json` using the `MySchema` schema and a custom JSON codec, `MyCodec`:\n\n    mix absinthe.schema.json --schema MySchema --json-codec MyCodec","ref":"Mix.Tasks.Absinthe.Schema.Json.html#module-examples","title":"Examples - mix absinthe.schema.json","type":"task"},{"doc":"Any module that provides `encode!/2` can be used as a custom codec:\n\n    encode!(value, options)\n\n* `value` will be provided as a Map containing the generated schema.\n* `options` will be a keyword list with a `:pretty` boolean, indicating whether the user requested pretty-printing.\n\nThe function should return a string to be written to the output file.","ref":"Mix.Tasks.Absinthe.Schema.Json.html#module-custom-codecs","title":"Custom Codecs - mix absinthe.schema.json","type":"task"},{"doc":"Callback implementation for `Mix.Task.run/1`, which receives a list of command-line args.","ref":"Mix.Tasks.Absinthe.Schema.Json.html#run/1","title":"Mix.Tasks.Absinthe.Schema.Json.run/1","type":"function"},{"doc":"Generate a `schema.graphql` file.","ref":"Mix.Tasks.Absinthe.Schema.Sdl.html","title":"mix absinthe.schema.sdl","type":"task"},{"doc":"mix absinthe.schema.sdl [OPTIONS] [FILENAME]","ref":"Mix.Tasks.Absinthe.Schema.Sdl.html#module-usage","title":"Usage - mix absinthe.schema.sdl","type":"task"},{"doc":"* `--schema` - The name of the `Absinthe.Schema` module defining the schema to be generated.\n   Default: As [configured](https://hexdocs.pm/mix/Mix.Config.html) for `:absinthe` `:schema`","ref":"Mix.Tasks.Absinthe.Schema.Sdl.html#module-options","title":"Options - mix absinthe.schema.sdl","type":"task"},{"doc":"Write to default path `./schema.graphql` using the `:schema` configured for the `:absinthe` application:\n\n    mix absinthe.schema.sdl\n\nWrite to path `/path/to/schema.graphql` using the `MySchema` schema\n\n    mix absinthe.schema.sdl --schema MySchema /path/to/schema.graphql","ref":"Mix.Tasks.Absinthe.Schema.Sdl.html#module-examples","title":"Examples - mix absinthe.schema.sdl","type":"task"},{"doc":"","ref":"Mix.Tasks.Absinthe.Schema.Sdl.html#generate_schema/1","title":"Mix.Tasks.Absinthe.Schema.Sdl.generate_schema/1","type":"function"},{"doc":"","ref":"Mix.Tasks.Absinthe.Schema.Sdl.html#parse_options/1","title":"Mix.Tasks.Absinthe.Schema.Sdl.parse_options/1","type":"function"},{"doc":"Handy functions for returning async or batched resolution functions\n\nUsing `Absinthe.Schema.Notation` or (by extension) `Absinthe.Schema` will\nautomatically import the `batch` and `async` helpers. Dataloader helpers\nrequire an explicit `import Absinthe.Resolution.Helpers` invocation, since\ndataloader is an optional dependency.","ref":"Absinthe.Resolution.Helpers.html","title":"Absinthe.Resolution.Helpers","type":"module"},{"doc":"Execute resolution field asynchronously.\n\nThis is a helper function for using the `Absinthe.Middleware.Async`.\n\nForbidden in mutation fields. (TODO: actually enforce this)","ref":"Absinthe.Resolution.Helpers.html#async/2","title":"Absinthe.Resolution.Helpers.async/2","type":"function"},{"doc":"- `:timeout` default: `30_000`. The maximum timeout to wait for running\n   the task.","ref":"Absinthe.Resolution.Helpers.html#async/2-options","title":"Options - Absinthe.Resolution.Helpers.async/2","type":"function"},{"doc":"Using the `Absinthe.Resolution.Helpers.async/1` helper function:\n```elixir\nfield :time_consuming, :thing do\n  resolve fn _, _, _ ->\n    async(fn ->\n      {:ok, long_time_consuming_function()}\n    end)\n  end\nend\n```","ref":"Absinthe.Resolution.Helpers.html#async/2-example","title":"Example - Absinthe.Resolution.Helpers.async/2","type":"function"},{"doc":"Batch the resolution of several functions together.\n\nHelper function for creating `Absinthe.Middleware.Batch`","ref":"Absinthe.Resolution.Helpers.html#batch/4","title":"Absinthe.Resolution.Helpers.batch/4","type":"function"},{"doc":"- `:timeout` default: `5_000`. The maximum timeout to wait for running\n  a batch.","ref":"Absinthe.Resolution.Helpers.html#batch/4-options","title":"Options - Absinthe.Resolution.Helpers.batch/4","type":"function"},{"doc":"Raw usage:\n```elixir\nobject :post do\n  field :name, :string\n  field :author, :user do\n    resolve fn post, _, _ ->\n      batch({__MODULE__, :users_by_id}, post.author_id, fn batch_results ->\n        {:ok, Map.get(batch_results, post.author_id)}\n      end)\n    end\n  end\nend\n\ndef users_by_id(_, user_ids) do\n  users = Repo.all from u in User, where: u.id in ^user_ids\n  Map.new(users, fn user -> {user.id, user} end)\nend\n```","ref":"Absinthe.Resolution.Helpers.html#batch/4-example","title":"Example - Absinthe.Resolution.Helpers.batch/4","type":"function"},{"doc":"Resolve a field with a dataloader source.\n\nThis function is not imported by default. To make it available in your module do\n\n```\nimport Absinthe.Resolution.Helpers\n```\n\nSame as `dataloader/3`, but it infers the resource name from the field name.","ref":"Absinthe.Resolution.Helpers.html#dataloader/1","title":"Absinthe.Resolution.Helpers.dataloader/1","type":"function"},{"doc":"```\nfield :author, :user, resolve: dataloader(Blog)\n```\n\nThis is identical to doing the following.\n\n```\nfield :author, :user, resolve: dataloader(Blog, :author, [])\n```","ref":"Absinthe.Resolution.Helpers.html#dataloader/1-examples","title":"Examples - Absinthe.Resolution.Helpers.dataloader/1","type":"function"},{"doc":"Resolve a field with a dataloader source.\n\nThis function is not imported by default. To make it available in your module do\n\n```\nimport Absinthe.Resolution.Helpers\n```\n\nSame as `dataloader/3`, but it infers the resource name from the field name. For `opts` see\n`dataloader/3` on what options can be passed in.","ref":"Absinthe.Resolution.Helpers.html#dataloader/2","title":"Absinthe.Resolution.Helpers.dataloader/2","type":"function"},{"doc":"```\nobject :user do\n  field :posts, list_of(:post),\n    resolve: dataloader(Blog, args: %{deleted: false})\n\n  field :organization, :organization do\n    resolve dataloader(Accounts, use_parent: false)\n  end\n\n  field(:account_active, non_null(:boolean), resolve: dataloader(\n      Accounts, callback: fn account, _parent, _args ->\n        {:ok, account.active}\n      end\n    )\n  )\nend\n```","ref":"Absinthe.Resolution.Helpers.html#dataloader/2-examples","title":"Examples - Absinthe.Resolution.Helpers.dataloader/2","type":"function"},{"doc":"Resolve a field with Dataloader\n\nThis function is not imported by default. To make it available in your module do\n\n```\nimport Absinthe.Resolution.Helpers\n```\n\nWhile `on_load/2` makes using dataloader directly easy within a resolver function,\nit is often unnecessary to need this level of direct control.\n\nThe `dataloader/3` function exists to provide a simple API for using dataloader.\nIt takes the name of a data source, the name of the resource you want to load,\nand then a variety of options.","ref":"Absinthe.Resolution.Helpers.html#dataloader/3","title":"Absinthe.Resolution.Helpers.dataloader/3","type":"function"},{"doc":"```\nobject :user do\n  field :posts, list_of(:post),\n    resolve: dataloader(Blog, :posts, args: %{deleted: false})\n\n  field :organization, :organization do\n    resolve dataloader(Accounts, :organization, use_parent: false)\n  end\n\n  field(:account_active, non_null(:boolean), resolve: dataloader(\n      Accounts, :account, callback: fn account, _parent, _args ->\n        {:ok, account.active}\n      end\n    )\n  )\nend\n```","ref":"Absinthe.Resolution.Helpers.html#dataloader/3-basic-usage","title":"Basic Usage - Absinthe.Resolution.Helpers.dataloader/3","type":"function"},{"doc":"Instead of passing in a literal like `:posts` or `:organization` in as the resource,\nit is also possible pass in a function:\n\n```\nobject :user do\n  field :posts, list_of(:post) do\n    arg :limit, non_null(:integer)\n    resolve dataloader(Blog, fn user, args, info ->\n      args = Map.update!(args, :limit, fn val ->\n        max(min(val, 20), 0)\n      end)\n      {:posts, args}\n    end)\n  end\nend\n```\n\nIn this case we want to make sure that the limit value cannot be larger than\n`20`. By passing a callback function to `dataloader/2` we can ensure that\nthe value will fall nicely between 0 and 20.\n\nIf you have defined a\n[custom batch query](https://hexdocs.pm/dataloader/Dataloader.Ecto.html#module-custom-batch-queries),\nyou can instead return a `Map` with `:batch` key to specify the `batch_key` of `Dataloader.load/4` /\n`Dataloader.get/4` and an `:item` key to specify `item_key` argument of `Dataloader.get/4`.\n\n```\n# ...\nresolve dataloader(Posts, fn user, _args, _info ->\n  %{batch: {{:one, Post}, %{}}, item: [post_count: user]}\nend)\n# ...\n```","ref":"Absinthe.Resolution.Helpers.html#dataloader/3-key-functions","title":"Key Functions - Absinthe.Resolution.Helpers.dataloader/3","type":"function"},{"doc":"- `:args` default: `%{}`. Any arguments you want to always pass into the\n`Dataloader.load/4` call. Resolver arguments are merged into this value and,\nin the event of a conflict, the resolver arguments win.\n- `:callback` default: return result wrapped in ok or error tuple.\nCallback that is run with result of dataloader. It receives the result as\nthe first argument, and the parent and args as second and third.\nOptionally can receive resolution as the fourth argument. Can be used\nto e.g. compute fields on the return value of the loader. Should return an\nok or error tuple.\n- `:use_parent` default: `false`. This option affects whether or not the `dataloader/2`\nhelper will use any pre-existing value on the parent. IE if you return\n`%{author: %User{...}}` from a blog post the helper will by default simply use\nthe pre-existing author. Set it to true if you want to opt into using the\npre-existing value instead of loading it fresh.\n\nUltimately, this helper calls `Dataloader.load/4`\nusing the loader in your context, the source you provide, the tuple `{resource, args}`\nas the batch key, and then the parent value of the field\n\n```\ndef dataloader(source_name, resource) do\n  fn parent, args, %{context: %{loader: loader}} ->\n    args = Map.merge(opts[:args] || %{}, args)\n    loader\n    |> Dataloader.load(source_name, {resource, args}, parent)\n    |> on_load(fn loader ->\n      {:ok, Dataloader.get(loader, source_name, {resource, args}, parent)}\n    end)\n  end\nend\n```","ref":"Absinthe.Resolution.Helpers.html#dataloader/3-options","title":"Options - Absinthe.Resolution.Helpers.dataloader/3","type":"function"},{"doc":"Dataloader helper function\n\nThis function is not imported by default. To make it available in your module do\n\n```\nimport Absinthe.Resolution.Helpers\n```\n\nThis function helps you use data loader in a direct way within your schema.\nWhile normally the `dataloader/1,2,3` helpers are enough, `on_load/2` is useful\nwhen you want to load multiple things in a single resolver, or when you need\nfine grained control over the dataloader cache.","ref":"Absinthe.Resolution.Helpers.html#on_load/2","title":"Absinthe.Resolution.Helpers.on_load/2","type":"function"},{"doc":"```elixir\nfield :reports, list_of(:report) do\n  resolve fn shipment, _, %{context: %{loader: loader}} ->\n    loader\n    |> Dataloader.load(SourceName, :automatic_reports, shipment)\n    |> Dataloader.load(SourceName, :manual_reports, shipment)\n    |> on_load(fn loader ->\n      reports =\n        loader\n        |> Dataloader.get(SourceName, :automatic_reports, shipment)\n        |> Enum.concat(Dataloader.get(loader, SourceName, :manual_reports, shipment))\n        |> Enum.sort_by(&reported_at/1)\n      {:ok, reports}\n    end)\n  end\nend\n```","ref":"Absinthe.Resolution.Helpers.html#on_load/2-examples","title":"Examples - Absinthe.Resolution.Helpers.on_load/2","type":"function"},{"doc":"","ref":"Absinthe.Resolution.Helpers.html#t:dataloader_key_fun/0","title":"Absinthe.Resolution.Helpers.dataloader_key_fun/0","type":"type"},{"doc":"","ref":"Absinthe.Resolution.Helpers.html#t:dataloader_opt/0","title":"Absinthe.Resolution.Helpers.dataloader_opt/0","type":"type"},{"doc":"","ref":"Absinthe.Resolution.Helpers.html#t:dataloader_tuple/0","title":"Absinthe.Resolution.Helpers.dataloader_tuple/0","type":"type"},{"doc":"Build GraphQL Schemas","ref":"Absinthe.Schema.html","title":"Absinthe.Schema","type":"behaviour"},{"doc":"In Absinthe 1.5 and up schemas are built using the same process by which queries are\nexecuted. All the macros in this module and in `Notation` build up an intermediary tree of structs in the\n`%Absinthe.Blueprint{}` namespace, which we generally call \"Blueprint structs\".\n\nAt the top you've got a `%Blueprint{}` struct which holds onto some schema\ndefinitions that look a bit like this:\n\n```\n%Blueprint.Schema.SchemaDefinition{\n  type_definitions: [\n    %Blueprint.Schema.ObjectTypeDefinition{identifier: :query, ...},\n    %Blueprint.Schema.ObjectTypeDefinition{identifier: :mutation, ...},\n    %Blueprint.Schema.ObjectTypeDefinition{identifier: :user, ...},\n    %Blueprint.Schema.EnumTypeDefinition{identifier: :sort_order, ...},\n  ]\n}\n```\n\nYou can see what your schema's blueprint looks like by calling\n`__absinthe_blueprint__` on any schema or type definition module.\n\n```\ndefmodule MyAppWeb.Schema do\n  use Absinthe.Schema\n\n  query do\n\n  end\nend\n\n> MyAppWeb.Schema.__absinthe_blueprint__\n#=> %Absinthe.Blueprint{...}\n```\n\nThese blueprints are manipulated by phases, which validate and ultimately\nconstruct a schema. This pipeline of phases you can hook into like you do for\nqueries.\n\n```\ndefmodule MyAppWeb.Schema do\n  use Absinthe.Schema\n\n  @pipeline_modifier MyAppWeb.CustomSchemaPhase\n\n  query do\n\n  end\n\nend\n\ndefmodule MyAppWeb.CustomSchemaPhase do\n  alias Absinthe.{Phase, Pipeline, Blueprint}\n\n  # Add this module to the pipeline of phases\n  # to run on the schema\n  def pipeline(pipeline) do\n    Pipeline.insert_after(pipeline, Phase.Schema.TypeImports, __MODULE__)\n  end\n\n  # Here's the blueprint of the schema, do whatever you want with it.\n  def run(blueprint, _) do\n    {:ok, blueprint}\n  end\nend\n```\n\nThe blueprint structs are pretty complex, but if you ever want to figure out\nhow to construct something in blueprints you can always just create the thing\nin the normal AST and then look at the output. Let's see what interfaces look\nlike for example:\n\n```\ndefmodule Foo do\n  use Absinthe.Schema.Notation\n\n  interface :named do\n    field :name, :string\n  end\nend\n\nFoo.__absinthe_blueprint__ #=> ...\n```","ref":"Absinthe.Schema.html#module-custom-schema-manipulation-in-progress","title":"Custom Schema Manipulation (in progress) - Absinthe.Schema","type":"behaviour"},{"doc":"","ref":"Absinthe.Schema.html#apply_modifiers/2","title":"Absinthe.Schema.apply_modifiers/2","type":"function"},{"doc":"","ref":"Absinthe.Schema.html#child_spec/1","title":"Absinthe.Schema.child_spec/1","type":"function"},{"doc":"Get all concrete types for union, interface, or object","ref":"Absinthe.Schema.html#concrete_types/2","title":"Absinthe.Schema.concrete_types/2","type":"function"},{"doc":"Used to set some values in the context that it may need in order to run.","ref":"Absinthe.Schema.html#c:context/1","title":"Absinthe.Schema.context/1","type":"callback"},{"doc":"Setup dataloader:\n\n```\ndef context(context) do\n  loader =\n    Dataloader.new\n    |> Dataloader.add_source(Blog, Blog.data())\n\n    Map.put(context, :loader, loader)\nend\n```","ref":"Absinthe.Schema.html#c:context/1-examples","title":"Examples - Absinthe.Schema.context/1","type":"callback"},{"doc":"List all directives on a schema","ref":"Absinthe.Schema.html#directives/1","title":"Absinthe.Schema.directives/1","type":"function"},{"doc":"Used to hydrate the schema with dynamic attributes.\n\nWhile this is normally used to add resolvers, etc, to schemas\ndefined using `import_sdl/1` and `import_sdl/2`, it can also be\nused in schemas defined using other macros.\n\nThe function is passed the blueprint definition node as the first\nargument and its ancestors in a list (with its parent node as the\nhead) as its second argument.\n\nSee the `Absinthe.Phase.Schema.Hydrate` implementation of\n`Absinthe.Schema.Hydrator` callbacks to see what hydration\nvalues can be returned.","ref":"Absinthe.Schema.html#c:hydrate/2","title":"Absinthe.Schema.hydrate/2","type":"callback"},{"doc":"Add a resolver for a field:\n\n```\ndef hydrate(%Absinthe.Blueprint.Schema.FieldDefinition{identifier: :health}, [%Absinthe.Blueprint.Schema.ObjectTypeDefinition{identifier: :query} | _]) do\n  {:resolve, &__MODULE__.health/3}\nend\n\n# Resolver implementation:\ndef health(_, _, _), do: {:ok, \"alive!\"}\n```\n\nNote that the values provided must be macro-escapable; notably, anonymous functions cannot\nbe used.\n\nYou can, of course, omit the struct names for brevity:\n\n```\ndef hydrate(%{identifier: :health}, [%{identifier: :query} | _]) do\n  {:resolve, &__MODULE__.health/3}\nend\n```\n\nAdd a description to a type:\n\n```\ndef hydrate(%Absinthe.Blueprint.Schema.ObjectTypeDefinition{identifier: :user}, _) do\n  {:description, \"A user\"}\nend\n```\n\nIf you define `hydrate/2`, don't forget to include a fallback, e.g.:\n\n```\ndef hydrate(_node, _ancestors), do: []\n```","ref":"Absinthe.Schema.html#c:hydrate/2-examples","title":"Examples - Absinthe.Schema.hydrate/2","type":"callback"},{"doc":"List all implementors of an interface on a schema","ref":"Absinthe.Schema.html#implementors/2","title":"Absinthe.Schema.implementors/2","type":"function"},{"doc":"Run the introspection query on a schema.\n\nConvenience function.","ref":"Absinthe.Schema.html#introspect/2","title":"Absinthe.Schema.introspect/2","type":"function"},{"doc":"Get all introspection types","ref":"Absinthe.Schema.html#introspection_types/1","title":"Absinthe.Schema.introspection_types/1","type":"function"},{"doc":"","ref":"Absinthe.Schema.html#lookup_directive/2","title":"Absinthe.Schema.lookup_directive/2","type":"function"},{"doc":"","ref":"Absinthe.Schema.html#lookup_type/3","title":"Absinthe.Schema.lookup_type/3","type":"function"},{"doc":"Used to apply middleware on all or a group of fields based on pattern matching.\n\nIt is passed the existing middleware for a field, the field itself, and the object\nthat the field is a part of.","ref":"Absinthe.Schema.html#c:middleware/3","title":"Absinthe.Schema.middleware/3","type":"callback"},{"doc":"Adding a `HandleChangesetError` middleware only to mutations:\n\n```\n# if it's a field for the mutation object, add this middleware to the end\ndef middleware(middleware, _field, %{identifier: :mutation}) do\n  middleware ++ [MyAppWeb.Middleware.HandleChangesetErrors]\nend\n\n# if it's any other object keep things as is\ndef middleware(middleware, _field, _object), do: middleware\n```","ref":"Absinthe.Schema.html#c:middleware/3-examples","title":"Examples - Absinthe.Schema.middleware/3","type":"callback"},{"doc":"Defines a root Mutation object\n\n```\nmutation do\n  field :create_user, :user do\n    arg :name, non_null(:string)\n    arg :email, non_null(:string)\n\n    resolve &MyApp.Web.BlogResolvers.create_user/2\n  end\nend\n```","ref":"Absinthe.Schema.html#mutation/2","title":"Absinthe.Schema.mutation/2","type":"macro"},{"doc":"Used to define the list of plugins to run before and after resolution.\n\nPlugins are modules that implement the `Absinthe.Plugin` behaviour. These modules\nhave the opportunity to run callbacks before and after the resolution of the entire\ndocument, and have access to the resolution accumulator.\n\nPlugins must be specified by the schema, so that Absinthe can make sure they are\nall given a chance to run prior to resolution.","ref":"Absinthe.Schema.html#c:plugins/0","title":"Absinthe.Schema.plugins/0","type":"callback"},{"doc":"Defines a root Query object","ref":"Absinthe.Schema.html#query/2","title":"Absinthe.Schema.query/2","type":"macro"},{"doc":"Get all types that are referenced by an operation","ref":"Absinthe.Schema.html#referenced_types/1","title":"Absinthe.Schema.referenced_types/1","type":"function"},{"doc":"Replace the default middleware.","ref":"Absinthe.Schema.html#replace_default/4","title":"Absinthe.Schema.replace_default/4","type":"function"},{"doc":"Replace the default for all fields with a string lookup instead of an atom lookup:\n\n```\ndef middleware(middleware, field, object) do\n  new_middleware = {Absinthe.Middleware.MapGet, to_string(field.identifier)}\n  middleware\n  |> Absinthe.Schema.replace_default(new_middleware, field, object)\nend\n```","ref":"Absinthe.Schema.html#replace_default/4-examples","title":"Examples - Absinthe.Schema.replace_default/4","type":"function"},{"doc":"","ref":"Absinthe.Schema.html#schema_declaration/1","title":"Absinthe.Schema.schema_declaration/1","type":"function"},{"doc":"Defines a root Subscription object\n\nSubscriptions in GraphQL let a client submit a document to the server that\noutlines what data they want to receive in the event of particular updates.\n\nFor a full walk through of how to setup your project with subscriptions and\n`Phoenix` see the `Absinthe.Phoenix` project moduledoc.\n\nWhen you push a mutation, you can have selections on that mutation result\nto get back data you need, IE\n\n```graphql\nmutation {\n  createUser(accountId: 1, name: \"bob\") {\n    id\n    account { name }\n  }\n}\n```\n\nHowever, what if you want to know when OTHER people create a new user, so that\nyour UI can update as well. This is the point of subscriptions.\n\n```graphql\nsubscription {\n  newUsers {\n    id\n    account { name }\n  }\n}\n```\n\nThe job of the subscription macros then is to give you the tools to connect\nsubscription documents with the values that will drive them. In the last example\nwe would get all users for all accounts, but you could imagine wanting just\n`newUsers(accountId: 2)`.\n\nIn your schema you articulate the interests of a subscription via the `config`\nmacro:\n\n```\nsubscription do\n  field :new_users, :user do\n    arg :account_id, non_null(:id)\n\n    config fn args, _info ->\n      {:ok, topic: args.account_id}\n    end\n  end\nend\n```\nThe topic can be any term. You can broadcast a value manually to this subscription\nby doing\n\n```\nAbsinthe.Subscription.publish(pubsub, user, [new_users: user.account_id])\n```\n\nIt's pretty common to want to associate particular mutations as the triggers\nfor one or more subscriptions, so Absinthe provides some macros to help with\nthat too.\n\n```\nsubscription do\n  field :new_users, :user do\n    arg :account_id, non_null(:id)\n\n    config fn args, _info ->\n      {:ok, topic: args.account_id}\n    end\n\n    trigger :create_user, topic: fn user ->\n      user.account_id\n    end\n  end\nend\n```\n\nThe idea with a trigger is that it takes either a single mutation `:create_user`\nor a list of mutations `[:create_user, :blah_user, ...]` and a topic function.\nThis function returns a value that is used to lookup documents on the basis of\nthe topic they returned from the `config` macro.\n\nNote that a subscription field can have `trigger` as many trigger blocks as you\nneed, in the event that different groups of mutations return different results\nthat require different topic functions.","ref":"Absinthe.Schema.html#subscription/2","title":"Absinthe.Schema.subscription/2","type":"macro"},{"doc":"Converts a schema to an SDL string\n\nPer the spec, only types that are actually referenced directly or transitively from\nthe root query, subscription, or mutation objects are included.","ref":"Absinthe.Schema.html#to_sdl/1","title":"Absinthe.Schema.to_sdl/1","type":"function"},{"doc":"Absinthe.Schema.to_sdl(MyAppWeb.Schema)\n    \"schema {\n      query {...}\n    }\"","ref":"Absinthe.Schema.html#to_sdl/1-example","title":"Example - Absinthe.Schema.to_sdl/1","type":"function"},{"doc":"List all types on a schema","ref":"Absinthe.Schema.html#types/1","title":"Absinthe.Schema.types/1","type":"function"},{"doc":"Get all types that are used by an operation","ref":"Absinthe.Schema.html#used_types/1","title":"Absinthe.Schema.used_types/1","type":"function"},{"doc":"","ref":"Absinthe.Schema.html#t:t/0","title":"Absinthe.Schema.t/0","type":"type"},{"doc":"","ref":"Absinthe.Schema.Hydrator.html","title":"Absinthe.Schema.Hydrator","type":"behaviour"},{"doc":"","ref":"Absinthe.Schema.Hydrator.html#c:apply_hydration/2","title":"Absinthe.Schema.Hydrator.apply_hydration/2","type":"callback"},{"doc":"","ref":"Absinthe.Schema.Hydrator.html#t:hydration/0","title":"Absinthe.Schema.Hydrator.hydration/0","type":"type"},{"doc":"Provides a set of macros to use when creating a schema. Especially useful\nwhen moving definitions out into a different module than the schema itself.","ref":"Absinthe.Schema.Notation.html","title":"Absinthe.Schema.Notation","type":"module"},{"doc":"defmodule MyAppWeb.Schema.Types do\n      use Absinthe.Schema.Notation\n\n      object :item do\n        field :id, :id\n        field :name, :string\n        field :status, :status_enum\n      end\n\n      enum :status_enum do\n        value :current\n        value :discontinued\n      end\n\n      # ...\n\n    end","ref":"Absinthe.Schema.Notation.html#module-example","title":"Example - Absinthe.Schema.Notation","type":"module"},{"doc":"Add an argument.\n\nSee `arg/3`","ref":"Absinthe.Schema.Notation.html#arg/2","title":"Absinthe.Schema.Notation.arg/2","type":"macro"},{"doc":"Add an argument.","ref":"Absinthe.Schema.Notation.html#arg/3","title":"Absinthe.Schema.Notation.arg/3","type":"macro"},{"doc":"Allowed under: `directive` `field`","ref":"Absinthe.Schema.Notation.html#arg/3-placement","title":"Placement - Absinthe.Schema.Notation.arg/3","type":"macro"},{"doc":"```\nfield do\n  arg :size, :integer\n  arg :name, non_null(:string), description: \"The desired name\"\n  arg :public, :boolean, default_value: true\nend\n```","ref":"Absinthe.Schema.Notation.html#arg/3-examples","title":"Examples - Absinthe.Schema.Notation.arg/3","type":"macro"},{"doc":"","ref":"Absinthe.Schema.Notation.html#build_reference/1","title":"Absinthe.Schema.Notation.build_reference/1","type":"function"},{"doc":"Set the complexity of a field\n\nFor a field, the first argument to the function you supply to `complexity/1` is the user arguments -- just as a field's resolver can use user arguments to resolve its value, the complexity function that you provide can use the same arguments to calculate the field's complexity.\n\nThe second argument passed to your complexity function is the sum of all the complexity scores of all the fields nested below the current field.\n\nAn optional third argument is passed an `Absinthe.Complexity` struct, which includes information\nlike the context passed to `Absinthe.run/3`.","ref":"Absinthe.Schema.Notation.html#complexity/1","title":"Absinthe.Schema.Notation.complexity/1","type":"macro"},{"doc":"Allowed under: `field`","ref":"Absinthe.Schema.Notation.html#complexity/1-placement","title":"Placement - Absinthe.Schema.Notation.complexity/1","type":"macro"},{"doc":"```\nquery do\n  field :people, list_of(:person) do\n    arg :limit, :integer, default_value: 10\n    complexity fn %{limit: limit}, child_complexity ->\n      # set complexity based on maximum number of items in the list and\n      # complexity of a child.\n      limit * child_complexity\n    end\n  end\nend\n```","ref":"Absinthe.Schema.Notation.html#complexity/1-examples","title":"Examples - Absinthe.Schema.Notation.complexity/1","type":"macro"},{"doc":"Configure a subscription field.\n\nThe first argument to the config function is the field arguments passed in the subscription.\nThe second argument is an `Absinthe.Resolution` struct, which includes information\nlike the context and other execution data.","ref":"Absinthe.Schema.Notation.html#config/1","title":"Absinthe.Schema.Notation.config/1","type":"macro"},{"doc":"Allowed under: `field`","ref":"Absinthe.Schema.Notation.html#config/1-placement","title":"Placement - Absinthe.Schema.Notation.config/1","type":"macro"},{"doc":"```elixir\nconfig fn args, %{context: context} ->\n  if authorized?(context) do\n    {:ok, topic: args.client_id}\n  else\n    {:error, \"unauthorized\"}\n  end\nend\n```\n\nAlternatively can provide a list of topics:\n\n```elixir\nconfig fn _, _ ->\n  {:ok, topic: [\"topic_one\", \"topic_two\", \"topic_three\"]}\nend\n```\n\nUsing `context_id` option to allow de-duplication of updates:\n\n```elixir\nconfig fn _, %{context: context} ->\n  if authorized?(context) do\n    {:ok, topic: \"topic_one\", context_id: \"authorized\"}\n  else\n    {:ok, topic: \"topic_one\", context_id: \"not-authorized\"}\n  end\nend\n```\n\nSee `Absinthe.Schema.subscription/1` for details","ref":"Absinthe.Schema.Notation.html#config/1-examples","title":"Examples - Absinthe.Schema.Notation.config/1","type":"macro"},{"doc":"Mark a field as deprecated\n\nIn most cases you can simply pass the deprecate: \"message\" attribute. However\nwhen using the block form of a field it can be nice to also use this macro.","ref":"Absinthe.Schema.Notation.html#deprecate/1","title":"Absinthe.Schema.Notation.deprecate/1","type":"macro"},{"doc":"Allowed under: `field`","ref":"Absinthe.Schema.Notation.html#deprecate/1-placement","title":"Placement - Absinthe.Schema.Notation.deprecate/1","type":"macro"},{"doc":"```\nfield :foo, :string do\n  deprecate \"Foo will no longer be supported\"\nend\n```\n\nThis is how to deprecate other things\n```\nfield :foo, :string do\n  arg :bar, :integer, deprecate: \"This isn't supported either\"\nend\n\nenum :colors do\n  value :red\n  value :blue, deprecate: \"This isn't supported\"\nend\n```","ref":"Absinthe.Schema.Notation.html#deprecate/1-examples","title":"Examples - Absinthe.Schema.Notation.deprecate/1","type":"macro"},{"doc":"Defines a description\n\nThis macro adds a description to any other macro which takes a block.\n\nNote that you can also specify a description by using `@desc` above any item\nthat can take a description attribute.","ref":"Absinthe.Schema.Notation.html#description/1","title":"Absinthe.Schema.Notation.description/1","type":"macro"},{"doc":"Allowed under any block. Not allowed to be top level","ref":"Absinthe.Schema.Notation.html#description/1-placement","title":"Placement - Absinthe.Schema.Notation.description/1","type":"macro"},{"doc":"","ref":"Absinthe.Schema.Notation.html#directive/1","title":"Absinthe.Schema.Notation.directive/1","type":"macro"},{"doc":"","ref":"Absinthe.Schema.Notation.html#directive/2","title":"Absinthe.Schema.Notation.directive/2","type":"macro"},{"doc":"Defines or applies a directive","ref":"Absinthe.Schema.Notation.html#directive/3","title":"Absinthe.Schema.Notation.directive/3","type":"macro"},{"doc":"#","ref":"Absinthe.Schema.Notation.html#directive/3-defining-a-directive","title":"Defining a directive - Absinthe.Schema.Notation.directive/3","type":"macro"},{"doc":"Top level in module or in an `extend` block.\n\n\n#","ref":"Absinthe.Schema.Notation.html#directive/3-placement","title":"Placement - Absinthe.Schema.Notation.directive/3","type":"macro"},{"doc":"```elixir\ndirective :mydirective do\n  arg :if, non_null(:boolean), description: \"Skipped when true.\"\n  on [:field, :fragment_spread, :inline_fragment]\n\n  expand fn\n    %{if: true}, node ->\n      Blueprint.put_flag(node, :skip, __MODULE__)\n    _, node ->\n      node\n  end\nend\n```","ref":"Absinthe.Schema.Notation.html#directive/3-examples","title":"Examples - Absinthe.Schema.Notation.directive/3","type":"macro"},{"doc":"Directives can be applied in your schema. E.g. by default the `@deprecated`\ndirective is available to be applied to fields and enum values.\n\nYou can define your own type system directives. See `Absinthe.Schema.Prototype`\nfor more information.\n\n#","ref":"Absinthe.Schema.Notation.html#directive/3-applying-a-type-system-directive","title":"Applying a type system directive - Absinthe.Schema.Notation.directive/3","type":"macro"},{"doc":"Allowed under: `arg` `enum` `field` `input_object` `interface` `object` `scalar` `schema_declaration` `union` `value`\n\n\n#","ref":"Absinthe.Schema.Notation.html#directive/3-placement","title":"Placement - Absinthe.Schema.Notation.directive/3","type":"macro"},{"doc":"When you have a type system directive named `:feature` it can be applied as\nfollows:\n\n```elixir\nobject :post do\n  directive :feature, name: \":object\"\n\n  field :name, :string do\n    deprecate \"Bye\"\n  end\nend\n\nscalar :sweet_scalar do\n  directive :feature, name: \":scalar\"\n  parse &Function.identity/1\n  serialize &Function.identity/1\nend\n```","ref":"Absinthe.Schema.Notation.html#directive/3-examples","title":"Examples - Absinthe.Schema.Notation.directive/3","type":"macro"},{"doc":"Defines an enum type\n\nSee `enum/3`","ref":"Absinthe.Schema.Notation.html#enum/2","title":"Absinthe.Schema.Notation.enum/2","type":"macro"},{"doc":"Defines an enum type","ref":"Absinthe.Schema.Notation.html#enum/3","title":"Absinthe.Schema.Notation.enum/3","type":"macro"},{"doc":"Top level in module or in an `extend` block.","ref":"Absinthe.Schema.Notation.html#enum/3-placement","title":"Placement - Absinthe.Schema.Notation.enum/3","type":"macro"},{"doc":"Handling `RED`, `GREEN`, `BLUE` values from the query document:\n\n```\nenum :color do\n  value :red\n  value :green\n  value :blue\nend\n```\n\nA given query document might look like:\n\n```graphql\n{\n  foo(color: RED)\n}\n```\n\nInternally you would get an argument in elixir that looks like:\n\n```elixir\n%{color: :red}\n```\n\nIf your return value is an enum, it will get serialized out as:\n\n```json\n{\"color\": \"RED\"}\n```\n\nYou can provide custom value mappings. Here we use `r`, `g`, `b` values:\n\n```\nenum :color do\n  value :red, as: \"r\"\n  value :green, as: \"g\"\n  value :blue, as: \"b\"\nend\n```","ref":"Absinthe.Schema.Notation.html#enum/3-examples","title":"Examples - Absinthe.Schema.Notation.enum/3","type":"macro"},{"doc":"Define the expansion for a directive","ref":"Absinthe.Schema.Notation.html#expand/1","title":"Absinthe.Schema.Notation.expand/1","type":"macro"},{"doc":"Allowed under: `directive`","ref":"Absinthe.Schema.Notation.html#expand/1-placement","title":"Placement - Absinthe.Schema.Notation.expand/1","type":"macro"},{"doc":"","ref":"Absinthe.Schema.Notation.html#extend/2","title":"Absinthe.Schema.Notation.extend/2","type":"macro"},{"doc":"Extend a GraphQL type.\n\nExtend an existing type with additional fields, values, types and interfaces.","ref":"Absinthe.Schema.Notation.html#extend/3","title":"Absinthe.Schema.Notation.extend/3","type":"macro"},{"doc":"Top level in module.","ref":"Absinthe.Schema.Notation.html#extend/3-placement","title":"Placement - Absinthe.Schema.Notation.extend/3","type":"macro"},{"doc":"```\nobject :user do\n  field :name, :string\n  # ...\nend\n\nextend object :user do\n    field :nick_name, :string\n    # ...\n  end\nend\n```","ref":"Absinthe.Schema.Notation.html#extend/3-examples","title":"Examples - Absinthe.Schema.Notation.extend/3","type":"macro"},{"doc":"Defines a GraphQL field\n\nSee `field/4`","ref":"Absinthe.Schema.Notation.html#field/2","title":"Absinthe.Schema.Notation.field/2","type":"macro"},{"doc":"Defines a GraphQL field\n\nSee `field/4`","ref":"Absinthe.Schema.Notation.html#field/3","title":"Absinthe.Schema.Notation.field/3","type":"macro"},{"doc":"Defines a GraphQL field.","ref":"Absinthe.Schema.Notation.html#field/4","title":"Absinthe.Schema.Notation.field/4","type":"macro"},{"doc":"Allowed under: `input_object` `interface` `object` `schema_declaration`\n\n\n`query`, `mutation`, and `subscription` are\nall objects under the covers, and thus you'll find `field` definitions under\nthose as well.","ref":"Absinthe.Schema.Notation.html#field/4-placement","title":"Placement - Absinthe.Schema.Notation.field/4","type":"macro"},{"doc":"```\nfield :id, :id\nfield :age, :integer, description: \"How old the item is\"\nfield :name, :string do\n  description \"The name of the item\"\nend\nfield :location, type: :location\n```","ref":"Absinthe.Schema.Notation.html#field/4-examples","title":"Examples - Absinthe.Schema.Notation.field/4","type":"macro"},{"doc":"","ref":"Absinthe.Schema.Notation.html#grab_functions/4","title":"Absinthe.Schema.Notation.grab_functions/4","type":"function"},{"doc":"","ref":"Absinthe.Schema.Notation.html#handle_arg_attrs/3","title":"Absinthe.Schema.Notation.handle_arg_attrs/3","type":"function"},{"doc":"","ref":"Absinthe.Schema.Notation.html#handle_enum_value_attrs/3","title":"Absinthe.Schema.Notation.handle_enum_value_attrs/3","type":"function"},{"doc":"Import directives from another module\n\nTo selectively import directives you can use the `:only` and `:except` opts.","ref":"Absinthe.Schema.Notation.html#import_directives/2","title":"Absinthe.Schema.Notation.import_directives/2","type":"macro"},{"doc":"Top level in module.","ref":"Absinthe.Schema.Notation.html#import_directives/2-placement","title":"Placement - Absinthe.Schema.Notation.import_directives/2","type":"macro"},{"doc":"```\nimport_directives MyApp.Schema.Directives\n\nimport_directives MyApp.Schema.Directives.{DirectivesA, DirectivesB}\n\nimport_directives MyApp.Schema.Directives, only: [:foo]\n\nimport_directives MyApp.Schema.Directives, except: [:bar]\n```","ref":"Absinthe.Schema.Notation.html#import_directives/2-examples","title":"Examples - Absinthe.Schema.Notation.import_directives/2","type":"macro"},{"doc":"Import fields from another object","ref":"Absinthe.Schema.Notation.html#import_fields/2","title":"Absinthe.Schema.Notation.import_fields/2","type":"macro"},{"doc":"```\nobject :news_queries do\n  field :all_links, list_of(:link)\n  field :main_story, :link\nend\n\nobject :admin_queries do\n  field :users, list_of(:user)\n  field :pending_posts, list_of(:post)\nend\n\nquery do\n  import_fields :news_queries\n  import_fields :admin_queries\nend\n```\n\nImport fields can also be used on objects created inside other modules that you\nhave used import_types on.\n\n```\ndefmodule MyApp.Schema.NewsTypes do\n  use Absinthe.Schema.Notation\n\n  object :news_queries do\n    field :all_links, list_of(:link)\n    field :main_story, :link\n  end\nend\ndefmodule MyApp.Schema.Schema do\n  use Absinthe.Schema\n\n  import_types MyApp.Schema.NewsTypes\n\n  query do\n    import_fields :news_queries\n    # ...\n  end\nend\n```","ref":"Absinthe.Schema.Notation.html#import_fields/2-example","title":"Example - Absinthe.Schema.Notation.import_fields/2","type":"macro"},{"doc":"Import types defined using the Schema Definition Language (SDL).\n\nTo add resolvers and middleware to the schema, use the callbacks defined in\n`Absinthe.Schema`, like `c:Absinthe.Schema.hydrate/2`.","ref":"Absinthe.Schema.Notation.html#import_sdl/1","title":"Absinthe.Schema.Notation.import_sdl/1","type":"macro"},{"doc":"Top level in module.","ref":"Absinthe.Schema.Notation.html#import_sdl/1-placement","title":"Placement - Absinthe.Schema.Notation.import_sdl/1","type":"macro"},{"doc":"Directly embedded SDL:\n\n```\nimport_sdl \"\"\"\ntype Query {\n  posts: [Post]\n}\n\ntype Post {\n  title: String!\n  body: String!\n}\n\"\"\"\n```\n\nLoaded from a file location (supporting recompilation on change):\n\n```\nimport_sdl path: \"/path/to/sdl.graphql\"\n```\n\nTODO: Example for dynamic loading during init","ref":"Absinthe.Schema.Notation.html#import_sdl/1-examples","title":"Examples - Absinthe.Schema.Notation.import_sdl/1","type":"macro"},{"doc":"","ref":"Absinthe.Schema.Notation.html#import_sdl/2","title":"Absinthe.Schema.Notation.import_sdl/2","type":"macro"},{"doc":"Import type_extensions from another module\n\nTo selectively import type_extensions you can use the `:only` and `:except` opts.","ref":"Absinthe.Schema.Notation.html#import_type_extensions/2","title":"Absinthe.Schema.Notation.import_type_extensions/2","type":"macro"},{"doc":"Top level in module.","ref":"Absinthe.Schema.Notation.html#import_type_extensions/2-placement","title":"Placement - Absinthe.Schema.Notation.import_type_extensions/2","type":"macro"},{"doc":"```\nimport_type_extensions MyApp.Schema.TypeExtensions\n\nimport_type_extensions MyApp.Schema.TypeExtensions.{TypeExtensionsA, TypeExtensionsB}\n\nimport_type_extensions MyApp.Schema.TypeExtensions, only: [:foo]\n\nimport_type_extensions MyApp.Schema.TypeExtensions, except: [:bar]\n```","ref":"Absinthe.Schema.Notation.html#import_type_extensions/2-examples","title":"Examples - Absinthe.Schema.Notation.import_type_extensions/2","type":"macro"},{"doc":"Import types from another module\n\nVery frequently your schema module will simply have the `query` and `mutation`\nblocks, and you'll want to break out your other types into other modules. This\nmacro imports those types for use the current module.\n\nTo selectively import types you can use the `:only` and `:except` opts.","ref":"Absinthe.Schema.Notation.html#import_types/2","title":"Absinthe.Schema.Notation.import_types/2","type":"macro"},{"doc":"Top level in module.","ref":"Absinthe.Schema.Notation.html#import_types/2-placement","title":"Placement - Absinthe.Schema.Notation.import_types/2","type":"macro"},{"doc":"```\nimport_types MyApp.Schema.Types\n\nimport_types MyApp.Schema.Types.{TypesA, TypesB}\n\nimport_types MyApp.Schema.Types, only: [:foo]\n\nimport_types MyApp.Schema.Types, except: [:bar]\n```","ref":"Absinthe.Schema.Notation.html#import_types/2-examples","title":"Examples - Absinthe.Schema.Notation.import_types/2","type":"macro"},{"doc":"Defines an input object\n\nSee `Absinthe.Type.InputObject`","ref":"Absinthe.Schema.Notation.html#input_object/3","title":"Absinthe.Schema.Notation.input_object/3","type":"macro"},{"doc":"Top level in module or in an `extend` block.","ref":"Absinthe.Schema.Notation.html#input_object/3-placement","title":"Placement - Absinthe.Schema.Notation.input_object/3","type":"macro"},{"doc":"```\ninput_object :contact_input do\n  field :email, non_null(:string)\nend\n```","ref":"Absinthe.Schema.Notation.html#input_object/3-examples","title":"Examples - Absinthe.Schema.Notation.input_object/3","type":"macro"},{"doc":"Declare an implemented interface for an object.\n\nAdds an `Absinthe.Type.Interface` to your schema.\n\nSee also `interfaces/1`, which can be used for multiple interfaces,\nand `interface/3`, used to define interfaces themselves.","ref":"Absinthe.Schema.Notation.html#interface/1","title":"Absinthe.Schema.Notation.interface/1","type":"macro"},{"doc":"Allowed under: `interface` `object`","ref":"Absinthe.Schema.Notation.html#interface/1-placement","title":"Placement - Absinthe.Schema.Notation.interface/1","type":"macro"},{"doc":"```\nobject :car do\n  interface :vehicle\n  # ...\nend\n```","ref":"Absinthe.Schema.Notation.html#interface/1-examples","title":"Examples - Absinthe.Schema.Notation.interface/1","type":"macro"},{"doc":"Define an interface type.\n\nAdds an `Absinthe.Type.Interface` to your schema.\n\nAlso see `interface/1` and `interfaces/1`, which declare\nthat an object implements one or more interfaces.","ref":"Absinthe.Schema.Notation.html#interface/3","title":"Absinthe.Schema.Notation.interface/3","type":"macro"},{"doc":"Top level in module or in an `extend` block.","ref":"Absinthe.Schema.Notation.html#interface/3-placement","title":"Placement - Absinthe.Schema.Notation.interface/3","type":"macro"},{"doc":"```\ninterface :vehicle do\n  field :wheel_count, :integer\nend\n\nobject :rally_car do\n  field :wheel_count, :integer\n  interface :vehicle\nend\n```","ref":"Absinthe.Schema.Notation.html#interface/3-examples","title":"Examples - Absinthe.Schema.Notation.interface/3","type":"macro"},{"doc":"Declare implemented interfaces for an object.\n\nSee also `interface/1`, which can be used for one interface,\nand `interface/3`, used to define interfaces themselves.","ref":"Absinthe.Schema.Notation.html#interfaces/1","title":"Absinthe.Schema.Notation.interfaces/1","type":"macro"},{"doc":"Allowed under: `interface` `object`","ref":"Absinthe.Schema.Notation.html#interfaces/1-placement","title":"Placement - Absinthe.Schema.Notation.interfaces/1","type":"macro"},{"doc":"```\nobject :car do\n  interfaces [:vehicle, :branded]\n  # ...\nend\n```","ref":"Absinthe.Schema.Notation.html#interfaces/1-examples","title":"Examples - Absinthe.Schema.Notation.interfaces/1","type":"macro"},{"doc":"","ref":"Absinthe.Schema.Notation.html#is_type_of/1","title":"Absinthe.Schema.Notation.is_type_of/1","type":"macro"},{"doc":"Allowed under: `object`","ref":"Absinthe.Schema.Notation.html#is_type_of/1-placement","title":"Placement - Absinthe.Schema.Notation.is_type_of/1","type":"macro"},{"doc":"","ref":"Absinthe.Schema.Notation.html#lift_functions/2","title":"Absinthe.Schema.Notation.lift_functions/2","type":"function"},{"doc":"","ref":"Absinthe.Schema.Notation.html#lift_functions/3","title":"Absinthe.Schema.Notation.lift_functions/3","type":"function"},{"doc":"Marks a type reference as a list of the given type\n\nSee `field/3` for examples","ref":"Absinthe.Schema.Notation.html#list_of/1","title":"Absinthe.Schema.Notation.list_of/1","type":"macro"},{"doc":"Defines list of metadata's key/value pair for a custom type.\n\nThis is generally used to facilitate libraries that want to augment Absinthe\nfunctionality","ref":"Absinthe.Schema.Notation.html#meta/1","title":"Absinthe.Schema.Notation.meta/1","type":"macro"},{"doc":"```\nobject :user do\n  meta cache: true, ttl: 22_000\nend\n\nobject :user, meta: [cache: true, ttl: 22_000] do\n  # ...\nend\n```\n\nThe meta can be accessed via the `Absinthe.Type.meta/2` function.\n\n```\nuser_type = Absinthe.Schema.lookup_type(MyApp.Schema, :user)\n\nAbsinthe.Type.meta(user_type, :cache)\n#=> true\n\nAbsinthe.Type.meta(user_type)\n#=> [cache: true, ttl: 22_000]\n```","ref":"Absinthe.Schema.Notation.html#meta/1-examples","title":"Examples - Absinthe.Schema.Notation.meta/1","type":"macro"},{"doc":"Allowed under: `directive` `enum` `extend` `field` `input_object` `interface` `object` `scalar` `union`","ref":"Absinthe.Schema.Notation.html#meta/1-placement","title":"Placement - Absinthe.Schema.Notation.meta/1","type":"macro"},{"doc":"Defines a metadata key/value pair for a custom type.\n\nFor more info see `meta/1`\n\n#","ref":"Absinthe.Schema.Notation.html#meta/2","title":"Absinthe.Schema.Notation.meta/2","type":"macro"},{"doc":"```\nmeta :cache, false\n```","ref":"Absinthe.Schema.Notation.html#meta/2-examples","title":"Examples - Absinthe.Schema.Notation.meta/2","type":"macro"},{"doc":"Allowed under: `directive` `enum` `extend` `field` `input_object` `interface` `object` `scalar` `union`","ref":"Absinthe.Schema.Notation.html#meta/2-placement","title":"Placement - Absinthe.Schema.Notation.meta/2","type":"macro"},{"doc":"","ref":"Absinthe.Schema.Notation.html#middleware/2","title":"Absinthe.Schema.Notation.middleware/2","type":"macro"},{"doc":"Marks a type reference as non null\n\nSee `field/3` for examples","ref":"Absinthe.Schema.Notation.html#non_null/1","title":"Absinthe.Schema.Notation.non_null/1","type":"macro"},{"doc":"","ref":"Absinthe.Schema.Notation.html#noop/1","title":"Absinthe.Schema.Notation.noop/1","type":"function"},{"doc":"Define an object type.\n\nAdds an `Absinthe.Type.Object` to your schema.","ref":"Absinthe.Schema.Notation.html#object/3","title":"Absinthe.Schema.Notation.object/3","type":"macro"},{"doc":"Top level in module or in an `extend` block.","ref":"Absinthe.Schema.Notation.html#object/3-placement","title":"Placement - Absinthe.Schema.Notation.object/3","type":"macro"},{"doc":"Basic definition:\n\n```\nobject :car do\n  # ...\nend\n```\n\nProviding a custom name:\n\n```\nobject :car, name: \"CarType\" do\n  # ...\nend\n```","ref":"Absinthe.Schema.Notation.html#object/3-examples","title":"Examples - Absinthe.Schema.Notation.object/3","type":"macro"},{"doc":"Declare a directive as operating an a AST node type\n\nSee `directive/2`","ref":"Absinthe.Schema.Notation.html#on/1","title":"Absinthe.Schema.Notation.on/1","type":"macro"},{"doc":"Allowed under: `directive`","ref":"Absinthe.Schema.Notation.html#on/1-placement","title":"Placement - Absinthe.Schema.Notation.on/1","type":"macro"},{"doc":"Defines a parse function for a `scalar` type\n\nThe specified `parse` function is used on incoming data to transform it into\nan elixir datastructure.\n\nIt should return `{:ok, value}` or `:error`","ref":"Absinthe.Schema.Notation.html#parse/1","title":"Absinthe.Schema.Notation.parse/1","type":"macro"},{"doc":"Allowed under: `scalar`","ref":"Absinthe.Schema.Notation.html#parse/1-placement","title":"Placement - Absinthe.Schema.Notation.parse/1","type":"macro"},{"doc":"","ref":"Absinthe.Schema.Notation.html#put_attr/2","title":"Absinthe.Schema.Notation.put_attr/2","type":"function"},{"doc":"","ref":"Absinthe.Schema.Notation.html#put_desc/2","title":"Absinthe.Schema.Notation.put_desc/2","type":"function"},{"doc":"","ref":"Absinthe.Schema.Notation.html#put_reference/2","title":"Absinthe.Schema.Notation.put_reference/2","type":"function"},{"doc":"","ref":"Absinthe.Schema.Notation.html#record!/5","title":"Absinthe.Schema.Notation.record!/5","type":"function"},{"doc":"","ref":"Absinthe.Schema.Notation.html#record_applied_directive!/3","title":"Absinthe.Schema.Notation.record_applied_directive!/3","type":"function"},{"doc":"","ref":"Absinthe.Schema.Notation.html#record_config!/2","title":"Absinthe.Schema.Notation.record_config!/2","type":"function"},{"doc":"","ref":"Absinthe.Schema.Notation.html#record_extend!/4","title":"Absinthe.Schema.Notation.record_extend!/4","type":"function"},{"doc":"","ref":"Absinthe.Schema.Notation.html#record_middleware!/3","title":"Absinthe.Schema.Notation.record_middleware!/3","type":"function"},{"doc":"","ref":"Absinthe.Schema.Notation.html#record_schema!/2","title":"Absinthe.Schema.Notation.record_schema!/2","type":"function"},{"doc":"","ref":"Absinthe.Schema.Notation.html#record_trigger!/3","title":"Absinthe.Schema.Notation.record_trigger!/3","type":"function"},{"doc":"Set whether the directive can be applied multiple times\nan entity.\n\nIf omitted, defaults to `false`","ref":"Absinthe.Schema.Notation.html#repeatable/1","title":"Absinthe.Schema.Notation.repeatable/1","type":"macro"},{"doc":"Allowed under: `directive`","ref":"Absinthe.Schema.Notation.html#repeatable/1-placement","title":"Placement - Absinthe.Schema.Notation.repeatable/1","type":"macro"},{"doc":"Defines a resolve function for a field\n\nSpecify a 2 or 3 arity function to call when resolving a field.\n\nYou can either hard code a particular anonymous function, or have a function\ncall that returns a 2 or 3 arity anonymous function. See examples for more information.\n\nNote that when using a hard coded anonymous function, the function will not\ncapture local variables.\n\n#","ref":"Absinthe.Schema.Notation.html#resolve/1","title":"Absinthe.Schema.Notation.resolve/1","type":"macro"},{"doc":"The first argument to the function is the parent entity.\n```\n{\n  user(id: 1) {\n    name\n  }\n}\n```\nA resolution function on the `name` field would have the result of the `user(id: 1)` field\nas its first argument. Top level fields have the `root_value` as their first argument.\nUnless otherwise specified, this defaults to an empty map.\n\nThe second argument to the resolution function is the field arguments. The final\nargument is an `Absinthe.Resolution` struct, which includes information like\nthe `context` and other execution data.\n\n#","ref":"Absinthe.Schema.Notation.html#resolve/1-3-arity-functions","title":"3 Arity Functions - Absinthe.Schema.Notation.resolve/1","type":"macro"},{"doc":"Exactly the same as the 3 arity version, but without the first argument (the parent entity)","ref":"Absinthe.Schema.Notation.html#resolve/1-2-arity-function","title":"2 Arity Function - Absinthe.Schema.Notation.resolve/1","type":"macro"},{"doc":"Allowed under: `field`","ref":"Absinthe.Schema.Notation.html#resolve/1-placement","title":"Placement - Absinthe.Schema.Notation.resolve/1","type":"macro"},{"doc":"```\nquery do\n  field :person, :person do\n    arg :id, non_null(:id)\n    resolve &Person.resolve/2\n  end\nend\n```\n\n```\nquery do\n  field :person, :person do\n    arg :id, non_null(:id)\n    resolve fn %{id: id}, _ ->\n      {:ok, Person.find(id)}\n    end\n  end\nend\n```\n\n```\nquery do\n  field :person, :person do\n    arg :id, non_null(:id)\n    resolve lookup(:person)\n  end\nend\n\ndef lookup(:person) do\n  fn %{id: id}, _ ->\n    {:ok, Person.find(id)}\n  end\nend\n```","ref":"Absinthe.Schema.Notation.html#resolve/1-examples","title":"Examples - Absinthe.Schema.Notation.resolve/1","type":"macro"},{"doc":"Define a type resolver for a union or interface.\n\nSee also:\n* `Absinthe.Type.Interface`\n* `Absinthe.Type.Union`","ref":"Absinthe.Schema.Notation.html#resolve_type/1","title":"Absinthe.Schema.Notation.resolve_type/1","type":"macro"},{"doc":"Allowed under: `interface` `union`","ref":"Absinthe.Schema.Notation.html#resolve_type/1-placement","title":"Placement - Absinthe.Schema.Notation.resolve_type/1","type":"macro"},{"doc":"```\ninterface :entity do\n  # ...\n  resolve_type fn\n    %{employee_count: _},  _ ->\n      :business\n    %{age: _}, _ ->\n      :person\n  end\nend\n```","ref":"Absinthe.Schema.Notation.html#resolve_type/1-examples","title":"Examples - Absinthe.Schema.Notation.resolve_type/1","type":"macro"},{"doc":"Defines a scalar type\n\nSee `scalar/3`","ref":"Absinthe.Schema.Notation.html#scalar/2","title":"Absinthe.Schema.Notation.scalar/2","type":"macro"},{"doc":"Define a scalar type\n\nA scalar type requires `parse/1` and `serialize/1` functions.","ref":"Absinthe.Schema.Notation.html#scalar/3","title":"Absinthe.Schema.Notation.scalar/3","type":"macro"},{"doc":"Top level in module or in an `extend` block.","ref":"Absinthe.Schema.Notation.html#scalar/3-placement","title":"Placement - Absinthe.Schema.Notation.scalar/3","type":"macro"},{"doc":"```\nscalar :isoz_datetime, description: \"UTC only ISO8601 date time\" do\n  parse &Timex.parse(&1, \"{ISO:Extended:Z}\")\n  serialize &Timex.format!(&1, \"{ISO:Extended:Z}\")\nend\n```","ref":"Absinthe.Schema.Notation.html#scalar/3-examples","title":"Examples - Absinthe.Schema.Notation.scalar/3","type":"macro"},{"doc":"Declare a schema\n\nOptional declaration of the schema. Useful if you want to add directives\nto your schema declaration","ref":"Absinthe.Schema.Notation.html#schema/1","title":"Absinthe.Schema.Notation.schema/1","type":"macro"},{"doc":"Top level in module or in an `extend` block.","ref":"Absinthe.Schema.Notation.html#schema/1-placement","title":"Placement - Absinthe.Schema.Notation.schema/1","type":"macro"},{"doc":"```\nschema do\n  directive :feature\n  field :query, :query\n  # ...\nend\n```","ref":"Absinthe.Schema.Notation.html#schema/1-examples","title":"Examples - Absinthe.Schema.Notation.schema/1","type":"macro"},{"doc":"Defines a serialization function for a `scalar` type\n\nThe specified `serialize` function is used on outgoing data. It should simply\nreturn the desired external representation.","ref":"Absinthe.Schema.Notation.html#serialize/1","title":"Absinthe.Schema.Notation.serialize/1","type":"macro"},{"doc":"Allowed under: `scalar`","ref":"Absinthe.Schema.Notation.html#serialize/1-placement","title":"Placement - Absinthe.Schema.Notation.serialize/1","type":"macro"},{"doc":"Sets triggers for a subscription, and configures which topics to publish to when that subscription\nis triggered.\n\nA trigger is the name of a mutation. When that mutation runs, data is pushed to the clients\nwho are subscribed to the subscription.\n\nA subscription can have many triggers, and a trigger can push to many topics.","ref":"Absinthe.Schema.Notation.html#trigger/2","title":"Absinthe.Schema.Notation.trigger/2","type":"macro"},{"doc":"Allowed under: `field`","ref":"Absinthe.Schema.Notation.html#trigger/2-placement","title":"Placement - Absinthe.Schema.Notation.trigger/2","type":"macro"},{"doc":"```elixir\nmutation do\n  field :gps_event, :gps_event\n  field :user_checkin, :user\nend\n\nsubscription do\n  field :location_update, :user do\n    arg :user_id, non_null(:id)\n\n    config fn args, _ ->\n      {:ok, topic: args.user_id}\n    end\n\n    trigger :gps_event, topic: fn gps_event ->\n      gps_event.user_id\n    end\n\n    # Trigger on a list of mutations\n    trigger [:user_checkin], topic: fn user ->\n      # Returning a list of topics triggers the subscription for each of the topics in the list.\n      [user.id, user.friend.id]\n    end\n  end\nend\n```\n\nTrigger functions are only called once per event, so database calls within\nthem do not present a significant burden.\n\nSee the `Absinthe.Schema.subscription/2` macro docs for additional details","ref":"Absinthe.Schema.Notation.html#trigger/2-example","title":"Example - Absinthe.Schema.Notation.trigger/2","type":"macro"},{"doc":"Defines the types possible under a union type\n\nSee `union/3`","ref":"Absinthe.Schema.Notation.html#types/1","title":"Absinthe.Schema.Notation.types/1","type":"macro"},{"doc":"Allowed under: `union`","ref":"Absinthe.Schema.Notation.html#types/1-placement","title":"Placement - Absinthe.Schema.Notation.types/1","type":"macro"},{"doc":"Defines a union type\n\nSee `Absinthe.Type.Union`","ref":"Absinthe.Schema.Notation.html#union/3","title":"Absinthe.Schema.Notation.union/3","type":"macro"},{"doc":"Top level in module or in an `extend` block.","ref":"Absinthe.Schema.Notation.html#union/3-placement","title":"Placement - Absinthe.Schema.Notation.union/3","type":"macro"},{"doc":"```\nunion :search_result do\n  description \"A search result\"\n\n  types [:person, :business]\n  resolve_type fn\n    %Person{}, _ -> :person\n    %Business{}, _ -> :business\n  end\nend\n```","ref":"Absinthe.Schema.Notation.html#union/3-examples","title":"Examples - Absinthe.Schema.Notation.union/3","type":"macro"},{"doc":"Defines a value possible under an enum type\n\nSee `enum/3`","ref":"Absinthe.Schema.Notation.html#value/2","title":"Absinthe.Schema.Notation.value/2","type":"macro"},{"doc":"Allowed under: `enum`","ref":"Absinthe.Schema.Notation.html#value/2-placement","title":"Placement - Absinthe.Schema.Notation.value/2","type":"macro"},{"doc":"","ref":"Absinthe.Schema.Notation.html#values/1","title":"Absinthe.Schema.Notation.values/1","type":"macro"},{"doc":"","ref":"Absinthe.Schema.Notation.html#t:import_sdl_option/0","title":"Absinthe.Schema.Notation.import_sdl_option/0","type":"type"},{"doc":"Provides the directives available for SDL schema definitions.\n\nBy default, the only directive provided is `@deprecated`, which supports\na `reason` argument (of GraphQL type `String`). This can be used to\nmark a field\n\nTo add additional schema directives, define your own prototype schema, e.g.:\n\n```\ndefmodule MyAppWeb.SchemaPrototype do\n  use Absinthe.Schema.Prototype\n\n  directive :feature do\n    arg :name, non_null(:string)\n    on [:interface]\n    # Define `expand`, etc.\n  end\n\n  # More directives...\nend\n```\n\nThen, set it as the prototype for your schema:\n\n```\ndefmodule MyAppWeb.Schema do\n  use Absinthe.Schema\n\n  @prototype_schema MyAppWeb.SchemaPrototype\n\n  # Use `import_sdl`, etc...\nend\n```","ref":"Absinthe.Schema.Prototype.html","title":"Absinthe.Schema.Prototype","type":"module"},{"doc":"Add a deprecation (with an optional reason) to a node.","ref":"Absinthe.Schema.Prototype.html#expand_deprecate/2","title":"Absinthe.Schema.Prototype.expand_deprecate/2","type":"function"},{"doc":"","ref":"Absinthe.Schema.Prototype.html#pipeline/1","title":"Absinthe.Schema.Prototype.pipeline/1","type":"function"},{"doc":"Used to define an argument.\n\nUsually these are defined using `Absinthe.Schema.Notation.arg/2`","ref":"Absinthe.Type.Argument.html","title":"Absinthe.Type.Argument","type":"module"},{"doc":"Argument configuration\n\n* `:name` - The name of the argument, usually assigned automatically using `Absinthe.Schema.Notation.arg/2`.\n* `:type` - The type values the argument accepts/will coerce to.\n* `:deprecation` - Deprecation information for an argument, usually\n  set-up using `Absinthe.Schema.Notation.deprecate/1`.\n* `:description` - Description of an argument, useful for introspection.\n\nThe `__private__` and `__reference__` fields are for internal use.","ref":"Absinthe.Type.Argument.html#t:t/0","title":"Absinthe.Type.Argument.t/0","type":"type"},{"doc":"This module contains the following additional data types:\n- datetime (UTC)\n- naive_datetime\n- date\n- time\n- decimal (only if [Decimal](https://hex.pm/packages/decimal) is available)\n\nFurther description of these types can be found in the source code.\n\nTo use: `import_types Absinthe.Type.Custom`.","ref":"Absinthe.Type.Custom.html","title":"Absinthe.Type.Custom","type":"module"},{"doc":"Used by the GraphQL runtime as a way of modifying execution\nbehavior.\n\nType system creators will usually not create these directly.","ref":"Absinthe.Type.Directive.html","title":"Absinthe.Type.Directive","type":"module"},{"doc":"","ref":"Absinthe.Type.Directive.html#t:location/0","title":"Absinthe.Type.Directive.location/0","type":"type"},{"doc":"A defined directive.\n\n* `:name` - The name of the directive. Should be a lowercase `binary`. Set automatically.\n* `:description` - A nice description for introspection.\n* `:args` - A map of `Absinthe.Type.Argument` structs. See `Absinthe.Schema.Notation.arg/2`.\n* `:locations` - A list of places the directives can be used.\n* `:repeatable` - A directive may be defined as repeatable by including the “repeatable” keyword\n\nThe `:__reference__` key is for internal use.","ref":"Absinthe.Type.Directive.html#t:t/0","title":"Absinthe.Type.Directive.t/0","type":"type"},{"doc":"Used to define an enum type, a special scalar that can only have a defined set\nof values.\n\nSee the `t` type below for details and examples.","ref":"Absinthe.Type.Enum.html","title":"Absinthe.Type.Enum","type":"module"},{"doc":"Given a type defined as the following (see `Absinthe.Schema.Notation`):\n\n```\n@desc \"The selected color channel\"\nenum :color_channel do\n  value :red, as: :r, description: \"Color Red\"\n  value :green, as: :g, description: \"Color Green\"\n  value :blue, as: :b, description: \"Color Blue\"\n  value :alpha, as: :a, deprecate: \"We no longer support opacity settings\", description: \"Alpha Channel\"\nend\n```\n\nThe \"ColorChannel\" type (referred inside Absinthe as `:color_channel`) is an\nEnum type, with values with names \"red\", \"green\", \"blue\", and \"alpha\" that map\nto internal, raw values `:r`, `:g`, `:b`, and `:a`. The alpha color channel\nis deprecated, just as fields and arguments can be.\n\nYou can omit the raw `value` if you'd like it to be the same as the\nidentifier. For instance, in this example the `value` is automatically set to\n`:red`:\n\n```\nenum :color_channel do\n  description \"The selected color channel\"\n\n  value :red, description: \"Color Red\"\n  value :green, description: \"Color Green\"\n  value :blue, description: \"Color Blue\"\n  value :alpha, deprecate: \"We no longer support opacity settings\", description: \"Alpha Channel\"\nend\n```\n\nIf you really want to use a shorthand, skipping support for descriptions,\ncustom raw values, and deprecation, you can just provide a list of atoms:\n\n```\nenum :color_channel, values: [:red, :green, :blue, :alpha]\n```\n\nKeep in mind that writing a terse definition that skips descriptions and\ndeprecations today may hamper tooling that relies on introspection tomorrow.","ref":"Absinthe.Type.Enum.html#module-examples","title":"Examples - Absinthe.Type.Enum","type":"module"},{"doc":"","ref":"Absinthe.Type.Enum.html#kind/0","title":"Absinthe.Type.Enum.kind/0","type":"function"},{"doc":"A defined enum type.\n\nShould be defined using `Absinthe.Schema.Notation.enum/2`.\n\n* `:name` - The name of the enum type. Should be a TitleCased `binary`. Set automatically.\n* `:description` - A nice description for introspection.\n* `:values` - The enum values, usually provided using the `Absinthe.Schema.Notation.values/1` or `Absinthe.Schema.Notation.value/1` macro.\n\n\nThe `__private__` and `:__reference__` fields are for internal use.","ref":"Absinthe.Type.Enum.html#t:t/0","title":"Absinthe.Type.Enum.t/0","type":"type"},{"doc":"A possible value for an enum.\n\nSee `Absinthe.Type.Enum` and `Absinthe.Schema.Notation.value/1`.","ref":"Absinthe.Type.Enum.Value.html","title":"Absinthe.Type.Enum.Value","type":"module"},{"doc":"A defined enum value entry.\n\nGenerally defined using `Absinthe.Schema.Notation.value/2` as\npart of a schema.\n\n* `:name` - The name of the value. This is also the incoming, external\n  value that will be provided by query documents.\n* `:description` - A nice description for introspection.\n* `:value` - The raw, internal value that `:name` map to. This will be\n  provided as the argument value to `resolve` functions.\n* `:deprecation` - Deprecation information for a value, usually\n  set-up using the `Absinthe.Schema.Notation.deprecate/1` convenience\n  function.\n\nThe `__private__` and `__reference__` fields are for internal use.","ref":"Absinthe.Type.Enum.Value.html#t:t/0","title":"Absinthe.Type.Enum.Value.t/0","type":"type"},{"doc":"Used to define a field.\n\nUsually these are defined using `Absinthe.Schema.Notation.field/4`\n\nSee the `t` type below for details and examples of how to define a field.","ref":"Absinthe.Type.Field.html","title":"Absinthe.Type.Field","type":"module"},{"doc":"A complexity function.\n\nSee the `Absinthe.Type.Field/t` explanation of `:complexity` for more\ninformation.","ref":"Absinthe.Type.Field.html#t:complexity_t/0","title":"Absinthe.Type.Field.complexity_t/0","type":"type"},{"doc":"A custom error may be a `map` or a `Keyword.t`, but must contain a `:message` key.\n\nNote that the values that make up a custom error must be serializable.","ref":"Absinthe.Type.Field.html#t:custom_error/0","title":"Absinthe.Type.Field.custom_error/0","type":"type"},{"doc":"An error message is a human-readable string describing the error that occurred.","ref":"Absinthe.Type.Field.html#t:error_message/0","title":"Absinthe.Type.Field.error_message/0","type":"type"},{"doc":"","ref":"Absinthe.Type.Field.html#t:error_result/0","title":"Absinthe.Type.Field.error_result/0","type":"type"},{"doc":"An error value is a simple error message, a custom error, or a list of either/both of them.","ref":"Absinthe.Type.Field.html#t:error_value/0","title":"Absinthe.Type.Field.error_value/0","type":"type"},{"doc":"","ref":"Absinthe.Type.Field.html#t:middleware_result/0","title":"Absinthe.Type.Field.middleware_result/0","type":"type"},{"doc":"","ref":"Absinthe.Type.Field.html#t:ok_result/0","title":"Absinthe.Type.Field.ok_result/0","type":"type"},{"doc":"A resolver function.\n\nSee the `Absinthe.Type.Field.t` explanation of `:resolve` for more information.","ref":"Absinthe.Type.Field.html#t:resolver_t/0","title":"Absinthe.Type.Field.resolver_t/0","type":"type"},{"doc":"The result of a resolver.","ref":"Absinthe.Type.Field.html#t:result/0","title":"Absinthe.Type.Field.result/0","type":"type"},{"doc":"Any serializable value.","ref":"Absinthe.Type.Field.html#t:serializable/0","title":"Absinthe.Type.Field.serializable/0","type":"type"},{"doc":"The configuration for a field.\n\n* `:name` - The name of the field, usually assigned automatically by\n   the `Absinthe.Schema.Notation.field/4`. Including this option will bypass the snake_case to camelCase conversion.\n* `:description` - Description of a field, useful for introspection.\n* `:deprecation` - Deprecation information for a field, usually\n   set-up using `Absinthe.Schema.Notation.deprecate/1`.\n* `:type` - The type the value of the field should resolve to\n* `:args` - The arguments of the field, usually created by using `Absinthe.Schema.Notation.arg/2`.\n* `:resolve` - The resolution function. See below for more information.\n* `:complexity` - The complexity function. See below for more information.\n* `:default_value` - The default value of a field. Note this is not used during resolution; only fields that are part of an `Absinthe.Type.InputObject` should set this value.","ref":"Absinthe.Type.Field.html#t:t/0","title":"Absinthe.Type.Field.t/0","type":"type"},{"doc":"#","ref":"Absinthe.Type.Field.html#t:t/0-resolution-functions","title":"Resolution Functions - Absinthe.Type.Field.t/0","type":"type"},{"doc":"If no resolution function is given, the default resolution function is used,\nwhich is roughly equivalent to this:\n\n    {:ok, Map.get(parent_object, field_name)}\n\nThis is commonly use when listing the available fields on a\n`Absinthe.Type.Object` that models a data record. For instance:\n\n```\nobject :person do\n  description \"A person\"\n\n  field :first_name, :string\n  field :last_name, :string\nend\n```\n#","ref":"Absinthe.Type.Field.html#t:t/0-default","title":"Default - Absinthe.Type.Field.t/0","type":"type"},{"doc":"When accepting arguments, however, you probably need to use them for\nsomething. Here's an example of defining a field that looks up a list of\nusers for a given `location_id`:\n```\nquery do\n  field :users, :person do\n    arg :location_id, non_null(:id)\n\n    resolve fn %{location_id: id}, _ ->\n      {:ok, MyApp.users_for_location_id(id)}\n    end\n  end\nend\n```\n\nCustom resolution functions are passed two arguments:\n\n1. A map of the arguments for the field, filled in with values from the\n   provided query document/variables.\n2. An `Absinthe.Resolution` struct, containing the execution environment\n   for the field (and useful for complex resolutions using the resolved source\n   object, etc)","ref":"Absinthe.Type.Field.html#t:t/0-custom-resolution","title":"Custom Resolution - Absinthe.Type.Field.t/0","type":"type"},{"doc":"#","ref":"Absinthe.Type.Field.html#t:t/0-complexity-function","title":"Complexity function - Absinthe.Type.Field.t/0","type":"type"},{"doc":"If no complexity function is given, the default complexity function is used,\nwhich is equivalent to:\n\n    fn(_, child_complexity) -> 1 + child_complexity end\n\n#","ref":"Absinthe.Type.Field.html#t:t/0-default","title":"Default - Absinthe.Type.Field.t/0","type":"type"},{"doc":"When accepting arguments, however, you probably need to use them for\nsomething. Here's an example of defining a field that looks up at most\n`limit` users:\n```\nquery do\n  field :users, :person do\n    arg :limit, :integer\n\n    complexity fn %{limit: limit}, child_complexity ->\n      10 + limit * child_complexity\n    end\n  end\nend\n```\n\nAn optional third argument, `Absinthe.Complexity` struct, provides extra\ninformation. Here's an example of changing the complexity using the context:\n```\nquery do\n  field :users, :person do\n    arg :limit, :integer\n\n    complexity fn _, child_complexity, %{context: %{admin: admin?}} ->\n      if admin?, do: 0, else: 10 + limit * child_complexity\n    end\n  end\nend\n```\n\nCustom complexity functions are passed two or three arguments:\n\n1. A map of the arguments for the field, filled in with values from the\n   provided query document/variables.\n2. A non negative integer, which is total complexity of the child fields.\n3. An `Absinthe.Complexity` struct with information about the context of the\n   field. This argument is optional when using an anonymous function.\n\nAlternatively complexity can be an integer greater than or equal to 0:\n```\nquery do\n  field :users, :person do\n    complexity 10\n  end\nend\n```","ref":"Absinthe.Type.Field.html#t:t/0-custom-complexity","title":"Custom Complexity - Absinthe.Type.Field.t/0","type":"type"},{"doc":"Defines a GraphQL input object\n\nInput objects enable nested arguments to queries and mutations.","ref":"Absinthe.Type.InputObject.html","title":"Absinthe.Type.InputObject","type":"module"},{"doc":"```\nmutation do\n  field :user, :user do\n    arg :name, :string\n    arg :contact, non_null(:contact)\n\n    resolve &User.create/2\n  end\nend\n\ninput_object :contact do\n  field :email, :string\nend\n```\n\nThis supports the following `mutation`:\n```graphql\nmutation CreateUser {\n  user(contact: {email: \"foo@bar.com\"}) {\n    id\n  }\n}\n```","ref":"Absinthe.Type.InputObject.html#module-example","title":"Example - Absinthe.Type.InputObject","type":"module"},{"doc":"","ref":"Absinthe.Type.InputObject.html#kind/0","title":"Absinthe.Type.InputObject.kind/0","type":"function"},{"doc":"Note new input object types should be defined using\n`Absinthe.Schema.Notation.input_object/3`.\n\n* `:name` - The name of the input object type. Should be a TitleCased `binary`. Set automatically.\n* `:description` - A nice description for introspection.\n* `:fields` - A map of `Absinthe.Type.Field` structs. Usually built via `Absinthe.Schema.Notation.field/4`.\n\nThe `__private__` and `:__reference__` fields are for internal use.","ref":"Absinthe.Type.InputObject.html#t:t/0","title":"Absinthe.Type.InputObject.t/0","type":"type"},{"doc":"A defined interface type that represent a list of named fields and their\narguments.\n\nFields on an interface have the same rules as fields on an\n`Absinthe.Type.Object`.\n\nIf an `Absinthe.Type.Object` lists an interface in its `:interfaces` entry,\nit guarantees that it defines the same fields and arguments that the\ninterface does.\n\nBecause sometimes it's for the interface to determine the implementing type of\na resolved object, you must either:\n\n* Provide a `:resolve_type` function on the interface\n* Provide a `:is_type_of` function on each implementing type\n\n```\ninterface :named_entity do\n  field :name, :string\n  resolve_type fn\n    %{age: _}, _ -> :person\n    %{employee_count: _}, _ -> :business\n    _, _ -> nil\n  end\nend\n\nobject :person do\n  field :name, :string\n  field :age, :string\n\n  interface :named_entity\nend\n\nobject :business do\n  field :name, :string\n  field :employee_count, :integer\n\n  interface :named_entity\nend\n```","ref":"Absinthe.Type.Interface.html","title":"Absinthe.Type.Interface","type":"module"},{"doc":"","ref":"Absinthe.Type.Interface.html#kind/0","title":"Absinthe.Type.Interface.kind/0","type":"function"},{"doc":"","ref":"Absinthe.Type.Interface.html#resolve_type/4","title":"Absinthe.Type.Interface.resolve_type/4","type":"function"},{"doc":"Whether the interface (or implementors) are correctly configured to resolve\nobjects.","ref":"Absinthe.Type.Interface.html#type_resolvable?/2","title":"Absinthe.Type.Interface.type_resolvable?/2","type":"function"},{"doc":"* `:name` - The name of the interface type. Should be a TitleCased `binary`. Set automatically.\n* `:description` - A nice description for introspection.\n* `:fields` - A map of `Absinthe.Type.Field` structs. See `Absinthe.Schema.Notation.field/4` and\n* `:args` - A map of `Absinthe.Type.Argument` structs. See `Absinthe.Schema.Notation.arg/2`.\n* `:resolve_type` - A function used to determine the implementing type of a resolved object. See also `Absinthe.Type.Object`'s `:is_type_of`.\n\nThe `:resolve_type` function will be passed two arguments; the object whose type needs to be identified, and the `Absinthe.Execution` struct providing the full execution context.\n\nThe `__private__` and `:__reference__` keys are for internal use.","ref":"Absinthe.Type.Interface.html#t:t/0","title":"Absinthe.Type.Interface.t/0","type":"type"},{"doc":"A wrapping type which declares the type of each item in the list.","ref":"Absinthe.Type.List.html","title":"Absinthe.Type.List","type":"module"},{"doc":"Given a type, `:item`, to declare the type of a field/argument as a list of\n`:item`-typed values, you could do:\n\n```\ntype: %Absinthe.Type.List{of_type: :item}\n```\n\nBut normally this would be done using `Absinthe.Schema.Notation.list_of/1`.\n\n```\ntype: list_of(:item)\n```","ref":"Absinthe.Type.List.html#module-examples","title":"Examples - Absinthe.Type.List","type":"module"},{"doc":"","ref":"Absinthe.Type.List.html#kind/0","title":"Absinthe.Type.List.kind/0","type":"function"},{"doc":"A defined list type.","ref":"Absinthe.Type.List.html#t:t/0","title":"Absinthe.Type.List.t/0","type":"type"},{"doc":"* `:of_type` - The underlying, wrapped type.","ref":"Absinthe.Type.List.html#t:t/0-options","title":"Options - Absinthe.Type.List.t/0","type":"type"},{"doc":"A type that wraps an underlying type, acting identically to that type but\nadding a non-null constraint.\n\nBy default, all types in GraphQL are nullable. To declare a type that\ndisallows null, wrap it in a `Absinthe.Type.NonNull` struct.\n\nAdding non_null/1 to a type is a breaking change, removing it is not. Client documents that specify non null on a\nvariable eg `query ($id: ID!)` are allowed to be passed to arguments which allow null. If the argument however is\nnon_null, then the variable type MUST be non null as well.","ref":"Absinthe.Type.NonNull.html","title":"Absinthe.Type.NonNull","type":"module"},{"doc":"Given a type, `:item`, to declare it as non-null, you could do the following:\n\n```\ntype: %Absinthe.Type.NonNull{of_type: :item}\n```\n\nBut normally this would be done using `Absinthe.Schema.Notation.non_null/1`.\n\n```\ntype: non_null(:item)\n```","ref":"Absinthe.Type.NonNull.html#module-examples","title":"Examples - Absinthe.Type.NonNull","type":"module"},{"doc":"","ref":"Absinthe.Type.NonNull.html#kind/0","title":"Absinthe.Type.NonNull.kind/0","type":"function"},{"doc":"A defined non-null type.","ref":"Absinthe.Type.NonNull.html#t:t/0","title":"Absinthe.Type.NonNull.t/0","type":"type"},{"doc":"* `:of_type` -- the underlying type to wrap","ref":"Absinthe.Type.NonNull.html#t:t/0-options","title":"Options - Absinthe.Type.NonNull.t/0","type":"type"},{"doc":"","ref":"Absinthe.Type.NonNull.html#t:t/1","title":"Absinthe.Type.NonNull.t/1","type":"type"},{"doc":"Represents a non-leaf node in a GraphQL tree of information.\n\nObjects represent a list of named fields, each of which yield a value of a\nspecific type. Object values are serialized as unordered maps, where the\nqueried field names (or aliases) are the keys and the result of evaluating the\nfield is the value.\n\nAlso see `Absinthe.Type.Scalar`.","ref":"Absinthe.Type.Object.html","title":"Absinthe.Type.Object","type":"module"},{"doc":"Given a type defined as the following (see `Absinthe.Schema.Notation.object/3`):\n\n```\n@desc \"A person\"\nobject :person do\n  field :name, :string\n  field :age, :integer\n  field :best_friend, :person\n  field :pets, list_of(:pet)\nend\n```\n\nThe \"Person\" type (referred inside Absinthe as `:person`) is an object, with\nfields that use `Absinthe.Type.Scalar` types (namely `:name` and `:age`), and\nother `Absinthe.Type.Object` types (`:best_friend` and `:pets`, assuming\n`:pet` is an object).\n\nGiven we have a query that supports getting a person by name\n(see `Absinthe.Schema`), and a query document like the following:\n\n```graphql\n{\n  person(name: \"Joe\") {\n    name\n    best_friend {\n      name\n      age\n    }\n    pets {\n      breed\n    }\n  }\n}\n```\n\nWe could get a result like this:\n\n```\n%{\n  data: %{\n    \"person\" => %{\n      \"best_friend\" => %{\n        \"name\" => \"Jill\",\n        \"age\" => 29\n      },\n      \"pets\" => [\n        %{\"breed\" => \"Wyvern\"},\n        %{\"breed\" => \"Royal Griffon\"}\n      ]\n    }\n  }\n}\n```","ref":"Absinthe.Type.Object.html#module-examples","title":"Examples - Absinthe.Type.Object","type":"module"},{"doc":"","ref":"Absinthe.Type.Object.html#kind/0","title":"Absinthe.Type.Object.kind/0","type":"function"},{"doc":"A defined object type.\n\nNote new object types (with the exception of the root-level `query`, `mutation`, and `subscription`)\nshould be defined using `Absinthe.Schema.Notation.object/3`.\n\n* `:name` - The name of the object type. Should be a TitleCased `binary`. Set automatically.\n* `:description` - A nice description for introspection.\n* `:fields` - A map of `Absinthe.Type.Field` structs. Usually built via `Absinthe.Schema.Notation.field/4`.\n* `:interfaces` - A list of interfaces that this type guarantees to implement. See `Absinthe.Type.Interface`.\n* `:is_type_of` - A function used to identify whether a resolved object belongs to this defined type. For use with `:interfaces` entry and `Absinthe.Type.Interface`. This function will be passed one argument; the object whose type needs to be identified, and should return `true` when the object matches this type.\n\nThe `__private__` and `:__reference__` keys are for internal use.","ref":"Absinthe.Type.Object.html#t:t/0","title":"Absinthe.Type.Object.t/0","type":"type"},{"doc":"Represents a primitive value.\n\nGraphQL responses take the form of a hierarchical tree; the leaves on these\ntrees are scalars.\n\nAlso see `Absinthe.Type.Object`.","ref":"Absinthe.Type.Scalar.html","title":"Absinthe.Type.Scalar","type":"module"},{"doc":"The following built-in scalar types are defined:\n\n* `:boolean` - Represents `true` or `false`. See the [GraphQL Specification](https://www.graphql.org/learn/schema/#scalar-types).\n* `:float` - Represents signed double-precision fractional values as specified by [IEEE 754](https://en.wikipedia.org/wiki/IEEE_754). See the [GraphQL Specification](https://www.graphql.org/learn/schema/#scalar-types).\n* `:id` - Represents a unique identifier, often used to refetch an object or as key for a cache. The ID type is serialized in the same way as a String; however, it is not intended to be human-readable. See the [GraphQL Specification](https://www.graphql.org/learn/schema/#scalar-types).\n* `:integer` - Represents non-fractional signed whole numeric value. **By default it is not compliant with the GraphQl Specification**, it can represent values between `-(2^53 - 1)` and `2^53 - 1` as specified by [IEEE 754](https://en.wikipedia.org/wiki/IEEE_754). To use the [GraphQl compliant Int](https://spec.graphql.org/October2021/#sec-Int), representing values between `-2^31` and `2^31 - 1`, in your schema `use Absinthe.Schema, use_spec_compliant_int_scalar: true`.\n* `:string` - Represents textual data, represented as UTF-8 character sequences. The String type is most often used by GraphQL to represent free-form human-readable text. See the [GraphQL Specification](https://www.graphql.org/learn/schema/#scalar-types).","ref":"Absinthe.Type.Scalar.html#module-built-in-scalars","title":"Built-In Scalars - Absinthe.Type.Scalar","type":"module"},{"doc":"Note that absinthe includes a few generic extra scalar types in `Absinthe.Type.Custom`,\nthese are not part of the GraphQL spec thus they must be explicitly imported.\nConsider using them before implementing a custom type.","ref":"Absinthe.Type.Scalar.html#module-built-in-custom-types","title":"Built-in custom types - Absinthe.Type.Scalar","type":"module"},{"doc":"Supporting ISO8601 week date format, using [Timex](https://hexdocs.pm/timex):\n\n```\nscalar :iso_week_date do\n  description \"ISO8601 week date (ex: 2022-W42-2)\"\n  parse &Timex.parse(&1, \"{YYYY}-W{Wiso}-{WDmon}\")\n  serialize &Timex.format!(&1, \"{YYYY}-W{Wiso}-{WDmon}\")\nend\n```","ref":"Absinthe.Type.Scalar.html#module-example","title":"Example - Absinthe.Type.Scalar","type":"module"},{"doc":"","ref":"Absinthe.Type.Scalar.html#kind/0","title":"Absinthe.Type.Scalar.kind/0","type":"function"},{"doc":"","ref":"Absinthe.Type.Scalar.html#parse/3","title":"Absinthe.Type.Scalar.parse/3","type":"function"},{"doc":"","ref":"Absinthe.Type.Scalar.html#serialize/2","title":"Absinthe.Type.Scalar.serialize/2","type":"function"},{"doc":"A defined scalar type.\n\nNote new scalars should be defined using `Absinthe.Schema.Notation.scalar`.\n\n* `:name` - The name of scalar. Should be a TitleCased `binary`. Set Automatically by `Absinthe.Schema.Notation.scalar`.\n* `:description` - A nice description for introspection.\n* `:serialize` - A function used to convert a value to a form suitable for JSON serialization\n* `:parse` - A function used to convert the raw, incoming form of a scalar to the canonical internal format.\n\nThe `:__private__` and `:__reference__` keys are for internal use.","ref":"Absinthe.Type.Scalar.html#t:t/0","title":"Absinthe.Type.Scalar.t/0","type":"type"},{"doc":"The internal, canonical representation of a scalar value","ref":"Absinthe.Type.Scalar.html#t:value_t/0","title":"Absinthe.Type.Scalar.value_t/0","type":"type"},{"doc":"A unions is an abstract type made up of multiple possible concrete types.\n\nNo common fields are declared in a union. Compare to `Absinthe.Type.Interface`.\n\nBecause it's necessary for the union to determine the concrete type of a\nresolved object, you must either:\n\n* Provide a `:resolve_type` function on the union\n* Provide a `:is_type_of` function on each possible concrete type\n\n```\nunion :search_result do\n  description \"A search result\"\n\n  types [:person, :business]\n  resolve_type fn\n    %Person{}, _ -> :person\n    %Business{}, _ -> :business\n  end\nend\n```","ref":"Absinthe.Type.Union.html","title":"Absinthe.Type.Union","type":"module"},{"doc":"","ref":"Absinthe.Type.Union.html#kind/0","title":"Absinthe.Type.Union.kind/0","type":"function"},{"doc":"* `:name` - The name of the union type. Should be a TitleCased `binary`. Set automatically.\n* `:description` - A nice description for introspection.\n* `:types` - The list of possible types.\n* `:resolve_type` - A function used to determine the concrete type of a resolved object. See also `Absinthe.Type.Object`'s `:is_type_of`. Either `resolve_type` is specified in the union type, or every object type in the union must specify `is_type_of`\n\nThe `:resolve_type` function will be passed two arguments; the object whose type needs to be identified, and the `Absinthe.Execution` struct providing the full execution context.\n\nThe `__private__` and `:__reference__` keys are for internal use.","ref":"Absinthe.Type.Union.html#t:t/0","title":"Absinthe.Type.Union.t/0","type":"type"},{"doc":"Middleware enables custom resolution behaviour on a field.\n\nAll resolution happens through middleware. Even `resolve` functions are\nmiddleware, as the `resolve` macro is just\n\n```\nquote do\n  middleware Absinthe.Resolution, unquote(function_ast)\nend\n```\n\nResolution happens by reducing a list of middleware spec onto an\n`%Absinthe.Resolution{}` struct.","ref":"Absinthe.Middleware.html","title":"Absinthe.Middleware","type":"behaviour"},{"doc":"```\ndefmodule MyApp.Web.Authentication do\n  @behaviour Absinthe.Middleware\n\n  def call(resolution, _config) do\n    case resolution.context do\n      %{current_user: _} ->\n        resolution\n      _ ->\n        resolution\n        |> Absinthe.Resolution.put_result({:error, \"unauthenticated\"})\n    end\n  end\nend\n```\n\nBy specifying `@behaviour Absinthe.Middleware` the compiler will ensure that\nwe provide a `def call` callback. This function takes an\n`%Absinthe.Resolution{}` struct and will also need to return one such struct.\n\nOn that struct there is a `context` key which holds the absinthe context. This\nis generally where things like the current user are placed. For more\ninformation on how the current user ends up in the context please see our full\nauthentication guide on the website.\n\nOur `call/2` function simply checks the context to see if there is a current\nuser. If there is, we pass the resolution onward. If there is not, we update\nthe resolution state to `:resolved` and place an error result.\n\nMiddleware can be placed on a field in three different ways:\n\n1. Using the `Absinthe.Schema.Notation.middleware/2`\nmacro used inside a field definition.\n2. Using the `middleware/3` callback in your schema.\n3. Returning a `{:middleware, middleware_spec, config}`\ntuple from a resolution function.","ref":"Absinthe.Middleware.html#module-example","title":"Example - Absinthe.Middleware","type":"behaviour"},{"doc":"For placing middleware on a particular field, it's handy to use\nthe `middleware/2` macro.\n\nMiddleware will be run in the order in which they are specified.\nThe `middleware/3` callback has final say on what middleware get\nset.\n\nExamples\n\n`MyApp.Web.Authentication` would run before resolution, and `HandleError` would run after.\n```\nfield :hello, :string do\n  middleware MyApp.Web.Authentication\n  resolve &get_the_string/2\n  middleware HandleError, :foo\nend\n```\n\nAnonymous functions are a valid middleware spec. A nice use case\nis altering the context in a logout mutation. Mutations are the\nonly time the context should be altered. This is not enforced.\n```\nfield :logout, :query do\n  middleware fn res, _ ->\n    %{res |\n      context: Map.delete(res.context, :current_user),\n      value: \"logged out\",\n      state: :resolved\n    }\n  end\nend\n```\n\n`middleware/2` even accepts local public function names. Note\nthat `middleware/2` is the only thing that can take local function\nnames without an associated module. If not using macros, use\n`{{__MODULE__, :function_name}, []}`\n```\ndef auth(res, _config) do\n  # auth logic here\nend\n\nquery do\n  field :hello, :string do\n    middleware :auth\n    resolve &get_the_string/2\n  end\nend\n```","ref":"Absinthe.Middleware.html#module-the-middleware-2-macro","title":"The `middleware/2` macro - Absinthe.Middleware","type":"behaviour"},{"doc":"`middleware/3` is a function callback on a schema. When you `use\nAbsinthe.Schema` a default implementation of this function is placed in your\nschema. It is passed the existing middleware for a field, the field itself,\nand the object that the field is a part of.\n\nSo for example if your schema contained:\n\n```\nobject :user do\n  field :name, :string\n  field :age, :integer\nend\n\nquery do\n  field :lookup_user, :user do\n    resolve fn _, _ ->\n      {:ok, %{name: \"Bob\"}}\n    end\n  end\nend\n\ndef middleware(middleware, field, object) do\n  middleware |> IO.inspect\n  field |> IO.inspect\n  object |> IO.inspect\n\n  middleware\nend\n```\n\nGiven a document like:\n```graphql\n{ lookupUser { name }}\n```\n\n`object` is each object that is accessed while executing the document. In our\ncase that is the `:user` object and the `:query` object. `field` is every\nfield on that object, and middleware is a list of whatever middleware\nspec have been configured by the schema on that field. Concretely\nthen, the function will be called , with the following arguments:\n\n```\nYourSchema.middleware([{Absinthe.Resolution, #Function }], lookup_user_field_of_root_query_object, root_query_object)\nYourSchema.middleware([{Absinthe.Middleware.MapGet, :name}], name_field_of_user, user_object)\nYourSchema.middleware([{Absinthe.Middleware.MapGet, :age}], age_field_of_user, user_object)\n```\n\nIn the latter two cases we see that the middleware list is empty. In the first\ncase we see one middleware spec, which is placed by the `resolve` macro used in the\n`:lookup_user` field.\n\n#","ref":"Absinthe.Middleware.html#module-the-middleware-3-callback","title":"The `middleware/3` callback - Absinthe.Middleware","type":"behaviour"},{"doc":"One use of `middleware/3` is setting the default middleware on a field.\nBy default middleware is placed on a\nfield that looks up a field by its snake case identifier, ie `:resource_name`.\nHere is an example of how to change the default to use a camel cased string,\nIE, \"resourceName\".\n\n```\ndef middleware(middleware, %{identifier: identifier} = field, object) do\n  camelized =\n    identifier\n    |> Atom.to_string\n    |> Macro.camelize\n\n  new_middleware_spec = {{__MODULE__, :get_camelized_key}, camelized}\n\n  Absinthe.Schema.replace_default(middleware, new_middleware_spec, field, object)\nend\n\ndef get_camelized_key(%{source: source} = res, key) do\n  %{res | state: :resolved, value: Map.get(source, key)}\nend\n```\n\nThere's a lot going on here so let's unpack it. We need to define a\nspecification to tell Absinthe what middleware to run. The form we're using is\n`{{MODULE, :function_to_call}, options_of_middleware}`. For our purposes we're\nsimply going to use a function in the schema module itself\n`get_camelized_key`.\n\nWe then use the `Absinthe.Schema.replace_default/4` function to swap out the\ndefault middleware already present in the middleware list with the new one we\nwant to use. It handles going through the existing list of middleware and\nseeing if it's using the default or if it has custom resolvers on it. If it's\nusing the default, the function applies our newly defined middleware spec.\n\nLike all middleware functions, `:get_camelized_key` takes a resolution struct,\nand options. The options is the camelized key we generated. We get the\ncamelized string from the parent map, and set it as the value of the\nresolution struct. Finally we mark the resolution state `:resolved`.\n\nSide note: This `middleware/3` function is called whenever we pull the type\nout of the schema. The middleware itself is run every time we get a field on\nan object. If we have 1000 objects and we were doing the camelization logic\nINSIDE the middleware, we would compute the camelized string 1000 times. By\ndoing it in the `def middleware` callback we do it just once.\n\n#","ref":"Absinthe.Middleware.html#module-default-middleware","title":"Default Middleware - Absinthe.Middleware","type":"behaviour"},{"doc":"Let's use our authentication middleware from earlier, and place it on every\nfield in the query object.\n\n```\ndefmodule MyApp.Web.Schema do\n  use Absinthe.Schema\n\n  query do\n    field :private_field, :string do\n      resolve fn _, _ ->\n        {:ok, \"this can only be viewed if authenticated\"}\n      end\n    end\n  end\n\n  def middleware(middleware, _field, %Absinthe.Type.Object{identifier: identifier})\n  when identifier in [:query, :subscription, :mutation] do\n    [MyApp.Web.Authentication | middleware]\n  end\n  def middleware(middleware, _field, _object) do\n    middleware\n  end\nend\n```\n\nIt is important to note that we are matching for the `:query`, `:subscription`\nor `:mutation` identifier types. We do this because the middleware function\nwill be called for each field in the schema. If we didn't limit it to those\ntypes, we would be applying authentication to every field in the entire\nschema, even stuff like `:name` or `:age`. This generally isn't necessary\nprovided you authenticate at the entrypoints.","ref":"Absinthe.Middleware.html#module-object-wide-authentication","title":"Object Wide Authentication - Absinthe.Middleware","type":"behaviour"},{"doc":"- Middleware functions take a `%Absinthe.Resolution{}` struct, and return one.\n- All middleware on a field are always run, make sure to pattern match on the\n  state if you care.","ref":"Absinthe.Middleware.html#module-main-points","title":"Main Points - Absinthe.Middleware","type":"behaviour"},{"doc":"This is the main middleware callback.\n\nIt receives an `%Absinthe.Resolution{}` struct and it needs to return an\n`%Absinthe.Resolution{}` struct. The second argument will be whatever value\nwas passed to the `middleware` call that setup the middleware.","ref":"Absinthe.Middleware.html#c:call/2","title":"Absinthe.Middleware.call/2","type":"callback"},{"doc":"For testing and inspection purposes","ref":"Absinthe.Middleware.html#unshim/2","title":"Absinthe.Middleware.unshim/2","type":"function"},{"doc":"","ref":"Absinthe.Middleware.html#t:function_name/0","title":"Absinthe.Middleware.function_name/0","type":"type"},{"doc":"","ref":"Absinthe.Middleware.html#t:spec/0","title":"Absinthe.Middleware.spec/0","type":"type"},{"doc":"This plugin enables asynchronous execution of a field.\n\nSee also `Absinthe.Resolution.Helpers.async/1`\n\n# Example Usage:\n\nUsing the `Absinthe.Resolution.Helpers.async/1` helper function:\n```elixir\nfield :time_consuming, :thing do\n  resolve fn _, _, _ ->\n    async(fn ->\n      {:ok, long_time_consuming_function()}\n    end)\n  end\nend\n```\n\nUsing the bare plugin API\n```elixir\nfield :time_consuming, :thing do\n  resolve fn _, _, _ ->\n    task = Task.async(fn ->\n      {:ok, long_time_consuming_function()}\n    end)\n    {:middleware, Elixir.Absinthe.Middleware.Async, task}\n  end\nend\n```\n\nThis module also serves as an example for how to build middleware that uses the\nresolution callbacks.\n\nSee the source code and associated comments for further details.","ref":"Absinthe.Middleware.Async.html","title":"Absinthe.Middleware.Async","type":"module"},{"doc":"","ref":"Absinthe.Middleware.Async.html#after_resolution/1","title":"Absinthe.Middleware.Async.after_resolution/1","type":"function"},{"doc":"","ref":"Absinthe.Middleware.Async.html#async/1","title":"Absinthe.Middleware.Async.async/1","type":"function"},{"doc":"","ref":"Absinthe.Middleware.Async.html#before_resolution/1","title":"Absinthe.Middleware.Async.before_resolution/1","type":"function"},{"doc":"","ref":"Absinthe.Middleware.Async.html#call/2","title":"Absinthe.Middleware.Async.call/2","type":"function"},{"doc":"","ref":"Absinthe.Middleware.Async.html#pipeline/2","title":"Absinthe.Middleware.Async.pipeline/2","type":"function"},{"doc":"Batch the resolution of multiple fields.","ref":"Absinthe.Middleware.Batch.html","title":"Absinthe.Middleware.Batch","type":"module"},{"doc":"Consider the following graphql query:\n```\n{\n  posts {\n    author {\n      name\n    }\n  }\n}\n```\n\n`posts` returns a list of `post` objects, which has an associated `author` field.\nIf the `author` field makes a call to the database we have the classic N + 1 problem.\nWhat we want is a way to load all authors for all posts in one database request.\n\nThis plugin provides this, without any eager loading at the parent level. That is,\nthe code for the `posts` field does not need to do anything to facilitate the\nefficient loading of its children.","ref":"Absinthe.Middleware.Batch.html#module-motivation","title":"Motivation - Absinthe.Middleware.Batch","type":"module"},{"doc":"The API for this plugin is a little on the verbose side because it is not specific\nto any particular batching mechanism. That is, this API is just as useful for an Ecto\nbased DB as it is for talking to S3 or the File System. Thus we anticipate people\n(including ourselves) will be creating additional functions more tailored to each\nof those specific use cases.\n\nHere is an example using the `Absinthe.Resolution.Helpers.batch/3` helper.\n```elixir\nobject :post do\n  field :name, :string\n  field :author, :user do\n    resolve fn post, _, _ ->\n      batch({__MODULE__, :users_by_id}, post.author_id, fn batch_results ->\n        {:ok, Map.get(batch_results, post.author_id)}\n      end)\n    end\n  end\nend\n\ndef users_by_id(_, user_ids) do\n  users = Repo.all from u in User, where: u.id in ^user_ids\n  Map.new(users, fn user -> {user.id, user} end)\nend\n```\n\nLet's look at this piece by piece:\n- `{__MODULE__, :users_by_id}`: is the batching function which will be used. It must\nbe a 2 arity function. For details see the `batch_fun` typedoc.\n- `post.author_id`: This is the information to be aggregated. The aggregated values\nare the second argument to the batching function.\n- `fn batch_results`: This function takes the results from the batching function.\nit should return one of the resolution function values.","ref":"Absinthe.Middleware.Batch.html#module-example-usage","title":"Example Usage - Absinthe.Middleware.Batch","type":"module"},{"doc":"","ref":"Absinthe.Middleware.Batch.html#after_resolution/1","title":"Absinthe.Middleware.Batch.after_resolution/1","type":"function"},{"doc":"","ref":"Absinthe.Middleware.Batch.html#async/1","title":"Absinthe.Middleware.Batch.async/1","type":"function"},{"doc":"","ref":"Absinthe.Middleware.Batch.html#before_resolution/1","title":"Absinthe.Middleware.Batch.before_resolution/1","type":"function"},{"doc":"","ref":"Absinthe.Middleware.Batch.html#call/2","title":"Absinthe.Middleware.Batch.call/2","type":"function"},{"doc":"","ref":"Absinthe.Middleware.Batch.html#pipeline/2","title":"Absinthe.Middleware.Batch.pipeline/2","type":"function"},{"doc":"The function to be called with the aggregate batch information.\n\nIt comes in both a 2 tuple and 3 tuple form. The first two elements are the module\nand function name. The third element is an arbitrary parameter that is passed\nas the first argument to the batch function.\n\nFor example, one could parameterize the `users_by_id` function from the moduledoc\nto make it more generic. Instead of doing `{__MODULE__, :users_by_id}` you could do\n`{__MODULE__, :by_id, User}`. Then the function would be:\n\n```elixir\ndef by_id(model, ids) do\n  model\n  |> where([m], m.id in ^ids)\n  |> Repo.all()\n  |> Map.new(&{&1.id, &1})\nend\n```\nIt could also be used to set options unique to the execution of a particular\nbatching function.","ref":"Absinthe.Middleware.Batch.html#t:batch_fun/0","title":"Absinthe.Middleware.Batch.batch_fun/0","type":"type"},{"doc":"","ref":"Absinthe.Middleware.Batch.html#t:post_batch_fun/0","title":"Absinthe.Middleware.Batch.post_batch_fun/0","type":"type"},{"doc":"","ref":"Absinthe.Middleware.Dataloader.html","title":"Absinthe.Middleware.Dataloader","type":"module"},{"doc":"This is the default middleware. It assumes the the object it receives is a map\nand uses `Map.get/2` to get the value for this field. If this field is already\nmarked as resolved, then this middleware does not touch it.\n\nIf you want to replace this middleware you should use\n`Absinthe.Schema.replace_default/4`","ref":"Absinthe.Middleware.MapGet.html","title":"Absinthe.Middleware.MapGet","type":"module"},{"doc":"","ref":"Absinthe.Middleware.MapGet.html#call/2","title":"Absinthe.Middleware.MapGet.call/2","type":"function"},{"doc":"Middleware that just passes the parent down to the children.\n\nThis is the default resolver for subscription fields.","ref":"Absinthe.Middleware.PassParent.html","title":"Absinthe.Middleware.PassParent","type":"module"},{"doc":"","ref":"Absinthe.Middleware.PassParent.html#call/2","title":"Absinthe.Middleware.PassParent.call/2","type":"function"},{"doc":"Gather and report telemetry about an individual field resolution","ref":"Absinthe.Middleware.Telemetry.html","title":"Absinthe.Middleware.Telemetry","type":"module"},{"doc":"","ref":"Absinthe.Middleware.Telemetry.html#on_complete/2","title":"Absinthe.Middleware.Telemetry.on_complete/2","type":"function"},{"doc":"Plugin Behaviour\n\nPlugins are an additional set of callbacks that can be used by module based\nmiddleware to run actions before and after resolution, as well as add additional\nphases to run after resolution","ref":"Absinthe.Plugin.html","title":"Absinthe.Plugin","type":"behaviour"},{"doc":"callback to do something with the resolution accumulator after\nresolution.\n\nNOTE: This function is given the full accumulator. Namespacing is suggested to\navoid conflicts.","ref":"Absinthe.Plugin.html#c:after_resolution/1","title":"Absinthe.Plugin.after_resolution/1","type":"callback"},{"doc":"callback to setup the resolution accumulator prior to resolution.\n\nNOTE: This function is given the full accumulator. Namespacing is suggested to\navoid conflicts.","ref":"Absinthe.Plugin.html#c:before_resolution/1","title":"Absinthe.Plugin.before_resolution/1","type":"callback"},{"doc":"Returns the list of default plugins.","ref":"Absinthe.Plugin.html#defaults/0","title":"Absinthe.Plugin.defaults/0","type":"function"},{"doc":"callback used to specify additional phases to run.\n\nPlugins may require additional resolution phases to be run. This function should\nuse values set in the resolution accumulator to determine\nwhether or not additional phases are required.\n\nNOTE: This function is given the whole pipeline to be inserted after the current\nphase completes.","ref":"Absinthe.Plugin.html#c:pipeline/2","title":"Absinthe.Plugin.pipeline/2","type":"callback"},{"doc":"","ref":"Absinthe.Plugin.html#t:t/0","title":"Absinthe.Plugin.t/0","type":"type"},{"doc":"Real time updates via GraphQL\n\nFor a how to guide on getting started with Absinthe.Subscriptions in your phoenix\nproject see the `Absinthe.Phoenix` package.\n\nDefine in your schema via `Absinthe.Schema.subscription/2`","ref":"Absinthe.Subscription.html","title":"Absinthe.Subscription","type":"module"},{"doc":"","ref":"Absinthe.Subscription.html#module-basic-usage","title":"Basic Usage - Absinthe.Subscription","type":"module"},{"doc":"There are a couple of limitations to the beta release of subscriptions that\nare worth keeping in mind if you want to use this in production:\n\nBy design, all subscription docs triggered by a mutation are run inside the\nmutation process as a form of back pressure.\n\nAt the moment however database batching does not happen across the set of\nsubscription docs. Thus if you have a lot of subscription docs and they each\ndo a lot of extra DB lookups you're going to delay incoming mutation responses\nby however long it takes to do all that work.\n\nBefore the final version of 1.4.0 we want\n\n- Batching across subscriptions\n- More user control over back pressure / async balance.","ref":"Absinthe.Subscription.html#module-performance-characteristics","title":"Performance Characteristics - Absinthe.Subscription","type":"module"},{"doc":"Build a child specification for subscriptions.\n\nIn order to use subscriptions in your application, you must add\n`Absinthe.Subscription` to your supervision tree after your endpoint.\n\nSee `guides/subscriptions.md` for more information on how to get up and\nrunning with subscriptions.","ref":"Absinthe.Subscription.html#child_spec/1","title":"Absinthe.Subscription.child_spec/1","type":"function"},{"doc":"* `:pubsub` - (Required) The `Phoenix.Pubsub` that should be used to publish\n  subscriptions. Typically this will be your `Phoenix.Endpoint`.\n* `:compress_registry?` - (Optional - default `true`) A boolean controlling\n  whether the Registry used to keep track of subscriptions will should be\n  compressed or not.\n* `:pool_size` - (Optional - default `System.schedulers() * 2`) An integer\n  specifying the number of `Absinthe.Subscription.Proxy` processes to start.","ref":"Absinthe.Subscription.html#child_spec/1-options","title":"Options - Absinthe.Subscription.child_spec/1","type":"function"},{"doc":"Publish a mutation\n\nThis function is generally used when trying to publish to one or more subscription\nfields \"out of band\" from any particular mutation.","ref":"Absinthe.Subscription.html#publish/3","title":"Absinthe.Subscription.publish/3","type":"function"},{"doc":"Note: As with all subscription examples if you're using Absinthe.Phoenix `pubsub`\nwill be `MyAppWeb.Endpoint`.\n\n```\nAbsinthe.Subscription.publish(pubsub, user, [new_users: user.account_id])\n```\n```\n# publish to two subscription fields\nAbsinthe.Subscription.publish(pubsub, user, [\n  new_users: user.account_id,\n  other_user_subscription_field: user.id,\n])\n```","ref":"Absinthe.Subscription.html#publish/3-examples","title":"Examples - Absinthe.Subscription.publish/3","type":"function"},{"doc":"Add Absinthe.Subscription to your process tree.","ref":"Absinthe.Subscription.html#start_link/1","title":"Absinthe.Subscription.start_link/1","type":"function"},{"doc":"","ref":"Absinthe.Subscription.html#t:opt/0","title":"Absinthe.Subscription.opt/0","type":"type"},{"doc":"","ref":"Absinthe.Subscription.html#t:subscription_field_spec/0","title":"Absinthe.Subscription.subscription_field_spec/0","type":"type"},{"doc":"This module handles broadcasting documents that are local to this node","ref":"Absinthe.Subscription.Local.html","title":"Absinthe.Subscription.Local","type":"module"},{"doc":"Publish a mutation to the local node only.\n\nSee also `Absinthe.Subscription.publish/3`","ref":"Absinthe.Subscription.Local.html#publish_mutation/3","title":"Absinthe.Subscription.Local.publish_mutation/3","type":"function"},{"doc":"Serializer responsible for packing and unpacking pipeline stored in the Elixir registry.\n\nThe purpose of this logic is saving memory by deduplicating repeating options - (ETS\nbacked registry stores them flat in the memory).","ref":"Absinthe.Subscription.PipelineSerializer.html","title":"Absinthe.Subscription.PipelineSerializer","type":"module"},{"doc":"","ref":"Absinthe.Subscription.PipelineSerializer.html#pack/1","title":"Absinthe.Subscription.PipelineSerializer.pack/1","type":"function"},{"doc":"","ref":"Absinthe.Subscription.PipelineSerializer.html#unpack/1","title":"Absinthe.Subscription.PipelineSerializer.unpack/1","type":"function"},{"doc":"","ref":"Absinthe.Subscription.PipelineSerializer.html#t:options_label/0","title":"Absinthe.Subscription.PipelineSerializer.options_label/0","type":"type"},{"doc":"","ref":"Absinthe.Subscription.PipelineSerializer.html#t:options_map/0","title":"Absinthe.Subscription.PipelineSerializer.options_map/0","type":"type"},{"doc":"","ref":"Absinthe.Subscription.PipelineSerializer.html#t:packed_phase_config/0","title":"Absinthe.Subscription.PipelineSerializer.packed_phase_config/0","type":"type"},{"doc":"","ref":"Absinthe.Subscription.PipelineSerializer.html#t:packed_pipeline/0","title":"Absinthe.Subscription.PipelineSerializer.packed_pipeline/0","type":"type"},{"doc":"Pubsub behaviour expected by Absinthe to power subscriptions\n\nA subscription includes a GraphQL query document that resolves to a set of \nobjects and fields. When the subscription is triggered, Absinthe will run the\ndocument and publish the resolved objects to subscribers through a module that\nimplements the behaviour defined here.\n\nEach application is free to implement the PubSub behavior in its own way.  \nFor example, the absinthe_phoenix project implements the subscription pubsub\nusing Phoenix.PubSub by way of the application's Endpoint.  Regardless \nof the underlying mechanisms, the implementation should maintain the type \nsignatures and expected behaviors of the callbacks below.","ref":"Absinthe.Subscription.Pubsub.html","title":"Absinthe.Subscription.Pubsub","type":"behaviour"},{"doc":"An Absinthe.Subscription.Pubsub system may extend across multiple nodes\nconnected by some mechanism. Regardless of this mechanism, all nodes should\nhave unique names.\n\nAbsinthe invokes `node_name` function to get current node's name. If you\nare running inside erlang cluster, you can use `Kernel.node/0` as a node\nname.","ref":"Absinthe.Subscription.Pubsub.html#c:node_name/0","title":"Absinthe.Subscription.Pubsub.node_name/0","type":"callback"},{"doc":"An Absinthe.Subscription.Pubsub system may extend across multiple nodes.\nProcesses need only subscribe to the pubsub process that\nis running on their own node.\n\nHowever, mutations can happen on any node in the cluster and must to be\nbroadcast to other nodes so that they can also reevaluate their GraphQL \nsubscriptions and notify subscribers on that node.\n\nWhen told of a mutation, Absinthe invokes the `publish_mutation` function \non the node in which the mutation is processed first. The function should\npublish a message to the given `proxy_topic`, with the identity of node \non which the mutation occurred included in the broadcast message.\n\nThe message broadcast should be a map that contains, at least\n\n    %{\n        node: node_name,      # should be equal to `node_name/0`\n        mutation_result: …,   # from arguments\n        subscribed_fields: …  # from arguments\n\n        # other fields as needed\n    }","ref":"Absinthe.Subscription.Pubsub.html#c:publish_mutation/3","title":"Absinthe.Subscription.Pubsub.publish_mutation/3","type":"callback"},{"doc":"After a mutation is published, and Absinthe has re-run the necessary GraphQL \nsubscriptions to generate a new set of resolved data, it calls\n`publish_subscription`.\n\nYour pubsub implementation should publish a message to the given topic, with\nthe newly resolved data. The broadcast should be limited to the current node \nonly.","ref":"Absinthe.Subscription.Pubsub.html#c:publish_subscription/2","title":"Absinthe.Subscription.Pubsub.publish_subscription/2","type":"callback"},{"doc":"Subscribe the current process for messages about the given topic.","ref":"Absinthe.Subscription.Pubsub.html#c:subscribe/1","title":"Absinthe.Subscription.Pubsub.subscribe/1","type":"callback"},{"doc":"","ref":"Absinthe.Subscription.Pubsub.html#t:t/0","title":"Absinthe.Subscription.Pubsub.t/0","type":"type"},{"doc":"Behaviour for Absinthe Phases.\n\nA phase takes an `Absinthe.Blueprint` document and returns another blueprint document.\nAll validation, resolution, and result building happens via phases. See\n`Absinthe.Pipeline` for information on how to run phases. See the code under\nthis namespace for information on individual phases.","ref":"Absinthe.Phase.html","title":"Absinthe.Phase","type":"behaviour"},{"doc":"","ref":"Absinthe.Phase.html#any_invalid?/1","title":"Absinthe.Phase.any_invalid?/1","type":"function"},{"doc":"","ref":"Absinthe.Phase.html#put_error/2","title":"Absinthe.Phase.put_error/2","type":"function"},{"doc":"","ref":"Absinthe.Phase.html#c:run/2","title":"Absinthe.Phase.run/2","type":"callback"},{"doc":"","ref":"Absinthe.Phase.html#t:result_t/0","title":"Absinthe.Phase.result_t/0","type":"type"},{"doc":"","ref":"Absinthe.Phase.html#t:t/0","title":"Absinthe.Phase.t/0","type":"type"},{"doc":"Pass on context and root value to document.","ref":"Absinthe.Phase.Document.Context.html","title":"Absinthe.Phase.Document.Context","type":"module"},{"doc":"","ref":"Absinthe.Phase.Document.Context.html#flag_invalid/1","title":"Absinthe.Phase.Document.Context.flag_invalid/1","type":"function"},{"doc":"","ref":"Absinthe.Phase.Document.Context.html#flag_invalid/2","title":"Absinthe.Phase.Document.Context.flag_invalid/2","type":"function"},{"doc":"","ref":"Absinthe.Phase.Document.Context.html#flag_invalid/3","title":"Absinthe.Phase.Document.Context.flag_invalid/3","type":"function"},{"doc":"","ref":"Absinthe.Phase.Document.Context.html#inherit_invalid/3","title":"Absinthe.Phase.Document.Context.inherit_invalid/3","type":"function"},{"doc":"","ref":"Absinthe.Phase.Document.Context.html#put_flag/2","title":"Absinthe.Phase.Document.Context.put_flag/2","type":"function"},{"doc":"","ref":"Absinthe.Phase.Document.Context.html#run/2","title":"Absinthe.Phase.Document.Context.run/2","type":"function"},{"doc":"Gather and report telemetry about an operation.","ref":"Absinthe.Phase.Telemetry.html","title":"Absinthe.Phase.Telemetry","type":"module"},{"doc":"","ref":"Absinthe.Phase.Telemetry.html#flag_invalid/1","title":"Absinthe.Phase.Telemetry.flag_invalid/1","type":"function"},{"doc":"","ref":"Absinthe.Phase.Telemetry.html#flag_invalid/2","title":"Absinthe.Phase.Telemetry.flag_invalid/2","type":"function"},{"doc":"","ref":"Absinthe.Phase.Telemetry.html#flag_invalid/3","title":"Absinthe.Phase.Telemetry.flag_invalid/3","type":"function"},{"doc":"","ref":"Absinthe.Phase.Telemetry.html#inherit_invalid/3","title":"Absinthe.Phase.Telemetry.inherit_invalid/3","type":"function"},{"doc":"","ref":"Absinthe.Phase.Telemetry.html#put_flag/2","title":"Absinthe.Phase.Telemetry.put_flag/2","type":"function"},{"doc":"","ref":"Absinthe.Phase.Telemetry.html#run/2","title":"Absinthe.Phase.Telemetry.run/2","type":"function"},{"doc":"Execute a pipeline of phases.\n\nA pipeline is merely a list of phases. This module contains functions for building,\nmodifying, and executing pipelines of phases.\n\nPipelines are used to build, validate and manipulate GraphQL documents or schema's.\n\n* See [`Absinthe.Plug`](https://hexdocs.pm/absinthe_plug/Absinthe.Plug.html) on adjusting the document pipeline for GraphQL over http requests.\n* See [`Absinthe.Phoenix`](https://hexdocs.pm/absinthe_phoenix/) on adjusting the document pipeline for GraphQL over Phoenix channels.\n* See `Absinthe.Schema` on adjusting the schema pipeline for schema manipulation.","ref":"Absinthe.Pipeline.html","title":"Absinthe.Pipeline","type":"module"},{"doc":"Return the part of a pipeline before a specific phase.","ref":"Absinthe.Pipeline.html#before/2","title":"Absinthe.Pipeline.before/2","type":"function"},{"doc":"iex> Pipeline.before([A, B, C], B)\n    [A]","ref":"Absinthe.Pipeline.html#before/2-examples","title":"Examples - Absinthe.Pipeline.before/2","type":"function"},{"doc":"","ref":"Absinthe.Pipeline.html#default_schema_options/0","title":"Absinthe.Pipeline.default_schema_options/0","type":"function"},{"doc":"The default document pipeline","ref":"Absinthe.Pipeline.html#for_document/2","title":"Absinthe.Pipeline.for_document/2","type":"function"},{"doc":"The default schema pipeline","ref":"Absinthe.Pipeline.html#for_schema/2","title":"Absinthe.Pipeline.for_schema/2","type":"function"},{"doc":"Return the part of a pipeline after (and including) a specific phase.","ref":"Absinthe.Pipeline.html#from/2","title":"Absinthe.Pipeline.from/2","type":"function"},{"doc":"iex> Pipeline.from([A, B, C], B)\n    [B, C]","ref":"Absinthe.Pipeline.html#from/2-examples","title":"Examples - Absinthe.Pipeline.from/2","type":"function"},{"doc":"Return the pipeline with the phase/list of phases inserted after\nthe supplied phase.","ref":"Absinthe.Pipeline.html#insert_after/3","title":"Absinthe.Pipeline.insert_after/3","type":"function"},{"doc":"Add one phase after another:\n\n    iex> Pipeline.insert_after([A, C, D], A, B)\n    [A, B, C, D]\n\nAdd list of phases after another:\n\n    iex> Pipeline.insert_after([A, D, E], A, [B, C])\n    [A, B, C, D, E]","ref":"Absinthe.Pipeline.html#insert_after/3-examples","title":"Examples - Absinthe.Pipeline.insert_after/3","type":"function"},{"doc":"Return the pipeline with the phase/list of phases inserted before\nthe supplied phase.","ref":"Absinthe.Pipeline.html#insert_before/3","title":"Absinthe.Pipeline.insert_before/3","type":"function"},{"doc":"Add one phase before another:\n\n    iex> Pipeline.insert_before([A, C, D], C, B)\n    [A, B, C, D]\n\nAdd list of phase before another:\n\n    iex> Pipeline.insert_before([A, D, E], D, [B, C])\n    [A, B, C, D, E]","ref":"Absinthe.Pipeline.html#insert_before/3-examples","title":"Examples - Absinthe.Pipeline.insert_before/3","type":"function"},{"doc":"","ref":"Absinthe.Pipeline.html#options/1","title":"Absinthe.Pipeline.options/1","type":"function"},{"doc":"Return the pipeline with the phases matching the regex removed.","ref":"Absinthe.Pipeline.html#reject/2","title":"Absinthe.Pipeline.reject/2","type":"function"},{"doc":"iex> Pipeline.reject([A, B, C], ~r/A|B/)\n    [C]","ref":"Absinthe.Pipeline.html#reject/2-examples","title":"Examples - Absinthe.Pipeline.reject/2","type":"function"},{"doc":"Replace a phase in a pipeline with another, supporting reusing the same\noptions.","ref":"Absinthe.Pipeline.html#replace/3","title":"Absinthe.Pipeline.replace/3","type":"function"},{"doc":"Replace a simple phase (without options):\n\n    iex> Pipeline.replace([A, B, C], B, X)\n    [A, X, C]\n\nReplace a phase with options, retaining them:\n\n    iex> Pipeline.replace([A, {B, [name: \"Thing\"]}, C], B, X)\n    [A, {X, [name: \"Thing\"]}, C]\n\nReplace a phase with options, overriding them:\n\n    iex> Pipeline.replace([A, {B, [name: \"Thing\"]}, C], B, {X, [name: \"Nope\"]})\n    [A, {X, [name: \"Nope\"]}, C]","ref":"Absinthe.Pipeline.html#replace/3-examples","title":"Examples - Absinthe.Pipeline.replace/3","type":"function"},{"doc":"","ref":"Absinthe.Pipeline.html#run/2","title":"Absinthe.Pipeline.run/2","type":"function"},{"doc":"","ref":"Absinthe.Pipeline.html#run_phase/3","title":"Absinthe.Pipeline.run_phase/3","type":"function"},{"doc":"Return the part of a pipeline up to and including a specific phase.","ref":"Absinthe.Pipeline.html#upto/2","title":"Absinthe.Pipeline.upto/2","type":"function"},{"doc":"iex> Pipeline.upto([A, B, C], B)\n    [A, B]","ref":"Absinthe.Pipeline.html#upto/2-examples","title":"Examples - Absinthe.Pipeline.upto/2","type":"function"},{"doc":"Return the pipeline with the supplied phase removed.","ref":"Absinthe.Pipeline.html#without/2","title":"Absinthe.Pipeline.without/2","type":"function"},{"doc":"iex> Pipeline.without([A, B, C], B)\n    [A, C]","ref":"Absinthe.Pipeline.html#without/2-examples","title":"Examples - Absinthe.Pipeline.without/2","type":"function"},{"doc":"","ref":"Absinthe.Pipeline.html#t:data_t/0","title":"Absinthe.Pipeline.data_t/0","type":"type"},{"doc":"","ref":"Absinthe.Pipeline.html#t:phase_config_t/0","title":"Absinthe.Pipeline.phase_config_t/0","type":"type"},{"doc":"","ref":"Absinthe.Pipeline.html#t:t/0","title":"Absinthe.Pipeline.t/0","type":"type"},{"doc":"Absinthe supports an adapter mechanism that allows developers to define their\nschema using one code convention (eg, `snake_cased` fields and arguments), but\naccept query documents and return results (including names in errors) in\nanother (eg, `camelCase`).\n\nAdapters aren't a part of GraphQL, but a utility that Absinthe adds so that\nboth client and server can use use conventions most natural to them.\n\nAbsinthe ships with four adapters:\n\n* `Absinthe.Adapter.LanguageConventions`, which expects schemas to be defined\n  in `snake_case` (the standard Elixir convention), translating to/from `camelCase`\n  for incoming query documents and outgoing results. (This is the default as of v0.3.)\n* `Absinthe.Adapter.Underscore`, which is similar to the `Absinthe.Adapter.LanguageConventions`\n  adapter but converts all incoming identifiers to underscores and does not\n  modify outgoing identifiers (since those are already expected to be\n  underscores). Unlike `Absinthe.Adapter.Passthrough` this does not break\n  introspection.\n* `Absinthe.Adapter.Passthrough`, which is a no-op adapter and makes no\n  modifications. (Note at the current time this does not support introspection\n  if you're using camelized conventions).\n* `Absinthe.Adapter.StrictLanguageConventions`, which expects schemas to be\n  defined in `snake_case`, translating to `camelCase` for outgoing results.\n  This adapter requires incoming query documents to use `camelCase`.\n\nTo set an adapter, you pass a configuration option at runtime:\n\nFor `Absinthe.run/3`:\n\n```\nAbsinthe.run(\n  query,\n  MyApp.Schema,\n  adapter: YourApp.Adapter.TheAdapterName\n)\n```\n\nFor `Absinthe.Plug`:\n\n```\nforward \"/api\",\n  to: Absinthe.Plug,\n  init_opts: [schema: MyAppWeb.Schema, adapter: YourApp.Adapter.TheAdapterName]\n```\n\nFor GraphiQL:\n\n```\nforward \"/graphiql\",\n  to: Absinthe.Plug.GraphiQL,\n  init_opts: [schema: MyAppWeb.Schema, adapter: YourApp.Adapter.TheAdapterName]\n```\n\nCheck `Absinthe.Plug` for full documentation on using the Plugs\n\nNotably, this means you're able to switch adapters on case-by-case basis.\nIn a Phoenix application, this means you could even support using different\nadapters for different clients.\n\nA custom adapter module must merely implement the `Absinthe.Adapter` protocol,\nin many cases with `use Absinthe.Adapter` and only overriding the desired\nfunctions.","ref":"Absinthe.Adapter.html","title":"Absinthe.Adapter","type":"behaviour"},{"doc":"All you may need to implement in your adapter is `to_internal_name/2` and\n`to_external_name/2`.\n\nCheck out `Absinthe.Adapter.LanguageConventions` for a good example.\n\nNote that types that are defined external to your application (including\nthe introspection types) may not be compatible if you're using a different\nadapter.","ref":"Absinthe.Adapter.html#module-writing-your-own","title":"Writing Your Own - Absinthe.Adapter","type":"behaviour"},{"doc":"Convert a name from an internal name to an external name.","ref":"Absinthe.Adapter.html#c:to_external_name/2","title":"Absinthe.Adapter.to_external_name/2","type":"callback"},{"doc":"Remove the role-prefix (the inverse of what we did in `to_internal_name/2` above):\n\n```\ndef to_external_name(internal_name, role) do\n  internal_name\n  |> String.replace(~r/^#{role}_/, \"\")\nend\n```","ref":"Absinthe.Adapter.html#c:to_external_name/2-examples","title":"Examples - Absinthe.Adapter.to_external_name/2","type":"callback"},{"doc":"Convert a name from an external name to an internal name.","ref":"Absinthe.Adapter.html#c:to_internal_name/2","title":"Absinthe.Adapter.to_internal_name/2","type":"callback"},{"doc":"Prefix all names with their role, just for fun!\n\n```\ndef to_internal_name(external_name, role) do\n  role_name = role |> to_string\n  role_name   \"_\"   external_name\nend\n```","ref":"Absinthe.Adapter.html#c:to_internal_name/2-examples","title":"Examples - Absinthe.Adapter.to_internal_name/2","type":"callback"},{"doc":"The lexical role of a name within the document/schema.","ref":"Absinthe.Adapter.html#t:role_t/0","title":"Absinthe.Adapter.role_t/0","type":"type"},{"doc":"","ref":"Absinthe.Adapter.html#t:t/0","title":"Absinthe.Adapter.t/0","type":"type"},{"doc":"This defines an adapter that supports GraphQL query documents in their\nconventional (in JS) camelcase notation, while allowing the schema to be\ndefined using conventional (in Elixir) underscore (snakecase) notation, and\ntransforming the names as needed for lookups, results, and error messages.\n\nFor example, this document:\n\n```graphql\n{\n  myUser: createUser(userId: 2) {\n    firstName\n    lastName\n  }\n}\n```\n\nWould map to an internal schema that used the following names:\n\n* `create_user` instead of `createUser`\n* `user_id` instead of `userId`\n* `first_name` instead of `firstName`\n* `last_name` instead of `lastName`\n\nLikewise, the result of executing this (camelcase) query document against our\n(snakecase) schema would have its names transformed back into camelcase on the\nway out:\n\n```\n%{\n  data: %{\n    \"myUser\" => %{\n      \"firstName\" => \"Joe\",\n      \"lastName\" => \"Black\"\n    }\n  }\n}\n```\n\nNote variables are a client-facing concern (they may be provided as\nparameters), so variable names should match the convention of the query\ndocument (eg, camelCase).","ref":"Absinthe.Adapter.LanguageConventions.html","title":"Absinthe.Adapter.LanguageConventions","type":"module"},{"doc":"Converts a snake_case name to camelCase","ref":"Absinthe.Adapter.LanguageConventions.html#to_external_name/2","title":"Absinthe.Adapter.LanguageConventions.to_external_name/2","type":"function"},{"doc":"Converts a camelCase to snake_case","ref":"Absinthe.Adapter.LanguageConventions.html#to_internal_name/2","title":"Absinthe.Adapter.LanguageConventions.to_internal_name/2","type":"function"},{"doc":"The default adapter, which makes no changes to incoming query document\nASTs or outgoing results.","ref":"Absinthe.Adapter.Passthrough.html","title":"Absinthe.Adapter.Passthrough","type":"module"},{"doc":"","ref":"Absinthe.Adapter.Passthrough.html#dump_results/1","title":"Absinthe.Adapter.Passthrough.dump_results/1","type":"function"},{"doc":"","ref":"Absinthe.Adapter.Passthrough.html#load_document/1","title":"Absinthe.Adapter.Passthrough.load_document/1","type":"function"},{"doc":"","ref":"Absinthe.Adapter.Passthrough.html#to_external_name/2","title":"Absinthe.Adapter.Passthrough.to_external_name/2","type":"function"},{"doc":"","ref":"Absinthe.Adapter.Passthrough.html#to_internal_name/2","title":"Absinthe.Adapter.Passthrough.to_internal_name/2","type":"function"},{"doc":"Strict version of `Absinthe.Adapter.LanguageConventions` that will reject\nimproperly formatted external names.\n\nFor example, this document:\n\n```graphql\n{\n  create_user(user_id: 2) {\n    first_name\n    last_name\n  }\n}\n```\n\nWould result in name-mismatch errors returned to the client.\n\nThe client should instead send the camelcase variant of the names:\n\n```graphql\n{\n  createUser(userId: 2) {\n    firstName\n    lastName\n  }\n}\n```\n\nSee `Absinthe.Adapter.LanguageConventions` for more information.","ref":"Absinthe.Adapter.StrictLanguageConventions.html","title":"Absinthe.Adapter.StrictLanguageConventions","type":"module"},{"doc":"Converts a snake_case to camelCase\n\nSee `Absinthe.Adapter.LanguageConventions.to_external_name/2`","ref":"Absinthe.Adapter.StrictLanguageConventions.html#to_external_name/2","title":"Absinthe.Adapter.StrictLanguageConventions.to_external_name/2","type":"function"},{"doc":"Converts a camelCase to snake_case\n\nReturns `nil` if the converted internal name does not match the converted external name.\n\nSee `Absinthe.Adapter.LanguageConventions.to_internal_name/2`","ref":"Absinthe.Adapter.StrictLanguageConventions.html#to_internal_name/2","title":"Absinthe.Adapter.StrictLanguageConventions.to_internal_name/2","type":"function"},{"doc":"Underscores external input and leaves external input alone. Unlike the\n`Absinthe.Adapter.Passthrough` this does not break introspection (because\nintrospection relies on underscoring incoming introspection queries which we\nstill do).","ref":"Absinthe.Adapter.Underscore.html","title":"Absinthe.Adapter.Underscore","type":"module"},{"doc":"","ref":"Absinthe.Adapter.Underscore.html#to_external_name/2","title":"Absinthe.Adapter.Underscore.to_external_name/2","type":"function"},{"doc":"","ref":"Absinthe.Adapter.Underscore.html#to_internal_name/2","title":"Absinthe.Adapter.Underscore.to_internal_name/2","type":"function"},{"doc":"Represents the graphql document to be executed.\n\nPlease see the code itself for more information on individual blueprint sub\nmodules.","ref":"Absinthe.Blueprint.html","title":"Absinthe.Blueprint","type":"module"},{"doc":"","ref":"Absinthe.Blueprint.html#add_field/3","title":"Absinthe.Blueprint.add_field/3","type":"function"},{"doc":"Get the currently selected operation.","ref":"Absinthe.Blueprint.html#current_operation/1","title":"Absinthe.Blueprint.current_operation/1","type":"function"},{"doc":"Append the given field or fields to the given type","ref":"Absinthe.Blueprint.html#extend_fields/2","title":"Absinthe.Blueprint.extend_fields/2","type":"function"},{"doc":"","ref":"Absinthe.Blueprint.html#find/2","title":"Absinthe.Blueprint.find/2","type":"function"},{"doc":"","ref":"Absinthe.Blueprint.html#find_field/2","title":"Absinthe.Blueprint.find_field/2","type":"function"},{"doc":"Determine whether a flag has been set on a node.","ref":"Absinthe.Blueprint.html#flagged?/2","title":"Absinthe.Blueprint.flagged?/2","type":"function"},{"doc":"","ref":"Absinthe.Blueprint.html#fragment/2","title":"Absinthe.Blueprint.fragment/2","type":"function"},{"doc":"","ref":"Absinthe.Blueprint.html#postwalk/2","title":"Absinthe.Blueprint.postwalk/2","type":"function"},{"doc":"","ref":"Absinthe.Blueprint.html#postwalk/3","title":"Absinthe.Blueprint.postwalk/3","type":"function"},{"doc":"","ref":"Absinthe.Blueprint.html#prewalk/2","title":"Absinthe.Blueprint.prewalk/2","type":"function"},{"doc":"","ref":"Absinthe.Blueprint.html#prewalk/3","title":"Absinthe.Blueprint.prewalk/3","type":"function"},{"doc":"Add a flag to a node.","ref":"Absinthe.Blueprint.html#put_flag/3","title":"Absinthe.Blueprint.put_flag/3","type":"function"},{"doc":"Index the types by their name","ref":"Absinthe.Blueprint.html#types_by_name/1","title":"Absinthe.Blueprint.types_by_name/1","type":"function"},{"doc":"Update the current operation.","ref":"Absinthe.Blueprint.html#update_current/2","title":"Absinthe.Blueprint.update_current/2","type":"function"},{"doc":"","ref":"Absinthe.Blueprint.html#t:flags_t/0","title":"Absinthe.Blueprint.flags_t/0","type":"type"},{"doc":"","ref":"Absinthe.Blueprint.html#t:node_t/0","title":"Absinthe.Blueprint.node_t/0","type":"type"},{"doc":"","ref":"Absinthe.Blueprint.html#t:result_t/0","title":"Absinthe.Blueprint.result_t/0","type":"type"},{"doc":"","ref":"Absinthe.Blueprint.html#t:t/0","title":"Absinthe.Blueprint.t/0","type":"type"},{"doc":"","ref":"Absinthe.Blueprint.html#t:use_t/0","title":"Absinthe.Blueprint.use_t/0","type":"type"},{"doc":"Blueprint Execution Data\n\nThe `%Absinthe.Blueprint.Execution{}` struct holds on to the core values that\ndrive a document's execution.\n\nHere's how the execution flow works. Given a document like:\n```graphql\n{\n  posts {\n    title\n    author { name }\n  }\n}\n```\n\nAfter all the validation happens, and we're actually going to execute this document,\nan `%Execution{}` struct is created. This struct is passed to each plugin's\n`before_resolution` callback, so that plugins can set initial values in the accumulator\nor context.\n\nThen the resolution phase walks the document until it hits the `posts` field.\nTo resolve the posts field, an `%Absinthe.Resolution{}` struct is created from\nthe `%Execution{}` struct. This resolution struct powers the normal middleware\nresolution process. When a field has resolved, the `:acc`, `:context`, and `:field_cache`\nvalues within the resolution struct are pulled out and used to update the execution.","ref":"Absinthe.Blueprint.Execution.html","title":"Absinthe.Blueprint.Execution","type":"module"},{"doc":"","ref":"Absinthe.Blueprint.Execution.html#get/2","title":"Absinthe.Blueprint.Execution.get/2","type":"function"},{"doc":"","ref":"Absinthe.Blueprint.Execution.html#get_result/2","title":"Absinthe.Blueprint.Execution.get_result/2","type":"function"},{"doc":"","ref":"Absinthe.Blueprint.Execution.html#get_result/3","title":"Absinthe.Blueprint.Execution.get_result/3","type":"function"},{"doc":"","ref":"Absinthe.Blueprint.Execution.html#update/4","title":"Absinthe.Blueprint.Execution.update/4","type":"function"},{"doc":"","ref":"Absinthe.Blueprint.Execution.html#t:acc/0","title":"Absinthe.Blueprint.Execution.acc/0","type":"type"},{"doc":"","ref":"Absinthe.Blueprint.Execution.html#t:node_t/0","title":"Absinthe.Blueprint.Execution.node_t/0","type":"type"},{"doc":"","ref":"Absinthe.Blueprint.Execution.html#t:t/0","title":"Absinthe.Blueprint.Execution.t/0","type":"type"},{"doc":"Extra metadata passed to aid complexity analysis functions, describing the\ncurrent field's environment.","ref":"Absinthe.Complexity.html","title":"Absinthe.Complexity","type":"module"},{"doc":"- `:definition` - The Blueprint definition for this field.\n- `:context` - The context passed to `Absinthe.run`.\n- `:root_value` - The root value passed to `Absinthe.run`, if any.\n- `:schema` - The current schema.","ref":"Absinthe.Complexity.html#t:t/0","title":"Absinthe.Complexity.t/0","type":"type"},{"doc":"Information about the current resolution. It is created by adding field specific\ninformation to the more general `%Absinthe.Blueprint.Execution{}` struct.\n\nIn many ways like the `%Conn{}` from `Plug`, the `%Absinthe.Resolution{}` is the\npiece of information that passed along from middleware to middleware as part of\nresolution.","ref":"Absinthe.Resolution.html","title":"Absinthe.Resolution","type":"module"},{"doc":"- `:adapter` - The adapter used for any name conversions.\n- `:definition` - The Blueprint definition for this field.\n- `:context` - The context passed to `Absinthe.run`.\n- `:root_value` - The root value passed to `Absinthe.run`, if any.\n- `:parent_type` - The parent type for the field.\n- `:private` - Operates similarly to the `:private` key on a `%Plug.Conn{}`\n  and is a place for libraries (and similar) to store their information.\n- `:schema` - The current schema.\n- `:source` - The resolved parent object; source of this field.\n\nWhen a `%Resolution{}` is accessed via middleware, you may want to update the\ncontext (e.g. to cache a dataloader instance or the result of an ecto query).\nUpdating the context can be done simply by using the map updating syntax (or\n`Map.put/3`):\n\n```elixir\n%{resolution | context: new_context}\n# OR\nMap.put(resolution, :context, new_context)\n```\n\nTo access the schema type for this field, see the `definition.schema_node`.","ref":"Absinthe.Resolution.html#module-contents","title":"Contents - Absinthe.Resolution","type":"module"},{"doc":"","ref":"Absinthe.Resolution.html#call/2","title":"Absinthe.Resolution.call/2","type":"function"},{"doc":"","ref":"Absinthe.Resolution.html#call/3","title":"Absinthe.Resolution.call/3","type":"function"},{"doc":"TODO: Deprecate","ref":"Absinthe.Resolution.html#call/4","title":"Absinthe.Resolution.call/4","type":"function"},{"doc":"Get the current path.\n\nEach `Absinthe.Resolution` struct holds the current result path as a list of\nblueprint nodes and indices. Usually however you don't need the full AST list\nand instead just want the path that will eventually end up in the result.\n\nFor that, use this function.","ref":"Absinthe.Resolution.html#path/1","title":"Absinthe.Resolution.path/1","type":"function"},{"doc":"Given some query:\n```graphql\n{users { email }}\n```\n\nIf you called this function inside a resolver on the users email field it\nreturns a value like:\n\n```elixir\nresolve fn _, _, resolution ->\n  Absinthe.Resolution.path(resolution) #=> [\"users\", 5, \"email\"]\nend\n```\n\nIn this case `5` is the 0 based index in the list of users the field is currently\nat.","ref":"Absinthe.Resolution.html#path/1-examples","title":"Examples - Absinthe.Resolution.path/1","type":"function"},{"doc":"","ref":"Absinthe.Resolution.html#path_string/1","title":"Absinthe.Resolution.path_string/1","type":"function"},{"doc":"Get the child fields under the current field.\n\nSee `project/2` for details.","ref":"Absinthe.Resolution.html#project/1","title":"Absinthe.Resolution.project/1","type":"function"},{"doc":"Get the child fields under the current field.","ref":"Absinthe.Resolution.html#project/2","title":"Absinthe.Resolution.project/2","type":"function"},{"doc":"Given a document like:\n```graphql\n{ user { id name }}\n```\n\n```\nfield :user, :user do\n  resolve fn _, info ->\n    child_fields = Absinthe.Resolution.project(info) |> Enum.map(&(&1.name))\n    # ...\n  end\nend\n```\n\n`child_fields` will be `[\"id\", \"name\"]`.\n\nIt correctly handles fragments, so for example if you had the document:\n```graphql\n{\n  user {\n    ... on User {\n      id\n    }\n    ... on Named {\n      name\n    }\n  }\n}\n```\n\nyou would still get a nice and simple `child_fields` that was `[\"id\", \"name\"]`.","ref":"Absinthe.Resolution.html#project/2-example","title":"Example - Absinthe.Resolution.project/2","type":"function"},{"doc":"Handy function for applying user function result tuples to a resolution struct\n\nUser facing functions generally return one of several tuples like `{:ok, val}`\nor `{:error, reason}`. This function handles applying those various tuples\nto the resolution struct.\n\nThe resolution state is updated depending on the tuple returned. `:ok` and\n`:error` tuples set the state to `:resolved`, whereas middleware tuples set it\nto `:unresolved`.\n\nThis is useful for middleware that wants to handle user facing functions, but\ndoes not want to duplicate this logic.","ref":"Absinthe.Resolution.html#put_result/2","title":"Absinthe.Resolution.put_result/2","type":"function"},{"doc":"","ref":"Absinthe.Resolution.html#resolver_spec/1","title":"Absinthe.Resolution.resolver_spec/1","type":"function"},{"doc":"","ref":"Absinthe.Resolution.html#result_error/4","title":"Absinthe.Resolution.result_error/4","type":"function"},{"doc":"The arguments that are passed from the schema. (e.g. id of the record to be\nfetched)","ref":"Absinthe.Resolution.html#t:arguments/0","title":"Absinthe.Resolution.arguments/0","type":"type"},{"doc":"","ref":"Absinthe.Resolution.html#t:field_state/0","title":"Absinthe.Resolution.field_state/0","type":"type"},{"doc":"","ref":"Absinthe.Resolution.html#t:source/0","title":"Absinthe.Resolution.source/0","type":"type"},{"doc":"","ref":"Absinthe.Resolution.html#t:t/0","title":"Absinthe.Resolution.t/0","type":"type"},{"doc":"Introspection support.\n\nYou can introspect your schema using `__schema`, `__type`, and `__typename`,\nas [described in the specification](https://spec.graphql.org/October2021/#sec-Introspection).","ref":"Absinthe.Introspection.html","title":"Absinthe.Introspection","type":"module"},{"doc":"Seeing the names of the types in the schema:\n\n```\n\"\"\"\n{\n  __schema {\n    types {\n      name\n    }\n  }\n}\n\"\"\"\n|> Absinthe.run(MyApp.Schema)\n{:ok,\n  %{data: %{\n    \"__schema\" => %{\n      \"types\" => [\n        %{\"name\" => \"Boolean\"},\n        %{\"name\" => \"Float\"},\n        %{\"name\" => \"ID\"},\n        %{\"name\" => \"Int\"},\n        %{\"name\" => \"String\"},\n        ...\n      ]\n    }\n  }}\n}\n```\n\nGetting the name of the queried type:\n\n```\n\"\"\"\n{\n  profile {\n    name\n    __typename\n  }\n}\n\"\"\"\n|> Absinthe.run(MyApp.Schema)\n{:ok,\n  %{data: %{\n    \"profile\" => %{\n      \"name\" => \"Joe\",\n      \"__typename\" => \"Person\"\n    }\n  }}\n}\n```\n\nGetting the name of the fields for a named type:\n\n```\n\"\"\"\n{\n  __type(name: \"Person\") {\n    fields {\n      name\n      type {\n        kind\n        name\n      }\n    }\n  }\n}\n\"\"\"\n|> Absinthe.run(MyApp.Schema)\n{:ok,\n  %{data: %{\n    \"__type\" => %{\n      \"fields\" => [\n        %{\n          \"name\" => \"name\",\n          \"type\" => %{\"kind\" => \"SCALAR\", \"name\" => \"String\"}\n        },\n        %{\n          \"name\" => \"age\",\n          \"type\" => %{\"kind\" => \"SCALAR\", \"name\" => \"Int\"}\n        },\n      ]\n    }\n  }}\n}\n```\n\n(Note that you may have to nest several depths of `type`/`ofType`, as\ntype information includes any wrapping layers of [List](https://spec.graphql.org/October2021/#sec-List)\nand/or [NonNull](https://spec.graphql.org/October2021/#sec-Non-Null).)","ref":"Absinthe.Introspection.html#module-examples","title":"Examples - Absinthe.Introspection","type":"module"},{"doc":"","ref":"Absinthe.Test.html","title":"Absinthe.Test","type":"module"},{"doc":"Run the introspection query on a schema.\n\nIn your `test_helper.exs` file add\n```\nAbsinthe.Test.prime(MyApp.Schema)\n```","ref":"Absinthe.Test.html#prime/1","title":"Absinthe.Test.prime/1","type":"function"},{"doc":"In the test environment mix loads code lazily, which means that it isn't until\nthe first GraphQL query in your test suite runs that Absinthe's code base is\nactually loaded. Absinthe is a lot of code, and so this can take several\nmilliseconds. This can be a problem for tests using message passing that expect\nmessages to happen within a certain amount of time.\n\nBy running the introspection query on your schema this function will cause mix\nto load the majority of the Absinthe code base.","ref":"Absinthe.Test.html#prime/1-explanation","title":"Explanation - Absinthe.Test.prime/1","type":"function"},{"doc":"Handles logging of Absinthe-specific events.","ref":"Absinthe.Logger.html","title":"Absinthe.Logger","type":"module"},{"doc":"Absinthe can filter out sensitive information like tokens and passwords\nduring logging. They are replaced by `\"[FILTERED]\"`.\n\nUse the `:filter_variables` configuration setting for this module.\nFor example:\n\n    config :absinthe, Absinthe.Logger,\n      filter_variables: [\"token\", \"password\", \"secret\"]\n\nWith the configuration above, Absinthe will filter any variable whose name\nincludes the terms `token`, `password`, or `secret`. The match is case\nsensitive.\n\nNote that filtering only applies to GraphQL variables - the query itself is\nlogged before any parsing happens.\n\nThe default is `[\"token\", \"password\"]`.","ref":"Absinthe.Logger.html#module-variable-filtering","title":"Variable filtering - Absinthe.Logger","type":"module"},{"doc":"Absinthe can optionally display the list of pipeline phases for each processed\ndocument when logging. To enable this feature, set the `:pipeline`\nconfiguration option for this module:\n\n    config :absinthe, Absinthe.Logger,\n      pipeline: true\n\nThe default is `false`.","ref":"Absinthe.Logger.html#module-pipeline-display","title":"Pipeline display - Absinthe.Logger","type":"module"},{"doc":"To disable Absinthe logging, set the `:log` configuration option to `false`:\n\n    config :absinthe,\n      log: false\n\nThe default is `true`.","ref":"Absinthe.Logger.html#module-disabling","title":"Disabling - Absinthe.Logger","type":"module"},{"doc":"Log a document being processed.","ref":"Absinthe.Logger.html#log_run/2","title":"Absinthe.Logger.log_run/2","type":"function"},{"doc":"","ref":"Absinthe.Utils.html","title":"Absinthe.Utils","type":"module"},{"doc":"Camelize a word, respecting underscore prefixes.","ref":"Absinthe.Utils.html#camelize/2","title":"Absinthe.Utils.camelize/2","type":"function"},{"doc":"With an uppercase first letter:\n\n```\niex> camelize(\"foo_bar\")\n\"FooBar\"\niex> camelize(\"foo\")\n\"Foo\"\niex> camelize(\"__foo_bar\")\n\"__FooBar\"\niex> camelize(\"__foo\")\n\"__Foo\"\niex> camelize(\"_foo\")\n\"_Foo\"\n```\n\nWith a lowercase first letter:\n```\niex> camelize(\"foo_bar\", lower: true)\n\"fooBar\"\niex> camelize(\"foo\", lower: true)\n\"foo\"\niex> camelize(\"__foo_bar\", lower: true)\n\"__fooBar\"\niex> camelize(\"__foo\", lower: true)\n\"__foo\"\niex> camelize(\"_foo\", lower: true)\n\"_foo\"\n\n```","ref":"Absinthe.Utils.html#camelize/2-examples","title":"Examples - Absinthe.Utils.camelize/2","type":"function"},{"doc":"","ref":"Absinthe.Utils.html#placement_docs/1","title":"Absinthe.Utils.placement_docs/1","type":"function"},{"doc":"","ref":"Absinthe.Utils.Suggestion.html","title":"Absinthe.Utils.Suggestion","type":"module"},{"doc":"Sort a list of suggestions by Jaro distance to a target string,\nsupporting a cut-off threshold.","ref":"Absinthe.Utils.Suggestion.html#sort_list/3","title":"Absinthe.Utils.Suggestion.sort_list/3","type":"function"},{"doc":"An error during analysis.","ref":"Absinthe.AnalysisError.html","title":"Absinthe.AnalysisError","type":"exception"},{"doc":"An error during execution.","ref":"Absinthe.ExecutionError.html","title":"Absinthe.ExecutionError","type":"exception"},{"doc":"Exception raised when a schema is invalid","ref":"Absinthe.Schema.Error.html","title":"Absinthe.Schema.Error","type":"exception"},{"doc":"","ref":"Absinthe.Schema.Error.html#message/1","title":"Absinthe.Schema.Error.message/1","type":"function"},{"doc":"Exception raised when a schema is invalid","ref":"Absinthe.Schema.Notation.Error.html","title":"Absinthe.Schema.Notation.Error","type":"exception"},{"doc":"An error during serialization.","ref":"Absinthe.SerializationError.html","title":"Absinthe.SerializationError","type":"exception"},{"doc":"# Overview\n\nAbsinthe is the GraphQL toolkit for Elixir, an implementation of the [GraphQL specification](https://spec.graphql.org) built to suit the language's capabilities and idiomatic style.\n\nThe Absinthe project consists of several complementary packages. You can find the full listing on the [absinthe-graphql](https://github.com/absinthe-graphql) GitHub organization page.","ref":"overview.html","title":"Overview","type":"extras"},{"doc":"If you're new to GraphQL, we suggest you read up a bit on GraphQL's foundational principles before you dive into Absinthe.\n\nHere are a few resources that might be helpful:\n\n- The official [GraphQL](https://graphql.org/) website\n- [How to GraphQL](https://www.howtographql.com/), which includes a [brief tutorial](https://www.howtographql.com/graphql-elixir/0-introduction/) using Absinthe","ref":"overview.html#graphql-basics","title":"GraphQL Basics - Overview","type":"extras"},{"doc":"Absinthe's functionality generally falls into two broad areas. You can read more about the details in the guides provided as part of this documentation and in the related packages/projects:\n\n* [Defining Schemas](schemas.md). A schema:\n  * defines the structure of data entities and the relationships between, as well as the available queries, mutations, and subscriptions, using an elegant collection of declarative macros\n  * defines [custom scalar](custom-scalars.md) types\n  * declares any [deprecated](deprecation.md) definitions\n  * defines resolution functions to access data, using a flexible and extensible middleware/plugin system\n* Executing Documents. A GraphQL document:\n  * can be any standard GraphQL query, mutation, or subscription\n  * may include reusable [variable](variables.md) definitions\n  * can be analyzed for its [complexity](complexity-analysis.md) and be rejected if it's unsafe/too expensive\n  * has a [context](context-and-authentication.md) that you can integrate with authentication and authorization strategies\n  * can contain standard GraphQL [introspection](introspection.md) fields\n  * can include multipart file uploads as GraphQL arguments (as part of the [absinthe_plug](https://hex.pm/packages/absinthe_plug) package)","ref":"overview.html#absinthe","title":"Absinthe - Overview","type":"extras"},{"doc":"Absinthe integrates with a number of other important projects, both on the backend and frontend, to provide a better experience for developers.\n\n* Elixir\n  * Support for HTTP APIs using [Plug and Phoenix](plug-phoenix.md) via the [absinthe_plug](https://hex.pm/packages/absinthe_plug) and [absinthe_phoenix](https://hex.pm/packages/absinthe_phoenix) packages\n  * Support for [Ecto](https://hex.pm/packages/ecto) via the [dataloader](https://github.com/absinthe-graphql/dataloader) package\n* JavaScript (client-side)\n  * Support for [Relay](relay.md) and [Apollo Client](apollo.md)\n  * Support for Absinthe's channel-based subscriptions. See [absinthe-socket](https://github.com/absinthe-graphql/absinthe-socket).","ref":"overview.html#integrations","title":"Integrations - Overview","type":"extras"},{"doc":"To contribute to the guides, please submit a pull request to the [absinthe](https://github.com/absinthe-graphql/absinthe) project on GitHub.\n\nYou'll find the content under `guides/`.","ref":"overview.html#guides","title":"Guides - Overview","type":"extras"},{"doc":"# Installation\n\nTo install Absinthe, just add an entry to your `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    # ...\n    {:absinthe, \"~> 1.7\"}\n  ]\nend\n```\n\n(Check [Hex](https://hex.pm/packages/absinthe) to make sure you're using an up-to-date version number.)","ref":"installation.html","title":"Installation","type":"extras"},{"doc":"Because the Absinthe project is made up of a large number of related packages to support integrations with other tools, sometimes you may want to update only part of your absinthe-related dependencies.\n\nDon't forget you can use the [:override](https://hexdocs.pm/mix/Mix.Tasks.Deps.html#module-dependency-definition-options) option for your Mix dependencies if you'd like to ensure a specific package is at a specific version number. For example, If you wanted to try a new version of Absinthe without updating something that depends on it (which is locked to an older version):\n\n```elixir\ndef deps do\n  [\n    # ...\n    {:absinthe, \"~> 1.7\", override: true}\n  ]\nend\n```","ref":"installation.html#overriding-dependencies","title":"Overriding Dependencies - Installation","type":"extras"},{"doc":"Most people use Absinthe to support an HTTP API.\n\nYou'll want to read the [Plug and Phoenix](plug-phoenix.md) for specific installation and configuration options, including how you can run the handy, included GraphiQL tool directly from your application.","ref":"installation.html#plug-phoenix-and-graphiql","title":"Plug, Phoenix, and GraphiQL - Installation","type":"extras"},{"doc":"# Learning\n\nThe following are some Absinthe-specific educational resources that are available.","ref":"learning.html","title":"Learning","type":"extras"},{"doc":"* [Craft GraphQL APIs in Elixir with Absinthe](https://pragprog.com/book/wwgraphql/craft-graphql-apis-in-elixir-with-absinthe) by the creators of Absinthe.","ref":"learning.html#books","title":"Books - Learning","type":"extras"},{"doc":"* [Website](https://absinthe-graphql.org) (mostly just links elsewhere)\n* [Documentation](https://hexdocs.pm/absinthe) (current stable release)\n* [How to GraphQL (with Absinthe)](https://www.howtographql.com/graphql-elixir/0-introduction/)","ref":"learning.html#online-resources","title":"Online Resources - Learning","type":"extras"},{"doc":"* [Live APIs with GraphQL Subscriptions](https://www.youtube.com/watch?v=PEckzwggd78), ElixirConf 2017 (Bellevue)\n* [GraphQL in Practice](https://www.youtube.com/watch?v=d2qNlXtpWXM), ElixirConf EU 2017 (Barcelona)","ref":"learning.html#videos","title":"Videos - Learning","type":"extras"},{"doc":"There's a ton of GraphQL resources on the web.\n\nThe [official website](https://graphql.org/) and [How to GraphQL](https://www.howtographql.com) are good places to start.","ref":"learning.html#general-graphql-information","title":"General GraphQL Information - Learning","type":"extras"},{"doc":"# Community","ref":"community.html","title":"Community","type":"extras"},{"doc":"Follow the project on Twitter as [`@absinthegraphql`](https://twitter.com/absinthegraphql) for news and additional resources.","ref":"community.html#twitter","title":"Twitter - Community","type":"extras"},{"doc":"You can find the maintainers and an active community of users and contributors in the `#absinthe-graphql` channel in the [Elixir](https://elixir-slackin.herokuapp.com/) Slack.","ref":"community.html#chat","title":"Chat - Community","type":"extras"},{"doc":"Questions and suggestions can be submitted on the [Elixir Forum](https://elixirforum.com). Please categorize/tag as `Absinthe`.","ref":"community.html#forum","title":"Forum - Community","type":"extras"},{"doc":"# Getting Started\n\nWe'll be building a very basic GraphQL API for a blog, written in Elixir using\nAbsinthe.","ref":"start.html","title":"Getting Started","type":"extras"},{"doc":"Before you start, it's a good idea to have some background into GraphQL in general. Here are a few resources that might be helpful:\n\n- The official [GraphQL](https://graphql.org/) website\n- [How to GraphQL](https://www.howtographql.com/) (this includes another [brief tutorial](https://www.howtographql.com/graphql-elixir/0-introduction/) using Absinthe)","ref":"start.html#background","title":"Background - Getting Started","type":"extras"},{"doc":"The tutorial expects you to have a properly set-up [Phoenix application](https://hexdocs.pm/phoenix/installation.html) with [absinthe](https://hex.pm/packages/absinthe) and [absinthe_plug](https://hex.pm/packages/absinthe_plug) added to the dependencies.\n\n>  If you'd like to cheat, you can find the finished code for the tutorial\n>  in the [Absinthe Example](https://github.com/absinthe-graphql/absinthe_tutorial)\n>  project on GitHub.","ref":"start.html#the-example","title":"The Example - Getting Started","type":"extras"},{"doc":"Let's get started with [our first query](our-first-query.md)!","ref":"start.html#first-step","title":"First Step - Getting Started","type":"extras"},{"doc":"# Our First Query\n\nThe first thing our viewers want is a list of our blog posts, so\nthat's what we're going to give them. Here's the query we want to\nsupport:\n\n```graphql\n{\n  posts {\n    title\n    body\n  }\n}\n```\n\nTo do this we're going to need a schema. Let's create some basic types\nfor our schema, starting with a `:post`. GraphQL has several fundamental\ntypes on top of which all of our types will be\nbuilt. The `Absinthe.Type.Object` type is the right one\nto use when representing a set of key value pairs.\n\nSince our `Post` Ecto schema lives in the `Blog.Content` Phoenix\ncontext, we'll define its GraphQL counterpart type, `:post`, in a\nmatching `BlogWeb.Schema.ContentTypes` module:\n\nIn `blog_web/schema/content_types.ex`:\n\n```elixir\ndefmodule BlogWeb.Schema.ContentTypes do\n  use Absinthe.Schema.Notation\n\n  object :post do\n    field :id, :id\n    field :title, :string\n    field :body, :string\n  end\nend\n```\n\n> The GraphQL specification requires that type names be unique, TitleCased words.\n> Absinthe does this automatically for us, extrapolating from our type identifier\n> (in this case `:post` gives us `\"Post\"`). If really needed, we could provide a\n> custom type name as a `:name` option to the `object` macro.\n\nIf you're curious what the type `:id` is used by the `:id` field, see\nthe [GraphQL spec](https://spec.graphql.org/October2021/#sec-ID). It's\nan opaque value, and in our case is just the regular Ecto id, but\nserialized as a string.\n\nWith our type completed we can now write a basic schema that will let\nus query a set of posts.\n\nIn `blog_web/schema.ex`:\n\n```elixir\ndefmodule BlogWeb.Schema do\n  use Absinthe.Schema\n  import_types BlogWeb.Schema.ContentTypes\n\n  alias BlogWeb.Resolvers\n\n  query do\n\n    @desc \"Get all posts\"\n    field :posts, list_of(:post) do\n      resolve &Resolvers.Content.list_posts/3\n    end\n\n  end\n\nend\n```\n\n> ##","ref":"our-first-query.html","title":"Our First Query","type":"extras"},{"doc":">\n> For more information on the macros available to build a schema, see\n> their definitions in `Absinthe.Schema` and `Absinthe.Schema.Notation`.\n\nThis uses a resolver module we've created (again, to match the Phoenix context naming)\nat `blog_web/resolvers/content.ex`:\n\n```elixir\ndefmodule BlogWeb.Resolvers.Content do\n\n  def list_posts(_parent, _args, _resolution) do\n    {:ok, Blog.Content.list_posts()}\n  end\n\nend\n```\n\nQueries are defined as fields inside the GraphQL object returned by\nour `query` function. We created a posts query that has a type\n`list_of(:post)` and is resolved by our\n`BlogWeb.Resolvers.Content.list_posts/3` function. Later we'll talk\nmore about the resolver function parameters; for now just remember\nthat resolver functions can take two forms:\n\n- A function with an arity of 3 (taking a parent, arguments, and resolution struct)\n- An alternate, short form with an arity of 2 (omitting the first parameter, the parent)\n\nThe job of the resolver function is to return the data for the\nrequested field. Our resolver calls out to the `Blog.Content` module,\nwhich is where all the domain logic for posts lives, invoking its\n`list_posts/0` function, then returns the posts in an `:ok` tuple.\n\n> ##","ref":"our-first-query.html#more-information-info","title":"More information {: .info} - Our First Query","type":"extras"},{"doc":">\n> Resolvers can return a wide variety of results, to include errors and configuration\n> for [advanced plugins](middleware-and-plugins.md) that further process the data.\n>\n> If you're asking yourself what the implementation of the domain logic looks like, and exactly how\n> the related Ecto schemas are built, read through the code in the [absinthe_tutorial](https://github.com/absinthe-graphql/absinthe_tutorial)\n> repository. The tutorial content here is intentionally focused on the Absinthe-specific code.\n\nNow that we have the functional pieces in place, let's configure our\nPhoenix router to wire this into HTTP:\n\nIn `blog_web/router.ex`:\n\n```elixir\ndefmodule BlogWeb.Router do\n  use BlogWeb, :router\n\n  pipeline :api do\n    plug :accepts, [\"json\"]\n  end\n\n  scope \"/api\" do\n    pipe_through :api\n\n    forward \"/graphiql\", Absinthe.Plug.GraphiQL,\n      schema: BlogWeb.Schema\n\n    forward \"/\", Absinthe.Plug,\n      schema: BlogWeb.Schema\n\n  end\n\nend\n```\n\nIn addition to our API, we've wired in a handy GraphiQL user interface to play with it. Absinthe integrates both the classic [GraphiQL](https://github.com/graphql/graphiql) and more advanced [GraphiQL Workspace](https://github.com/OlegIlyenko/graphiql-workspace) interfaces as part of the [absinthe_plug](https://hex.pm/packages/absinthe_plug) package.\n\nNow let's check to make sure everything is working. Start the server:\n\n```shell\n$ mix phx.server\n```\n\nAbsinthe does a number of sanity checks during compilation, so if you misspell a type or make another schema-related gaffe, you'll be notified.\n\nOnce it's up-and-running, take a look at [http://localhost:4000/api/graphiql](http://localhost:4000/api/graphiql):\n\n \n\nMake sure that the `URL` is pointing to the correct place and press the play button. If everything goes according to plan, you should see something like this:","ref":"our-first-query.html#note-info","title":"Note {: .info} - Our First Query","type":"extras"},{"doc":"Now let's look at how we can [add arguments to our queries](query-arguments.md).","ref":"our-first-query.html#next-step","title":"Next Step - Our First Query","type":"extras"},{"doc":"# Query Arguments\n\nOur GraphQL API would be pretty boring (and useless) if clients\ncouldn't retrieve filtered data.\n\nLet's assume that our API needs to add the ability to look-up users by\ntheir ID and get the posts that they've authored. Here's what a basic query to do that\nmight look like:\n\n```graphql\n{\n  user(id: \"1\") {\n    name\n    posts {\n      id\n      title\n    }\n  }\n}\n```\n\nThe query includes a field argument, `id`, contained within the\nparentheses after the `user` field name. To make this all work, we need to modify\nour schema a bit.","ref":"query-arguments.html","title":"Query Arguments","type":"extras"},{"doc":"First, let's create a `:user` type and define its relationship to\n`:post` while we're at it. We'll create a new module for the\naccount-related types and put it there; in\n`blog_web/schema/account_types.ex`:\n\n```elixir\ndefmodule BlogWeb.Schema.AccountTypes do\n  use Absinthe.Schema.Notation\n\n  @desc \"A user of the blog\"\n  object :user do\n    field :id, :id\n    field :name, :string\n    field :email, :string\n    field :posts, list_of(:post)\n  end\n\nend\n```\n\nThe `:posts` field points to a list of `:post` results. (This matches\nup with what we have on the Ecto side, where `Blog.Accounts.User`\ndefines a `has_many` association with `Blog.Content.Post`.)\n\nWe've already defined the `:post` type, but let's go ahead and add an\n`:author` field that points back to our `:user` type. In\n`blog_web/schema/content_types.ex`:\n\n```elixir\nobject :post do\n\n  # post fields we defined earlier...\n\n  field :author, :user\n\nend\n```\n\nNow let's add the `:user` field to our query root object in our\nschema, defining a mandatory `:id` argument and using the\n`Resolvers.Accounts.find_user/3` resolver function. We also need to\nmake sure we import the types from `BlogWeb.Schema.AccountTypes` so\nthat `:user` is available.\n\nIn `blog_web/schema.ex`:\n\n```elixir\ndefmodule BlogWeb.Schema do\n  use Absinthe.Schema\n\n  import_types Absinthe.Type.Custom\n\n  # Add this `import_types`:\n  import_types BlogWeb.Schema.AccountTypes\n\n  import_types BlogWeb.Schema.ContentTypes\n\n  alias BlogWeb.Resolvers\n\n  query do\n\n    @desc \"Get all posts\"\n    field :posts, list_of(:post) do\n      resolve &Resolvers.Content.list_posts/3\n    end\n\n    # Add this field:\n    @desc \"Get a user of the blog\"\n    field :user, :user do\n      arg :id, non_null(:id)\n      resolve &Resolvers.Accounts.find_user/3\n    end\n\n  end\n\nend\n```\n\nNow let's use the argument in our resolver. In `blog_web/resolvers/accounts.ex`:\n\n```elixir\ndefmodule BlogWeb.Resolvers.Accounts do\n\n  def find_user(_parent, %{id: id}, _resolution) do\n    case Blog.Accounts.find_user(id) do\n      nil ->\n        {:error, \"User ID #{id} not found\"}\n      user ->\n        {:ok, user}\n    end\n  end\n\nend\n```\n\nOur schema marks the `:id` argument as `non_null`, so we can be\ncertain we will receive it. If `:id` is left out of the query,\nAbsinthe will return an informative error to the user, and the resolve\nfunction will not be called.\n\n> If you have experience writing Phoenix controller actions, you might\n> wonder why we can match incoming arguments with atoms instead of\n> having to use strings.\n>\n> The answer is simple: you've defined the arguments in the schema\n> using atom identifiers, so Absinthe knows what arguments will be\n> used ahead of time, and will coerce as appropriate---culling any\n> extraneous arguments given to a query. This means that all arguments\n> can be supplied to the resolve functions with atom keys.\n\nFinally you'll see that we can handle the possibility that the query,\nwhile valid from GraphQL's perspective, may still ask for a user that\ndoes not exist. We've decided to return an error in that case.\n\n> There's a valid argument for just returning `{:ok, nil}` when a\n> record can't be found. Whether the absence of data constitutes an\n> error is a decision you get to make.","ref":"query-arguments.html#defining-arguments","title":"Defining Arguments - Query Arguments","type":"extras"},{"doc":"Let's assume we want to query all posts from a user published within a\ngiven time range. First, let's add a new field to our `:post` object\ntype, `:published_at`.\n\nThe GraphQL specification doesn't define any official date or time\ntypes, but it does support custom scalar types (you can read more\nabout them in the [related guide](custom-scalars.md)), and\nAbsinthe ships with several built-in scalar types. We'll use\n`:naive_datetime` (which doesn't include timezone information) here.\n\nEdit `blog_web/schema/content_types.ex`:\n\n```elixir\ndefmodule BlogWeb.Schema.ContentTypes do\n  use Absinthe.Schema.Notation\n\n  @desc \"A blog post\"\n  object :post do\n    field :id, :id\n    field :title, :string\n    field :body, :string\n    field :author, :user\n    # Add this:\n    field :published_at, :naive_datetime\n  end\nend\n```\n\nTo make the `:naive_datetime` type available, add an `import_types` line to\nyour `blog_web/schema.ex`:\n\n```elixir\nimport_types Absinthe.Type.Custom\n```\n\n> For more information about how types are imported,\n> read [the guide on the topic](importing-types.md).\n>\n> For now, just remember that `import_types` should _only_ be\n> used in top-level schema module. (Think of it like a manifest.)\n\nHere's the query we'd like to be able to use, getting the posts for a user\non a given date:\n\n```graphql\n{\n  user(id: \"1\") {\n    name\n    posts(date: \"2017-01-01\") {\n      title\n      body\n      publishedAt\n    }\n  }\n}\n```\n\nTo use the passed date, we need to update our `:user` object type and\nmake some changes to its `:posts` field; it needs to support a `:date`\nargument and use a custom resolver. In `blog_web/schema/account_types.ex`:\n\n```elixir\ndefmodule BlogWeb.Schema.AccountTypes do\n  use Absinthe.Schema.Notation\n\n  alias BlogWeb.Resolvers\n\n  object :user do\n    field :id, :id\n    field :name, :string\n    field :email, :string\n    # Add the block here:\n    field :posts, list_of(:post) do\n      arg :date, :date\n      resolve &Resolvers.Content.list_posts/3\n    end\n  end\n\nend\n```\n\nFor the resolver, we've added another function head to\n`Resolvers.Content.list_posts/3`. This illustrates how you can use the\nfirst argument to a resolver to match the parent object of a field. In\nthis case, that parent object would be a `Blog.Accounts.User` Ecto\nschema:\n\n```elixir\n# Add this:\ndef list_posts(%Blog.Accounts.User{} = author, args, _resolution) do\n  {:ok, Blog.Content.list_posts(author, args)}\nend\n# Before this:\ndef list_posts(_parent, _args, _resolution) do\n  {:ok, Blog.Content.list_posts()}\nend\n```\n\nHere we pass on the user and arguments to the domain logic function,\n`Blog.Content.list_posts/3`, which will find the posts for the user\nand date (if it's provided; the `:date` argument is optional). The\nresolver, just as when it's used for the top level query `:posts`,\nreturns the posts in an `:ok` tuple.\n\n> ##","ref":"query-arguments.html#arguments-and-non-root-fields","title":"Arguments and Non-Root Fields - Query Arguments","type":"extras"},{"doc":">\n> Check out the full implementation of logic for\n> `Blog.Content.list_posts/3`--and some simple seed data--in\n> the\n> [absinthe_tutorial](https://github.com/absinthe-graphql/absinthe_tutorial) repository.\n\nIf you've done everything correctly (and have some data handy), if you\nstart up your server with `mix phx.server` and head over\nto  , you should be able to play\nwith the query.\n\nIt should look something like this:","ref":"query-arguments.html#tip-tip","title":"Tip {: .tip} - Query Arguments","type":"extras"},{"doc":"Next up, we look at how to modify our data using [mutations](mutations.md).","ref":"query-arguments.html#next-step","title":"Next Step - Query Arguments","type":"extras"},{"doc":"# Mutations\n\nA blog is no good without new content. We want to support a mutation\nto create a blog post:\n\n```graphql\nmutation CreatePost {\n  createPost(title: \"Second\", body: \"We're off to a great start!\") {\n    id\n  }\n}\n```\n\nNow we just need to define a `mutation` portion of our schema and\na `:create_post` field:\n\nIn `blog_web/schema.ex`:\n\n```elixir\nmutation do\n\n  @desc \"Create a post\"\n  field :create_post, type: :post do\n    arg :title, non_null(:string)\n    arg :body, non_null(:string)\n    arg :published_at, :naive_datetime\n\n    resolve &Resolvers.Content.create_post/3\n  end\n\nend\n```\n\nThe resolver in this case is responsible for making any changes and\nreturning an `{:ok, post}` tuple matching the `:post` type we defined\nearlier:\n\nIn our `blog_web/resolvers/content.ex` module, we'll add the\n`create_post/3` resolver function:\n\n```elixir\ndef create_post(_parent, args, %{context: %{current_user: user}}) do\n  Blog.Content.create_post(user, args)\nend\ndef create_post(_parent, _args, _resolution) do\n  {:error, \"Access denied\"}\nend\n```\n\n> Obviously things can go wrong in a mutation. To learn more about the\n> types of error results that Absinthe supports, read [the guide](errors.md).","ref":"mutations.html","title":"Mutations","type":"extras"},{"doc":"This resolver adds a new concept: authorization. The resolution struct\n(that is, an `Absinthe.Resolution`)\npassed to the resolver as the third argument carries along with it the\nAbsinthe context, a data structure that serves as the integration\npoint with external mechanisms---like a Plug that authenticates the\ncurrent user. You can learn more about how the context can be used in\nthe [Context and Authentication](context-and-authentication.md)\nguide.\n\nGoing back to the resolver code:\n\n- If the match for a current user is successful, the underlying\n  `Blog.Content.create_post/2` function is invoked. It will return a\n  tuple suitable for return. (To read the Ecto-related nitty-gritty,\n  check out the [absinthe_tutorial](https://github.com/absinthe-graphql/absinthe_tutorial)\n  repository.)\n- If the match for a current user isn't successful, the fall-through\n  match will return an error indicating that a post can't be created.","ref":"mutations.html#authorization","title":"Authorization - Mutations","type":"extras"},{"doc":"Now let's take a look at [more complex arguments](complex-arguments.md)","ref":"mutations.html#next-step","title":"Next Step - Mutations","type":"extras"},{"doc":"# Complex Arguments\n\nIn preparation for supporting comments on our blog, let's create users. We're building\na modern mobile first blog of course, and thus want to support either a phone number\nor an email as the contact method for a user.\n\nWe want to support the following mutations.\n\nSupport creation of a user with their email address:\n\n```graphql\nmutation CreateEmailUser {\n  createUser(\n    contact: { type: EMAIL, value: \"foo@bar.com\" }\n    name: \"Jane\"\n    password: \"hunter1\"\n  ) {\n    id\n    contacts {\n      type\n      value\n    }\n  }\n}\n```\n\nAnd by using their phone number:\n\n```graphql\nmutation CreatePhoneUser {\n  createUser(\n    contact: { type: PHONE, value: \"+1 123 5551212\" }\n    name: \"Joe\"\n    password: \"hunter2\"\n  ) {\n    id\n    contacts {\n      type\n      value\n    }\n  }\n}\n```\n\nTo do this we need the ability to create nested arguments. GraphQL has input objects\nfor this purpose. Input objects, like regular object, contain key value pairs, but\nthey are intended for input only (you can't do circular references with them for example).\n\nAnother notion we'll look at here is an enumerable type. We only want to support contact\ntypes `\"email\"` and `\"phone\"` at the moment, and GraphQL gives us the ability to\nspecify this in our schema.\n\nLet's start with our `:contact_type` Enum. In `blog_web/schema/account_types.ex`:\n\n```graphql\nenum :contact_type do\n  value :phone, as: \"phone\"\n  value :email, as: \"email\"\nend\n```\n\nWe're using the `:as` option here to make sure the parsed enum is represented by a string\nwhen it's passed to our controllers; this is to ease integration with our Ecto schema\n(by default, the enum values are passed as atoms).\n\n> ##","ref":"complex-arguments.html","title":"Complex Arguments","type":"extras"},{"doc":">\n> The standard convention for representing incoming enum values in\n> GraphQL documents are in all caps. For instance, given our settings\n> here, the accepted values would be `PHONE` and `EMAIL` (without\n> quotes). See the GraphQL document examples above for examples.\n>\n> While the `enum` macro supports configuring this incoming format, we\n> highly recommend you just use the GraphQL convention.\n\nNow if a user tries to send some other kind of contact type they'll\nget a nice error without any extra effort on your part. Enum types are\nnot a substitute for modeling layer validations however, be sure to\nstill enforce things like this on that layer too.\n\nNow for our contact input object.\n\nIn `blog_web/schema/account_types.ex`:\n\n```graphql\ninput_object :contact_input do\n  field :type, non_null(:contact_type)\n  field :value, non_null(:string)\nend\n```\n\nNote that we name this type `:contact_input`. Input object types have\ntheir own names, and the `_input` suffix is common.\n\n> ##","ref":"complex-arguments.html#note-neutral","title":"Note {: .neutral} - Complex Arguments","type":"extras"},{"doc":">\n> It's very important to remember that only input\n> types---basically scalars and input objects---can be used to model\n> input.\n\nFinally our schema, in `blog_web/schema.ex`:\n\n```elixir\nmutation do\n\n  #... other mutations\n\n  @desc \"Create a user\"\n  field :create_user, :user do\n    arg :name, non_null(:string)\n    arg :contact, non_null(:contact_input)\n    arg :password, non_null(:string)\n\n    resolve &Resolvers.Accounts.create_user/3\n  end\n\nend\n```\n\nSuppose in our database that we store contact information in a different database\ntable. Our mutation would be used to create both records in this case.\n\nThere does not need to be a one to one correspondence between how data is structured\nin your underlying data store and how things are presented by your GraphQL API.\n\nOur resolver, `blog_web/resolvers/accounts.ex` might look something like this:\n\n```elixir\ndef create_user(_parent, args, %{context: %{current_user: %{admin: true}}}) do\n  Blog.Accounts.create_user(args)\nend\ndef create_user(_parent, args, _resolution) do\n  {:error, \"Access denied\"}\nend\n```\n\nYou'll notice we're checking for `:current_user` again in our Absinthe\ncontext, just as we did before for posts. In this case we're taking\nthe authorization check a step further and verifying that only\nadministrators (in this simple example, an administrator is a user\naccount with `:admin` set to `true`) can create a user.\n\nEveryone else gets an `\"Access denied\"` error for this field.\n\n> ##","ref":"complex-arguments.html#important-tip","title":"Important {: .tip} - Complex Arguments","type":"extras"},{"doc":">\n> To see the Ecto-related implementation of the\n> `Blog.Accounts.create_user/1` function and the (stubbed) authentication logic we're\n> using for this example, see the [absinthe_tutorial](https://github.com/absinthe-graphql/absinthe_tutorial)\n> repository.\n\nHere's our mutation in action in GraphiQL.\n\n \n\n> ##","ref":"complex-arguments.html#note-neutral","title":"Note {: .neutral} - Complex Arguments","type":"extras"},{"doc":">\n> Note we're sending a `Authorization` header to authenticate, which a\n> plug is handling. Make sure to read the\n> related [guide](context-and-authentication.md) for more\n> information on how to set-up authentication in your own\n> applications.\n>\n> Our simple tutorial application is just using a simple stub: any\n> authorization token logs you in the first user. Obviously not what\n> you want in production!","ref":"complex-arguments.html#note-neutral","title":"Note {: .neutral} - Complex Arguments","type":"extras"},{"doc":"Now let's [wrap things up](conclusion.md).","ref":"complex-arguments.html#next-step","title":"Next Step - Complex Arguments","type":"extras"},{"doc":"# Conclusion\n\nWith this we have a basic GraphQL based API for a blog. Head on over\nto [the github page](https://github.com/absinthe-graphql/absinthe_tutorial) if\nyou want the final code.\n\nWe hope to expand this tutorial to include a comment system that will\nacquaint you with Union types and Fragments in the coming days.\n\nHead on over to the topic guides for further reading, and see\nthe [community page](community.md) for information\non how to get help, ask questions, or contribute!","ref":"conclusion.html","title":"Conclusion","type":"extras"},{"doc":"This tutorial is a work in progress, and while it covers the basics of\nusing Absinthe, there is plenty more that can be added and improved\nupon. It's important that it's kept up-to-date, too, so if you notice\nsomething that's slipped by us, please help us fix it!\n\nPlease contribute your GitHub issues (and pull requests!):\n\n- The tutorial text is under `guides/tutorial` in the [absinthe](https://github.com/absinthe-graphql/absinthe)\n  repository. It's in Markdown and easy to edit!\n- The tutorial code located in the [absinthe_tutorial](https://github.com/absinthe-graphql/absinthe_tutorial) repository.","ref":"conclusion.html#please-help","title":"Please Help! - Conclusion","type":"extras"},{"doc":"# Writing Schemas\n\nA GraphQL API starts by building a schema. Using Absinthe, schemas are normal\nmodules that use `Absinthe.Schema`.\n\nHere's a schema that supports looking up an item by ID:\n\n```elixir\n# filename: myapp/schema.ex\ndefmodule MyAppWeb.Schema do\n\n  use Absinthe.Schema\n\n  # Example data\n  @items %{\n    \"foo\" => %{id: \"foo\", name: \"Foo\"},\n    \"bar\" => %{id: \"bar\", name: \"Bar\"}\n  }\n\n  query do\n    field :item, :item do\n      arg :id, non_null(:id)\n      resolve fn %{id: item_id}, _ ->\n        {:ok, @items[item_id]}\n      end\n    end\n  end\n\nend\n```\n\n\nYou may want to refer to the [Absinthe API documentation](https://hexdocs.pm/absinthe/) for more detailed information as you look this over.\n\n\nSome macros and functions used here that are worth mentioning, pulled in automatically from\n`Absinthe.Schema.Notation` by `use Absinthe.Schema`:\n\n- `query` - Defines the root query object. It's like using `object` but with\n   nice defaults. There is a matching `mutation` macro as well.\n- `field` - Defines a field in the enclosing `object`, `input_object`, or `interface`.\n- `arg` - Defines an argument in the enclosing `field` or `directive`.\n- `resolve` - Sets the resolve function for the enclosing `field`.\n\nYou'll notice we mention some types being referenced: `:item` and `:id`. `:id`\nis a built-in scalar type (like `:string`, `:boolean`, and others), but `:item`\nwe need to define ourselves.\n\nWe can do it in the same `MyAppWeb.Schema` module, using the `object` macro defined by `Absinthe.Schema.Notation`:\n\n```elixir\n# filename: myapp/schema.ex\n@desc \"An item\"\nobject :item do\n  field :id, :id\n  field :name, :string\nend\n```\n\nNow you can use Absinthe to execute a query document. Keep in mind that for\nHTTP, you'll probably want to use\n[Absinthe.Plug](plug-phoenix.md) instead of executing\nGraphQL query documents yourself. Absinthe doesn't know or care about HTTP,\nbut the `absinthe_plug` project does: it handles the vagaries of interacting\nwith HTTP GraphQL clients so you don't have to.\n\nIf you _were_ executing query documents yourself (let's assume for a local tool),\nit would go something like this:\n\n```elixir\n\"\"\"\n{\n  item(id: \"foo\") {\n    name\n  }\n}\n\"\"\"\n|> Absinthe.run(MyAppWeb.Schema)\n\n# Result\n{:ok, %{data: %{\"item\" => %{\"name\" => \"Foo\"}}}}\n```\n\nYour schemas can be further customized using the options available to\n`Absinthe.Schema.Notation.field/4` to help provide for a richer experience for\nyour users, customize the field names, or mark fields as deprecated.\n\n```elixir\n# filename: myapp/language_schema.ex\n@desc \"A Language\"\nobject :language do\n  field :id, :id\n  field :iso_639_1, :string, description: \"2 character ISO 639-1 code\", name: \"iso639\"\n  field :name, :string, description: \"English name of the language\"\nend\n```","ref":"schemas.html","title":"Writing Schemas","type":"extras"},{"doc":"We could also move our type definitions out into a different module, for instance, `MyAppWeb.Schema.Types`, and then use `import_types` in our `MyAppWeb.Schema`:\n\n```elixir\n# filename: myapp/schema/types.ex\ndefmodule MyAppWeb.Schema.Types do\n  use Absinthe.Schema.Notation\n\n  object :item do\n    field :id, :id\n    field :name, :string\n  end\n\n  # ...\n\nend\n\n# filename: myapp/schema.ex\ndefmodule MyAppWeb.Schema do\n  use Absinthe.Schema\n\n  import_types MyAppWeb.Schema.Types\n\n  # ...\n\nend\n```\n\nIt's a nice way of separating the top-level `query` and `mutation` information,\nwhich define the surface area of the API, with the actual types that it uses.\n\nSee [Importing Types](importing-types.md) for a full guide to importing types.","ref":"schemas.html#importing-types","title":"Importing Types - Writing Schemas","type":"extras"},{"doc":"# Plug and Phoenix Setup\n\nFirst, install Absinthe.Plug and a JSON codec of your choice,\neg, [Jason](https://hex.pm/packages/jason):\n\n```elixir\n# filename: mix.exs\ndef deps do\n  [\n    {:absinthe_plug, \"~> 1.5\"},\n    {:jason, \"~> 1.0\"},\n  ]\nend\n```","ref":"plug-phoenix.html","title":"Plug and Phoenix Setup","type":"extras"},{"doc":"To use, simply `plug` Absinthe.Plug in your pipeline.\n\n```elixir\nplug Absinthe.Plug,\n  schema: MyAppWeb.Schema\n```\n\nIf you are going to support content types other than simply `application/graphql`\nyou should plug Absinthe.Plug after Plug.Parsers.\n\n```elixir\nplug Plug.Parsers,\n  parsers: [:urlencoded, :multipart, :json, Absinthe.Plug.Parser],\n  json_decoder: Jason\n\nplug Absinthe.Plug,\n  schema: MyAppWeb.Schema\n```\n\nFor more information on how the content types work, see [General Usage](#general-usage).","ref":"plug-phoenix.html#plug","title":"Plug - Plug and Phoenix Setup","type":"extras"},{"doc":"If your entire API is going to be based on GraphQL, we recommend simply plugging\nAbsinthe.Plug in at the bottom of your endpoint, and removing your router altogether.\n\n```elixir\ndefmodule MyApp.Endpoint do\n  use Phoenix.Endpoint, otp_app: :my_app\n\n  plug Plug.RequestId\n  plug Plug.Logger\n\n  plug Plug.Parsers,\n    parsers: [:urlencoded, :multipart, :json],\n    pass: [\"*/*\"],\n    json_decoder: Jason\n\n  plug Absinthe.Plug,\n    schema: MyAppWeb.Schema\nend\n```\n\nIf you want only `Absinthe.Plug` to serve a particular route, configure your router\nlike:\n\n```elixir\ndefmodule MyAppWeb.Router do\n  use Phoenix.Router\n\n  resource \"/pages\", MyAppWeb.PagesController\n\n  forward \"/api\", Absinthe.Plug,\n    schema: MyAppWeb.Schema\nend\n```\n\nNow Absinthe.Plug will only serve GraphQL from the `/api` url.","ref":"plug-phoenix.html#phoenix","title":"Phoenix - Plug and Phoenix Setup","type":"extras"},{"doc":"`Absinthe.Plug` will pass any values found inside `conn.private[:absinthe][:context]`\non to `Absinthe.run` as the context. This is how you should handle logic that\nuses headers -- most notably, Authentication.\n\nFor more information, see the [Context](context-and-authentication.md) guide.","ref":"plug-phoenix.html#absinthe-context","title":"Absinthe Context - Plug and Phoenix Setup","type":"extras"},{"doc":"See the [absinthe_plug](https://github.com/absinthe-graphql/absinthe_plug)\nproject and the GraphiQL portion of the [Introspection](introspection.md) guide to\nlearn how to use the built-in `Absinthe.Plug.GraphiQL` plug.","ref":"plug-phoenix.html#graphiql","title":"GraphiQL - Plug and Phoenix Setup","type":"extras"},{"doc":"This plug supports requests in a number of ways:\n\n#","ref":"plug-phoenix.html#general-usage","title":"General Usage - Plug and Phoenix Setup","type":"extras"},{"doc":"With a query string:\n\n```\n?query=query+GetItem($id:ID!){item(id:$id){name}}&variables={id:\"foo\"}\n```\n\nDue to [varying limits on the maximum size of URLs](https://stackoverflow.com/questions/417142/what-is-the-maximum-length-of-a-url-in-different-browsers),\nwe recommend using one of the POST options below instead, putting the `query` into the body of the request.\n\n#","ref":"plug-phoenix.html#via-a-get","title":"Via a GET - Plug and Phoenix Setup","type":"extras"},{"doc":"With a POST body:\n\n```json\n{\n  \"query\": \"query GetItem($id: ID!) { item(id: $id) { name } }\",\n  \"variables\": {\n    \"id\": \"foo\"\n  }\n}\n```\n\n(We could also pull either `query` or `variables` out to the query string, just\nas in the [GET example](#via-a-get).)\n\n#","ref":"plug-phoenix.html#via-an-application-json-post","title":"Via an `application/json` POST - Plug and Phoenix Setup","type":"extras"},{"doc":"With a query string:\n\n`?variables={id:\"foo\"}`\n\nAnd a POST body:\n\n```graphql\nquery GetItem($id: ID!) {\n  item(id: $id) {\n    name\n  }\n}\n```\n\n#","ref":"plug-phoenix.html#via-an-application-graphql-post","title":"Via an `application/graphql` POST - Plug and Phoenix Setup","type":"extras"},{"doc":"How clients interact with the plug over HTTP is designed to closely match that\nof the official\n[express-graphql](https://github.com/graphql/express-graphql) middleware.\n\nIn the [example above](#example), we went over the various ways to\nmake a request, but here are the details:\n\nOnce installed at a path, the plug will accept requests with the\nfollowing parameters:\n\n  * `query` - A string GraphQL document to be executed.\n\n  * `variables` - The runtime values to use for any GraphQL query variables\n    as a JSON object.\n\n  * `operationName` - If the provided `query` contains multiple named\n    operations, this specifies which operation should be executed. If not\n    provided, a 400 error will be returned if the `query` contains multiple\n    named operations.\n\nThe plug will first look for each parameter in the query string, eg:\n\n```\n/graphql?query=query+getUser($id:ID){user(id:$id){name}}&variables={\"id\":\"4\"}\n```\n\nIf not found in the query string, it will look in the POST request body, using\na strategy based on the `Content-Type` header.\n\nFor content types `application/json` and `application/x-www-form-urlencoded`,\nconfigure `Plug.Parsers` (or equivalent) to parse the request body before `Absinthe.Plug`, eg:\n\n```elixir\nplug Plug.Parsers,\n  parsers: [:urlencoded, :multipart, :json],\n  pass: [\"*/*\"],\n  json_decoder: Jason\n```\n\nFor `application/graphql`, the POST body will be parsed as GraphQL query string,\nwhich provides the `query` parameter. If `variables` or `operationName` are\nneeded, they should be passed as part of the","ref":"plug-phoenix.html#http-api","title":"HTTP API - Plug and Phoenix Setup","type":"extras"},{"doc":"As a plug, `Absinthe.Plug` requires very little configuration. If you want to support\n`application/x-www-form-urlencoded` or `application/json` you'll need to plug\n`Plug.Parsers` first.\n\n\n```elixir\nplug Plug.Parsers,\n  parsers: [:urlencoded, :multipart, :json],\n  pass: [\"*/*\"],\n  json_decoder: Jason\n\nplug Absinthe.Plug,\n  schema: MyApp.Linen.Schema\n```\n\n`Absinthe.Plug` requires a `schema:` config.\n\nIt also takes several options. See [the documentation](https://hexdocs.pm/absinthe_plug/Absinthe.Plug.html#init/1)\nfor the full listing.","ref":"plug-phoenix.html#configuration-notes","title":"Configuration Notes - Plug and Phoenix Setup","type":"extras"},{"doc":"You can use GraphQL as the datasource for your Phoenix controllers. For this \nyou'll need to add `absinthe_phoenix` to your dependencies. See [Absinthe Phoenix](https://github.com/absinthe-graphql/absinthe_phoenix) for installation instructions.\n\n```elixir\n@graphql \"\"\"\n  query ($filter: UserFilter) {\n    users(filter: $filter, limit: 10)\n  }\n\"\"\"\ndef index(conn, %{data: data}) do\n  render conn, \"index.html\", data\nend\n```\nThe results of the query are now available in the \"index.html\" template. For\nmore information, see [`Absinthe.Phoenix.Controller`](https://hexdocs.pm/absinthe_phoenix/Absinthe.Phoenix.Controller.html)","ref":"plug-phoenix.html#inside-phoenix-controllers","title":"Inside Phoenix controllers - Plug and Phoenix Setup","type":"extras"},{"doc":"# Writing Middleware and Plugins\n\nMiddleware enables custom resolution behaviour on a field. You can use them to share common logic that needs to happen before or after resolving fields. Things like authentication and error handling.","ref":"middleware-and-plugins.html","title":"Writing Middleware and Plugins","type":"extras"},{"doc":"In order to create a `Middleware` you need a module that implements `Absinthe.Middleware` behaviour . Your module needs to have a `call/2` method which receives an `%Absinthe.Resolution{}` struct and some options as its parameters, and then returns a possibly altered resolution struct.\n\nHere is an example of a middleware that handle `Ecto.Changeset` errors and makes sure we properly add error message to the `errors` in the response.\n\n```elixir\ndefmodule MyApp.Middlewares.HandleChangesetErrors do\n  @behaviour Absinthe.Middleware\n  def call(resolution, _) do\n    %{resolution |\n      errors: Enum.flat_map(resolution.errors, &handle_error/1)\n    }\n  end\n\n  defp handle_error(%Ecto.Changeset{} = changeset) do\n    changeset\n      |> Ecto.Changeset.traverse_errors(fn {err, _opts} -> err end)\n      |> Enum.map(fn({k,v}) -> \"#{k}: #{v}\" end)\n  end\n  defp handle_error(error), do: [error]\nend\n```\n\nThe resolution struct has all kinds of useful values inside of it. You can access the Absinthe context, the root value, information about the current field's AST, and more. For more information on how the current user ends up in the context please see our full [authentication guide](context-and-authentication.md).","ref":"middleware-and-plugins.html#create-a-middleware","title":"Create a Middleware - Writing Middleware and Plugins","type":"extras"},{"doc":"Middleware can be placed on a field in few different ways:\n\n#","ref":"middleware-and-plugins.html#using-middlewares","title":"Using Middlewares - Writing Middleware and Plugins","type":"extras"},{"doc":"This option is good when you want to add your middleware on few specific fields. You can use `middleware` to add multiple Middlewares before or after `resolve`. In this example `MyApp.Web.Authentication` would run before resolution, and `HandleError` would run after.\n\n```elixir\nfield :hello, :string do\n  middleware MyApp.Web.Authentication\n  resolve &get_the_string/2\n  middleware HandleError, :foo\nend\n```\n\n#","ref":"middleware-and-plugins.html#1-using-the-absinthe-schema-notation-middleware-2-macro-used-inside-a-field-definition","title":"1. Using the `Absinthe.Schema.Notation.middleware/2` macro used inside a field definition - Writing Middleware and Plugins","type":"extras"},{"doc":"This option is good if you want to add your middleware on all or a group of fields based on the type of query. `middleware/3` is a function callback on a schema. When you `use Absinthe.Schema` a default implementation of this function is placed in your schema. It is passed the existing middleware for a field, the field itself, and the object that the field is a part of.\nYou can override this callback to add your Middleware in the list of existing middlewares. In this example we add our `HandleChangesetError` Middleware only to mutations.\n\n```elixir\n# add this to your schema module\n\n# if it's a field for the mutation object, add this middleware to the end\ndef middleware(middleware, _field, %{identifier: :mutation}) do\n  middleware ++ [MyApp.Middlewares.HandleChangesetErrors]\nend\n# if it's any other object keep things as is\ndef middleware(middleware, _field, _object), do: middleware\n```\n\n#","ref":"middleware-and-plugins.html#2-using-the-middleware-3-callback-in-your-schema","title":"2. Using the `middleware/3` callback in your schema. - Writing Middleware and Plugins","type":"extras"},{"doc":"You can update your resolution function to return `{:middleware, MyApp.Middlewares.HandleChangesetErrors, config}`, note that in this case the middleware can only be applied after the resolution.","ref":"middleware-and-plugins.html#3-returning-a-middleware-middleware_spec-config-tuple-from-a-resolution-function","title":"3. Returning a `{:middleware, middleware_spec, config}` tuple from a resolution function. - Writing Middleware and Plugins","type":"extras"},{"doc":"# Returning Errors\n\n> ##","ref":"errors.html","title":"Returning Errors","type":"extras"},{"doc":">\n> You can help! Please fork the [absinthe](https://github.com/absinthe-graphql/absinthe) repository, edit `guides/errors.md`, and submit a [pull request](https://github.com/absinthe-graphql/absinthe/pulls).\n\nOne or more errors for a field can be returned in a single `{:error, error_value}` tuple.\n\n`error_value` can be:\n\n- A simple error message string.\n- A map containing `:message` key, plus any additional serializable metadata.\n- A keyword list containing a `:message` key, plus any additional serializable metadata.\n- A list containing multiple of any/all of these.\n- Any other value compatible with `to_string/1`.","ref":"errors.html#this-guide-could-use-some-improvement-warning","title":"This guide could use some improvement. {: .warning} - Returning Errors","type":"extras"},{"doc":"A simple error message:\n\n```elixir\n{:error, \"Something bad happened\"}\n```\n\nMultiple error messages:\n\n```elixir\n{:error, [\"Something bad\", \"Even worse\"]}\n```\n\nSingle custom errors (note the required `:message` keys):\n\n```elixir\n{:error, message: \"Unknown user\", code: 21}\n{:error, %{message: \"A database error occurred\", details: format_db_error(some_value)}}\n```\n\nThree errors of mixed types:\n\n```elixir\n{:error, [\"Simple message\", [message: \"A keyword list error\", code: 1], %{message: \"A map error\"}]}\n```\n\nGeneric handler for interoperability with errors from other libraries:\n\n```elixir\n{:error, :foo}\n{:error, 1.0}\n{:error, 2}\n```","ref":"errors.html#basic-errors","title":"Basic Errors - Returning Errors","type":"extras"},{"doc":"You may want to look at the [Absinthe ErrorPayload](https://hex.pm/packages/absinthe_error_payload) package.","ref":"errors.html#ecto-changeset-errors","title":"Ecto.Changeset Errors - Returning Errors","type":"extras"},{"doc":"# Batching Resolution","ref":"batching.html","title":"Batching Resolution","type":"extras"},{"doc":"In general, you want to make sure that when accessing Ecto associations that you\npreload the data in the top level resolver functions to avoid N+1 queries.\n\nImagine this scenario: You have posts and users. A `Post` has an `author` field, which\nreturns a user. You want to list all posts, and get the name of their author:\n\n```graphql\n{\n  posts {\n    author {\n      name\n    }\n  }\n}\n```\n\nIf you write your schema like this, you're going to have a _bad_ time due to issues with _N + 1_:\n\n```elixir\nobject :post do\n  @desc \"Author of the post\"\n  field :author, :user do\n    resolve fn post, _, _ ->\n      author =\n        post\n        |> Ecto.assoc(:author)\n        |> Repo.one\n\n      {:ok, author}\n    end\n  end\nend\n\nquery do\n  field :posts, list_of(:post) do\n    resolve fn _, _ ->\n      {:ok, Post |> Repo.all}\n    end\n  end\nend\n```\n\nWhat this schema will do when presented with the GraphQL query is\nrun `Post |> Repo.all`, which will retrieve _N_ posts. Then for each\npost it will resolve child fields, which runs our `Repo.one` query\nfunction, resulting in _N+1_ calls to the database.\n\nOne way to handle this issue is with Absinthe's support for\nbatching. The idea with batching is that we're gonna aggregate all the\n`author_id`s from each post, and then make one call to the user.\n\nLet's first make a function to get a model by ids:\n\n```elixir\ndefmodule MyAppWeb.Schema.Helpers do\n  def by_id(model, ids) do\n    import Ecto.Query\n\n    ids = ids |> Enum.uniq\n\n    model\n    |> where([m], m.id in ^ids)\n    |> Repo.all\n    |> Map.new(&{&1.id, &1})\n  end\nend\n```\n\nNow we can use this function to batch our author lookups:\n\n```elixir\nobject :post do\n\n  @desc \"Author of the post\"\n  field :author, :user do\n    resolve fn post, _, _ ->\n      batch({MyAppWeb.Schema.Helpers, :by_id, User}, post.author_id, fn batch_results ->\n        {:ok, Map.get(batch_results, post.author_id)}\n      end)\n    end\n  end\n\nend\n```\n\nNow we make just two calls to the database. The first call loads all of the posts.\nThen as Absinthe walks through each post and tries to get the author, it's instead\ntold to aggregate its information. That aggregate information is passed on to our `by_id/2` function from earlier.\nIt grabs ALL the users in just one database call, and creates a map of user ids\nto users.\n\nAbsinthe then does a second pass and calls the `batch_results` function with that\nmap, letting us retrieve the individual author for each post.\n\nNot only is this a very efficient way to query the data, it's also 100% dynamic.\nIf a query document asks for authors, they're loaded efficiently. If it does not,\nthey aren't loaded at all.\n\nSee the documentation for `Absinthe.Middleware.Batch` for more information.\n\n`Absinthe.Middleware.Batch` achieves a lot and, with some helpers, was the\nstandard way to solve this problem for a long time. While batching still has a\nplace, it has a few limitations that have driven the development of Dataloader.\nThere are small scale annoyances like the limitation of only being able to batch\none thing at a time in a field, or the fact that the API can get very verbose.\n\nThere's also some larger scale issues however. Ecto has a fair number of quirks\nthat make it a difficult library to abstract access to. If you want the\nconcurrent test system to work, you need to add `self()` to all the batch keys\nand do `Repo.all(caller: pid)` in every batch function so that it knows which\nsandbox to use. It gets very easy for your GraphQL functions to become full of\ndirect database access, inevitably going around important data access rules you\nmay want to enforce in your contexts. Alternatively, your context functions can\nend up with dozens of little functions that only exist to support batching items\nby ID.\n\nIn time, people involved in larger projects have been able to build some\nabstractions, helpers, and conventions around the `Absinthe.Middleware.Batch`\nplugin that have done a good job of addressing these issues. That effort has been\nextracted into the Dataloader project, which also draws inspiration from similar\nprojects in the GraphQL world.\n\nWe've made it easier and more flexible, however, with\nElixir's [dataloader](https://hex.pm/packages/dataloader) package.\n\n#","ref":"batching.html#avoiding-n-1-queries","title":"Avoiding N+1 Queries - Batching Resolution","type":"extras"},{"doc":"Let's jump straight in to getting Dataloader working, and then we'll expand on\nwhat's actually happening behind the scenes.\n\nUsing Dataloader is as simple as doing:\n\n```elixir\nalias MyApp.Blog # Dataloader source, see below\nimport Absinthe.Resolution.Helpers, only: [dataloader: 1]\n\nobject :post do\n  field :posts, list_of(:post), resolve: dataloader(Blog)\n\n  @desc \"Author of the post\"\n  field :author, :user do\n    resolve dataloader(Blog)\n  end\nend\n```\n\nTo make this work we need to setup a dataloader, add the `Blog` source to it, and\nmake sure our schema knows it needs to run the dataloader.\n\nLatest install instructions found here: https://github.com/absinthe-graphql/dataloader\n\nLet's start with a data source. Dataloader data sources are just structs that encode\na way of retrieving data in batches. In a Phoenix application you'll generally have one\nsource per context, so that each context can control how its data is loaded.\n\nHere is a hypothetical `Blog` context and a dataloader ecto source:\n\n```elixir\ndefmodule MyApp.Blog do\n  def data() do\n    Dataloader.Ecto.new(MyApp.Repo, query: &query/2)\n  end\n\n  def query(queryable, _params) do\n    queryable\n  end\nend\n```\n\nIn this example, the query returned by `query/2` is used as a starting point by Dataloader to build the final query, which it does by traversing schema associations.\nIn other words, Dataloader can determine that an author has many posts, and that to retrieve posts it needs to get those with the relevant `author_id`.\nIf that's sufficient for your needs, `query/2` need not modify the query it's given.\nBut if you only want to load published posts, `query/2` can narrow the query accordingly.\n\nWhen integrating Dataloader with GraphQL, we want to place it in our context so\nthat we can access it in our resolvers. In your schema module add:\n\n```elixir\nalias MyApp.{Blog}\n\ndef context(ctx) do\n  loader =\n    Dataloader.new\n    |> Dataloader.add_source(Blog, Blog.data())\n\n  Map.put(ctx, :loader, loader)\nend\n\ndef plugins do\n  [Absinthe.Middleware.Dataloader] ++ Absinthe.Plugin.defaults()\nend\n```\n\nThat's it! If you run a GraphQL query that hits that field, it will be loaded efficiently without N+1.\n\nSee the documentation for [Dataloader](dataloader.md) for more information.","ref":"batching.html#dataloader","title":"Dataloader - Batching Resolution","type":"extras"},{"doc":"# Dataloader\n\nDataloader provides an easy way efficiently load data in batches.\nIt's inspired by https://github.com/facebook/dataloader","ref":"dataloader.html","title":"Dataloader","type":"extras"},{"doc":"```elixir\ndef deps do\n  [\n    {:dataloader, \"~> 1.0.0\"}\n  ]\nend\n```","ref":"dataloader.html#installation","title":"Installation - Dataloader","type":"extras"},{"doc":"The core concept of dataloader is a data source which is just a struct\nthat encodes a way of retrieving data. More info in the [Sources](#sources) section.\n\n#","ref":"dataloader.html#usage","title":"Usage - Dataloader","type":"extras"},{"doc":"Absinthe provides some dataloader helpers out of the box that you can import into your schema\n\n```elixir\nimport Absinthe.Resolution.Helpers, only: [dataloader: 1]\n```\n\nThis is needed to use the various `dataloader` helpers to resolve a field:\n\n```elixir\nfield(:posts, list_of(:post), resolve: dataloader(Blog))\n```\n\nLet's start with a data source. Dataloader data sources are just structs that encode\na way of retrieving data in batches. In a Phoenix application you'll generally have one\nsource per context, so that each context can control how its data is loaded.\n\nHere is a hypothetical `Blog` context and a dataloader ecto source:\n\n```elixir\ndefmodule MyApp.Blog do\n  def data() do\n    Dataloader.Ecto.new(MyApp.Repo, query: &query/2)\n  end\n\n  def query(queryable, _params) do\n    queryable\n  end\nend\n```\n\nIn this example, the query returned by `query/2` is used as a starting point by Dataloader to build the final query, which it does by traversing schema associations.\nIn other words, Dataloader can determine that an author has many posts, and that to retrieve posts it needs to get those with the relevant `author_id`.\nIf that's sufficient for your needs, `query/2` need not modify the query it's given.\nBut if you only want to load published posts, `query/2` can narrow the query accordingly.\n\nWhen integrating Dataloader with GraphQL, we want to place it in our context so\nthat we can access it in our resolvers. In your schema module add:\n\n```elixir\nalias MyApp.{Blog, Foo}\n\ndef context(ctx) do\n  loader =\n    Dataloader.new\n    |> Dataloader.add_source(Blog, Blog.data())\n    |> Dataloader.add_source(Foo, Foo.data()) # Foo source could be a Redis source\n\n  Map.put(ctx, :loader, loader)\nend\n\ndef plugins do\n  [Absinthe.Middleware.Dataloader] ++ Absinthe.Plugin.defaults()\nend\n```\n\nThe `context/1` function is a callback specified by the `Absinthe.Schema` behaviour that gives\nthe schema itself an opportunity to set some values in the context that it may need in order to run.\n\nThe `plugins/0` function has been around for a while, and specifies what plugins the schema needs to resolve.\nSee [the documentation](`c:Absinthe.Schema.plugins/0`) for more.\n\n##","ref":"dataloader.html#schema","title":"Schema - Dataloader","type":"extras"},{"doc":"The `data/0` function creates an Ecto data source, to which you pass your repo and a query function. This query function\nis called every time you want to load something, and provides an opportunity to apply arguments or\nset defaults. So for example if you always want to only load non-deleted posts you can do:\n\n```elixir\ndef query(Post, _), do: from p in Post, where: is_nil(p.deleted_at)\n\ndef query(queryable, _), do: queryable\n```\n\nNow any time you're loading posts, you'll just get posts that haven't been\ndeleted.\n\nWe can also use the context to ensure access conditions, so we can only show deleted posts for admins:\n\n```elixir\ndef query(Post, %{has_admin_rights: true}), do: Post\n\ndef query(Post, _), do: from p in Post, where: is_nil(p.deleted_at)\n\ndef query(queryable, _), do: queryable\n```\n\nHelpfully, those rules are defined within your context, helping ensure\nthat it has the final say about data access.\n\n#","ref":"dataloader.html#unpacking-dataloader","title":"Unpacking Dataloader - Dataloader","type":"extras"},{"doc":"Dataloader ships with two different built in sources:\n\n* `Dataloader.Ecto` - for easily pulling out data with ecto\n* `Dataloader.KV` - a simple KV key value source.\n\n##","ref":"dataloader.html#sources","title":"Sources - Dataloader","type":"extras"},{"doc":"Here is a simple example of a loader using the `KV` source in combination with absinthe:\n\n```elixir\ndefmodule MyProject.Loaders.Nhl do\n  @teams [%{\n    id: 1,\n    name: \"New Jersey Devils\",\n    abbreviation: \"NJD\"\n  },\n  %{\n    id: 2,\n    name: \"New York Islanders\",\n    abbreviation: \"NYI\"\n  }\n  # etc.\n  ]\n\n  def data() do\n    Dataloader.KV.new(&fetch/2)\n  end\n\n  def fetch(:teams, [%{}]) do\n    %{\n      %{} => @teams\n    }\n  end\n\n  def fetch(:team, args) do\n   # must return a map keyed by the args\n   # args is a list of the args used to resolve your field\n   # for example, if you have arg(:foo, non_null(:string))\n   # args will look like: [%{foo: \"value of foo here\")}]\n\n    args\n    |> Enum.reduce(%{}, fn(%{id: id} = arg, result) ->\n      Map.put(result, arg, find_team(id))\n    end)\n  end\n\n  def fetch(_batch, args) do\n    args |> Enum.reduce(%{}, fn(arg, accum) -> Map.put(accum, arg, nil) end)\n  end\n\n  defp find_team(id) do\n    @teams |> Enum.find(fn(t) -> t |> Map.get(:id) == id end)\n  end\nend\n```\n\n`Dataloader.KV` requires a load function that accepts a batch and args. It must return a map of values keyed by the args.\nThis is the purpose of the `fetch/2` function. The `dataloader/1` helper we imported above uses the field name as the batch, and a map where the argument name is the key. For example: `fetch(:team, [%{ id: 1 }])`\n\nPattern matching can be used to fetch differently depending on the batch. For example, when the :teams batch is requested, the args will actually be an empty map (i.e. `%{}`).\n\nIf you’re interested in more generic use of Dataloader, see the [dataloader project source](https://github.com/absinthe-graphql/dataloader).","ref":"dataloader.html#kv","title":"KV - Dataloader","type":"extras"},{"doc":"# The Context and Authentication\n\nAbsinthe context exists to provide shared values to a given document execution.\nA common use would be to pass in the current user of a given request. The context\nis set at the call to `Absinthe.run`, and cannot be modified over the course of\na given execution.","ref":"context-and-authentication.html","title":"The Context and Authentication","type":"extras"},{"doc":"As a basic example let's think about a profile page, where we want the current user\nto be able to access basic information about themselves, but not other users.\n\nFirst we'll need a very basic schema:\n\n```elixir\ndefmodule MyAppWeb.Schema do\n  use Absinthe.Schema\n\n  @fakedb %{\n    \"1\" => %{name: \"Bob\", email: \"bubba@foo.com\"},\n    \"2\" => %{name: \"Fred\", email: \"fredmeister@foo.com\"},\n  }\n\n  query do\n    field :profile, :user do\n      resolve fn _, _, _ ->\n        # How could we get a current user here?\n      end\n    end\n  end\n\n  object :user do\n    field :id, :id\n    field :name, :string\n    field :email, :string\n  end\nend\n```\n\nA query we might want could look like:\n\n```graphql\n{\n  profile {\n    email\n  }\n}\n```\n\nIf we're signed in as user 1, we should get only user 1's email, for example:\n\n```json\n{\n  \"profile\": {\n    \"email\": \"bubba@foo.com\"\n  }\n}\n```\n\nIn order to set the context, our call to `Absinthe.run/3` should look like:\n\n```elixir\nAbsinthe.run(document, MyAppWeb.Schema, context: %{current_user: %{id: \"1\"}})\n```\n\nTo access this, we need to update our query's resolve function:\n\n```elixir\nquery do\n  field :profile, :user do\n    resolve fn _, _, %{context: %{current_user: current_user}} ->\n      {:ok, Map.get(@fakedb, current_user.id)}\n    end\n  end\nend\n```\n\nAnd now it works!","ref":"context-and-authentication.html#basic-usage","title":"Basic Usage - The Context and Authentication","type":"extras"},{"doc":"When using Absinthe.Plug you don't have direct access to the Absinthe.run call.\nInstead, we can use `Absinthe.Plug.put_options/2` to set context values which\nAbsinthe.Plug will use to pass it along to Absinthe.run.\n\nSetting up your GraphQL context is as simple as writing a plug that inserts the\nappropriate values into the connection.\n\nLet's use this mechanism to set our current_user from the previous example via\nan authentication header. We will use the same Schema as before.\n\nFirst, our plug. We'll be checking the connection for the `authorization` header, and calling\nout to some unspecified authentication mechanism.\n\n```elixir\ndefmodule MyAppWeb.Context do\n  @behaviour Plug\n\n  import Plug.Conn\n  import Ecto.Query, only: [where: 2]\n\n  alias MyApp.{Repo, User}\n\n  def init(opts), do: opts\n\n  def call(conn, _) do\n    context = build_context(conn)\n    Absinthe.Plug.put_options(conn, context: context)\n  end\n\n  @doc \"\"\"\n  Return the current user context based on the authorization header\n  \"\"\"\n  def build_context(conn) do\n    with [\"Bearer \"   token]   %{}\n    end\n  end\n\n  defp authorize(token) do\n    User\n    |> where(token: ^token)\n    |> Repo.one\n    |> case do\n      nil -> {:error, \"invalid authorization token\"}\n      user -> {:ok, user}\n    end\n  end\n\nend\n```\n\nThis plug will use the `authorization` header to lookup the current user. If one\nis found, it correctly sets the absinthe context. If you're using Guardian or\nsome other library that provides utilities for authenticating users you can use\nthose here too, and just add their output to the context.\n\nIf there is no current user it's better to simply not have the `:current_user`\nkey inside the map, instead of doing `%{current_user: nil}`. This way you can\njust pattern match for `%{current_user: user}` in your code and not need to\nworry about the nil case.\n\nUsing this plug is very simple. If we're just in a normal plug context we can\njust make sure it's plugged prior to Absinthe.Plug\n\n```elixir\nplug MyAppWeb.Context\n\nplug Absinthe.Plug,\n  schema: MyAppWeb.Schema\n```\n\nIf you're using a Phoenix router, add the context plug to a pipeline.\n\n```elixir\ndefmodule MyAppWeb.Router do\n  use Phoenix.Router\n\n  resource \"/pages\", MyAppWeb.PagesController\n\n  pipeline :graphql do\n    plug MyAppWeb.Context\n  end\n\n  scope \"/api\" do\n    pipe_through :graphql\n\n    forward \"/\", Absinthe.Plug,\n      schema: MyAppWeb.Schema\n  end\nend\n```","ref":"context-and-authentication.html#context-and-plugs","title":"Context and Plugs - The Context and Authentication","type":"extras"},{"doc":"# Understanding Subscriptions\n\nGraphQL subscriptions are a way to have events in the server push data out to clients in real time. The client submits a subscription document that asks for particular data, and then when events happen that document is run against that event and the resulting data is pushed out.\n\nLike queries and mutations, subscriptions are not intrinsically tied to any particular transport, and they're built within Absinthe itself to be able to operate on many different platforms.\n\nAt the moment however the most common and fully featured platform that you can run them on with Elixir is via Phoenix channels, so this guide will walk you through the basics of getting them hooked up to a phoenix application.\n\n#","ref":"subscriptions.html","title":"Understanding Subscriptions","type":"extras"},{"doc":"Packages you'll need:\n\n```elixir\n{:absinthe, \"~> 1.7\"},\n{:phoenix_pubsub, \"~> 2.0\"},\n{:absinthe_phoenix, \"~> 2.0\"},\n```\n\nYou need to have a working Phoenix pubsub configured. Here is what the default looks like if you create a new Phoenix project:\n\n```elixir\nconfig :my_app, MyAppWeb.Endpoint,\n  pubsub_server: :my_pubsub\n\n```\n\nIn your application supervisor add a line _after_ your existing endpoint supervision\nline:\n\n```elixir\n    # List all child processes to be supervised\n    children = [\n      # Start the Ecto repository\n      MyAppWeb.Repo,\n      # Start the PubSub server\n      {Phoenix.PubSub, name: :my_pubsub},\n      # Start the endpoint when the application starts\n      MyAppWeb.Endpoint,\n      {Absinthe.Subscription, MyAppWeb.Endpoint}\n    ]\n\n    # See https://hexdocs.pm/elixir/Supervisor.html\n    # for other strategies and supported options\n    opts = [strategy: :one_for_one, name: MyAppWeb.Supervisor]\n    Supervisor.start_link(children, opts)\n```\n\nSee `Absinthe.Subscription.child_spec/1` for more information on the supported\noptions.\n\nIn your `MyAppWeb.Endpoint` module add:\n\n```elixir\nuse Absinthe.Phoenix.Endpoint\n```\n\nNow, you need to configure your socket. I.e. in your `MyAppWeb.UserSocket` module add:\n\n```elixir\nuse Absinthe.Phoenix.Socket,\n  schema: MyAppWeb.Schema\n```\n\nWhere `MyAppWeb.Schema` is the name of your Absinthe schema module. See the [`Absinthe.Phoenix`](https://hexdocs.pm/absinthe_phoenix/readme.html) package for more options.\n\n#","ref":"subscriptions.html#absinthe-phoenix-setup","title":"Absinthe.Phoenix Setup - Understanding Subscriptions","type":"extras"},{"doc":"If you're using the GraphiQL plug, in your `MyAppWeb.Router`, specify the `socket` option:\n\n```elixir\nforward \"/graphiql\",\n        Absinthe.Plug.GraphiQL,\n        schema: MyAppWeb.Schema,\n        socket: MyAppWeb.UserSocket\n```\n\nThat is all that's required for setup on the server.\n\n#","ref":"subscriptions.html#graphiql-optional","title":"GraphiQL (optional) - Understanding Subscriptions","type":"extras"},{"doc":"Options like the context can be configured in the `connect/2` callback in your\nsocket module.\n\n```elixir\ndefmodule MyAppWeb.UserSocket do\n  use Phoenix.Socket\n  use Absinthe.Phoenix.Socket,\n    schema: MyAppWeb.Schema\n\n  def connect(params, socket) do\n    current_user = current_user(params)\n    socket = Absinthe.Phoenix.Socket.put_options(socket, context: %{\n      current_user: current_user\n    })\n    {:ok, socket}\n  end\n\n  defp current_user(%{\"user_id\" => id}) do\n    MyApp.Repo.get(User, id)\n  end\n\n  def id(_socket), do: nil\nend\n```\n\n#","ref":"subscriptions.html#setting-options","title":"Setting Options - Understanding Subscriptions","type":"extras"},{"doc":"Here's an example schema that lets you use subscriptions to get notified when a comment\nis submitted to a GitHub repository:\n\n```elixir\nmutation do\n  field :submit_comment, :comment do\n    arg :repo_name, non_null(:string)\n    arg :content, non_null(:string)\n\n    resolve &Github.submit_comment/3\n  end\nend\n\nsubscription do\n  field :comment_added, :comment do\n    arg :repo_name, non_null(:string)\n\n    # The topic function is used to determine what topic a given subscription\n    # cares about based on its arguments. You can think of it as a way to tell the\n    # difference between\n    # subscription {\n    #   commentAdded(repoName: \"absinthe-graphql/absinthe\") { content }\n    # }\n    #\n    # and\n    #\n    # subscription {\n    #   commentAdded(repoName: \"elixir-lang/elixir\") { content }\n    # }\n    #\n    # If needed, you can also provide a list of topics:\n    #   {:ok, topic: [\"absinthe-graphql/absinthe\", \"elixir-lang/elixir\"]}\n    config fn args, _ ->\n      {:ok, topic: args.repo_name}\n    end\n\n    # this tells Absinthe to run any subscriptions with this field every time\n    # the :submit_comment mutation happens.\n    # It also has a topic function used to find what subscriptions care about\n    # this particular comment\n    trigger :submit_comment, topic: fn comment ->\n      comment.repository_name\n    end\n\n    resolve fn comment, _, _ ->\n      # this function is often not actually necessary, as the default resolver\n      # for subscription functions will just do what we're doing here.\n      # The point is, subscription resolvers receive whatever value triggers\n      # the subscription, in our case a comment.\n      {:ok, comment}\n    end\n\n  end\nend\n```\n\nConcretely, if client A submits a subscription document:\n\n```graphql\nsubscription {\n  commentAdded(repoName: \"absinthe-graphql/absinthe\") {\n    content\n  }\n}\n```\n\nThis tells Absinthe to subscribe client A in the `:comment_added` field on the `\"absinthe-graphql/absinthe\"` topic, because that's what comes back from the `config` function.\n\nThen, if client B submits a mutation:\n\n```graphql\nmutation {\n  submitComment(\n    repoName: \"absinthe-graphql/absinthe\"\n    content: \"Great library!\"\n  ) {\n    id\n  }\n}\n```\n\nClient B will get the normal response to their mutation, and since they just ask for the `id` that's what they'll get.\n\nAdditionally, the `:submit_comment` mutation is configured as a trigger on the `:comment_added` subscription field, so the trigger function is called. That function returns `\"absinthe-graphql/absinthe\"` because that's the repository name for the comment, and now Absinthe knows it needs to get all subscriptions on the `:comment_added` field that have the `\"absinthe-graphql/absinthe\"` topic, so client A gets back:\n\n```json\n{ \"data\": { \"commentAdded\": { \"content\": \"Great library!\" } } }\n```\n\nIf you want to publish to this subscription manually (not using triggers in the schema) you can do:\n\n```elixir\nAbsinthe.Subscription.publish(MyAppWeb.Endpoint, comment, comment_added: \"absinthe-graphql/absinthe\")\n```\n\nIf you want to subscribe to mutations from within your application, you can do:\n\n```elixir\n{:ok, %{\"subscribed\" => topic}} = Absinthe.run(subscription_query, MyAppWeb.Schema, context: %{pubsub: MyAppWeb.Endpoint})\nMyAppWeb.Endpoint.subscribe(topic)\n```\n\n#","ref":"subscriptions.html#schema","title":"Schema - Understanding Subscriptions","type":"extras"},{"doc":"By default, Absinthe will resolve each outgoing publish once per individual subscription. This ensures:\n\n- Different GraphQL documents each receive the different fields they requested\n- User-specific updates are sent out, in case `context` contains user-specific data\n\nTo improve the scale at which your subscriptions operate, you may tell Absinthe when it is safe to de-duplicate updates. Simply return a `context_id` from your field's `config` function:\n\n```elixir\nsubscription do\n  field :news_article_published, :article do\n    config fn _, _ ->\n      {:ok, topic: \"*\", context_id: \"global\"}\n    end\n  end\nend\n```\n\nHere we return a constant (`\"global\"`) because our `:article` type doesn't contain any user-specific fields on it.\n\nGiven these three active subscriptions:\n\n```graphql\n# user 1\nsubscription {\n  newsArticlePublished {\n    content\n  }\n}\n\n# user 2\nsubscription {\n  newsArticlePublished {\n    content\n    author\n  }\n}\n\n# user 3\nsubscription {\n  newsArticlePublished {\n    content\n  }\n}\n```\n\nSince we provided a `context_id`, Absinthe will only run two documents per publish to this field:\n\n1. Once for _user 1_ and _user 3_ because they have the same context ID (`\"global\"`) and sent the same document.\n2. Once for _user 2_. While _user 2_ has the same context ID (`\"global\"`), they provided a different document, so it cannot be de-duplicated with the other two.","ref":"subscriptions.html#de-duplicating-updates","title":"De-duplicating Updates - Understanding Subscriptions","type":"extras"},{"doc":"# Custom Scalar Types\n\nOne of the strengths of GraphQL is its extensibility---which doesn't end with\nits object types, but is present all the way down to the scalar value level.\n\nSometimes it makes sense to build custom scalar types to better model your\ndomain. Here's how to do it.\n\nThe GraphQL Specification doesn't define date and datetime types, but Absinthe ships with several pre-built for use via [import_types](importing-types.md). In this example we'll look at how `:datetime` is defined.","ref":"custom-scalars.html","title":"Custom Scalar Types","type":"extras"},{"doc":"Supporting additional scalar types is as easy as using the `scalar` macro and\nproviding `parse` and `serialize` functions.\n\nHere's the definition for `:datetime` from `Absinthe.Type.Custom`:\n\n```elixir\n@desc \"\"\"\nThe `DateTime` scalar type represents a date and time in the UTC\ntimezone. The DateTime appears in a JSON response as an ISO8601 formatted\nstring, including UTC timezone (\"Z\"). The parsed date and time string will\nbe converted to UTC and any UTC offset other than 0 will be rejected.\n\"\"\"\nscalar :datetime, name: \"DateTime\" do\n  serialize &DateTime.to_iso8601/1\n  parse &parse_datetime/1\nend\n\n@spec parse_datetime(Absinthe.Blueprint.Input.String.t) :: {:ok, DateTime.t} | :error\n@spec parse_datetime(Absinthe.Blueprint.Input.Null.t) :: {:ok, nil}\ndefp parse_datetime(%Absinthe.Blueprint.Input.String{value: value}) do\n  case DateTime.from_iso8601(value) do\n    {:ok, datetime, 0} -> {:ok, datetime}\n    {:ok, _datetime, _offset} -> :error\n    _error -> :error\n  end\nend\ndefp parse_datetime(%Absinthe.Blueprint.Input.Null{}) do\n  {:ok, nil}\nend\ndefp parse_datetime(_) do\n  :error\nend\n```\n\nScalar definitions, created using the `scalar` macro from\n`Absinthe.Schema.Notation`, are made up of two major elements:\n\n- A function provided to `parse` that defines how input is converted\n  from an [AST](https://en.wikipedia.org/wiki/Abstract_syntax_tree)\n  value (an `Absinthe.Blueprint.Input.t`) to a value that's suitable\n  for use in an argument and passed to a resolver.\n- A function provided to `serialize` that defines how input is\n  serialized back out when used as a result that should be sent back\n  to the user.\n\nIn this example:\n\n```graphql\nmutation CreatePost {\n  post(title: \"Second\", body: \"We're off to a great start!\", publishedAt: \"2017-11-01T12:00:00Z\") {\n    id\n    publishedAt\n  }\n}\n```\n\nIf our schema defines a `:published_at` argument with the `:datetime` type:\n\n```elixir\nfield :post, :post do\n  arg :published_at, :datetime\n  resolve fn _, args, _ ->\n    # If the arg is provided, `args.published_at` will be a DateTime struct\n    # ...\n  end\nend\n```\n\nThen, the value of the `publishedAt` GraphQL field ends up being\nparsed by the `parse_datetime/1` function; it's been defined as the\nparse function for the `:datetime` type using the `parse` macro.\n\nThe value, `\"2017-11-01T12:00:00Z\"`, will come into the function as an `%Absinthe.Blueprint.Input.String{}`, thanks to the hard work of the Absinthe parser and processing pipeline.\n\nThe `parse_datetime/1` function pulls the string value out of the\ninput, parses it with `DateTime.from_iso8601/1`, then returns the\ncorrect result. That result will be used as the value of the\n`:published_at` argument when it's passed to the `:post` field\nresolver.\n\nIn the event of an error, the parse function can return either a lone\natom `:error` or error tuple e.g. `{:error, \"Reason of error\"}`. The\nreason will be included in the output of the executed GraphQL document.\n\nThe serializer of `:datetime` is a pretty simple affair; it uses the\n`DateTime.to_iso8601/1` utility function. It would be called to\nserialize the `%DateTime{}` struct for the requested `:published_at`\nfield in the result.\n\nMore custom scalar examples can be found under [Absinthe Wiki - Scalar Recipes](https://github.com/absinthe-graphql/absinthe/wiki/Scalar-Recipes).","ref":"custom-scalars.html#defining-a-scalar","title":"Defining a scalar - Custom Scalar Types","type":"extras"},{"doc":"# Importing Types\n\nIt doesn't take long for a schema module to become crowded with types,\nresolvers, and other customizations.\n\nA good first step in cleaning up your schema is extracting your types,\norganizing them into other modules, and then using `Absinthe.Schema.Notation.import_types/1`\nto make them available to your schema.","ref":"importing-types.html","title":"Importing Types","type":"extras"},{"doc":"Let's say you have a schema that looks something like this:\n\n``` elixir\ndefmodule MyAppWeb.Schema do\n  use Absinthe.Schema\n\n  object :user do\n    field :name, :string\n  end\n\n  # Rest of the schema...\n\nend\n```\n\nYou could extract your `:user` type into a module, `MyAppWeb.Schema.AccountTypes`:\n\n``` elixir\ndefmodule MyAppWeb.Schema.AccountTypes do\n  use Absinthe.Schema.Notation\n\n  object :user do\n    field :name, :string\n  end\nend\n```\n\n> Note that, unlike your schema module, _type modules_ should use\n> `Absinthe.Schema.Notation`, *not* `Absinthe.Schema`.\n\nNow, you need to make sure you use `import_types` to tell your schema\nwhere to find additional types:\n\n``` elixir\ndefmodule MyAppWeb.Schema do\n  use Absinthe.Schema\n\n  import_types MyAppWeb.Schema.AccountTypes\n\n  # Rest of the schema...\nend\n```\n\n> Important: You should _only_ use `import_types` from your schema\n> module; think of it like a manifest.\n\nNow, your schema will be able to resolve any references to your `:user` type\nduring compilation.","ref":"importing-types.html#example","title":"Example - Importing Types","type":"extras"},{"doc":"Root types (which are defined using the `query`, `mutation`, and\n`subscription` macros), can only be defined on the schema module---you\ncan't extract them, but you can use the `import_fields` mechanism to\nextract their contents.\n\nHere's an example:\n\n``` elixir\nquery do\n  import_fields :account_queries\nend\n```\n\nThis will look for a matching object type `:account_queries`, and pull\nits fields into the root query type.\n\nFor more information, see the [guide](importing-fields.md).","ref":"importing-types.html#what-about-root-types","title":"What about root types? - Importing Types","type":"extras"},{"doc":"# Importing Fields\n\nSometimes an object type becomes too large and needs to be broken into\npieces. This is especially true of the root query, mutation, and\nsubscription types that are defined in the schema module itself.\n\nAbsinthe provides a mechanism,\n`Absinthe.Schema.Notation.import_fields/1`, to support objects being\nable to import fields from other object types.","ref":"importing-fields.html","title":"Importing Fields","type":"extras"},{"doc":"Let's say you're building a content management system. Your root query\ntype has become unwieldy over time, and your schema looks something\nlike this:\n\n``` elixir\ndefmodule MyAppWeb.Schema do\n  use Absinthe.Schema\n\n  query do\n\n    @desc \"Get all the users, optionally filtering\"\n    field :users, list_of(:user) do\n      # ...\n    end\n\n    @desc \"Get a user using criteria\"\n    field :user, :user do\n      # ...\n    end\n\n    # More account-related fields..\n\n    @desc \"Get all the articles, optionally filtering\"\n    field :articles, list_of(:article) do\n      # ...\n    end\n\n    @desc \"Get an article using criteria\"\n    field :article, :article do\n      # ...\n    end\n\n    # More content-related fields...\n\n  end\n\n  # Other types...\n\nend\n```\n\nThis could be cleaned up to look something like this:\n\n``` elixir\ndefmodule MyAppWeb.Schema do\n  use Absinthe.Schema\n\n  import_types MyAppWeb.Schema.AccountTypes\n  import_types MyAppWeb.Schema.ContentTypes\n\n  query do\n\n    # Using :account_queries from MyAppWeb.Schema.AccountTypes\n    import_fields :account_queries\n\n    # Using :content_queries from MyAppWeb.Schema.ContentTypes\n    import_fields :content_queries\n\n  end\n\n  # Other types...\n\nend\n```\n\n`import_fields` here is pulling fields in from separate object types.\n\n> Before you can import fields from another object type, make sure\n> that the type in question is available to your schema. See\n> the [guide](importing-types.md) on importing types for information\n> on how that's done.\n\nHere's how those object types are defined.\n\nFirst, `AccountTypes`:\n\n``` elixir\ndefmodule MyAppWeb.Schema.AccountTypes do\n  use Absinthe.Schema.Notation\n\n  object :account_queries do\n\n    @desc \"Get all the users, optionally filtering\"\n    field :users, list_of(:user) do\n      # ...\n    end\n\n    @desc \"Get a user using criteria\"\n    field :user, :user do\n      # ...\n    end\n\n    # More account-related fields...\n\n  end\n\n  # More account-related types...\n\nend\n```\n\nAnd `ContentTypes`:\n\n``` elixir\ndefmodule MyAppWeb.Schema.ContentTypes do\n  use Absinthe.Schema.Notation\n\n  object :content_queries do\n\n    @desc \"Get all the articles, optionally filtering\"\n    field :articles, list_of(:article) do\n      # ...\n    end\n\n    @desc \"Get an article using criteria\"\n    field :article, :article do\n      # ...\n    end\n\n    # More content-related fields\n\n  end\n\n  # More content-related types...\n\nend\n```\n\nFor more information on `import_types`, see [the guide](importing-types.md).","ref":"importing-fields.html#an-example","title":"An Example - Importing Fields","type":"extras"},{"doc":"# Using Document Variables\n\nGraphQL supports query documents that declare variables that can be accepted to fill-in values. This is a useful mechanism for reusing GraphQL documents---instead of attempting to interpolate values yourself.\n\n- To support variables, simply define them for your query document [as the specification expects](https://spec.graphql.org/October2021/#sec-Language.Variables), and pass in a `variables` option to `Absinthe.run`.\n- If you're using [absinthe_plug](https://github.com/absinthe-graphql/absinthe_plug), variables are passed in for you automatically after being parsed\nfrom the query parameters or `POST` body.\n\nHere's an example of defining a non-nullable variable, `id`, in a document and then executing the document with a value for the variable:\n\n```elixir\n\"\"\"\nquery GetItem($id: ID!) {\n  item(id: $id) {\n    name\n  }\n}\n\"\"\"\n|> Absinthe.run(MyAppWeb.Schema, variables: %{\"id\" => \"bar\"})\n\n# Result\n{:ok, %{data: %{\"item\" => %{\"name\" => \"Bar\"}}}}\n```","ref":"variables.html","title":"Using Document Variables","type":"extras"},{"doc":"# Schema Introspection\n\nYou can introspect your schema using `__schema`, `__type`, and `__typename`,\nas [described in the specification](https://spec.graphql.org/October2021/#sec-Introspection).\n\n#","ref":"introspection.html","title":"Schema Introspection","type":"extras"},{"doc":"Seeing the names of the types in the schema:\n\n```elixir\n\"\"\"\n{\n  __schema {\n    types {\n      name\n    }\n  }\n}\n\"\"\" |> Absinthe.run(MyAppWeb.Schema)\n{:ok,\n  %{data: %{\n    \"__schema\" => %{\n      \"types\" => [\n        %{\"name\" => \"Boolean\"},\n        %{\"name\" => \"Float\"},\n        %{\"name\" => \"ID\"},\n        %{\"name\" => \"Int\"},\n        %{\"name\" => \"String\"},\n        ...\n      ]\n    }\n  }}\n}\n```\n\nGetting the name of the queried type:\n\n```elixir\n\"\"\"\n{\n  profile {\n    name\n    __typename\n  }\n}\n\"\"\" |> Absinthe.run(MyAppWeb.Schema)\n{:ok,\n  %{data: %{\n    \"profile\" => %{\n      \"name\" => \"Joe\",\n      \"__typename\" => \"Person\"\n    }\n  }}\n}\n```\n\nGetting the name of the fields for a named type:\n\n```elixir\n\"\"\"\n{\n  __type(name: \"Person\") {\n    fields {\n      name\n      type {\n        kind\n        name\n      }\n    }\n  }\n}\n\"\"\" |> Absinthe.run(MyAppWeb.Schema)\n{:ok,\n  %{data: %{\n    \"__type\" => %{\n      \"fields\" => [\n        %{\n          \"name\" => \"name\",\n          \"type\" => %{\"kind\" => \"SCALAR\", \"name\" => \"String\"}\n        },\n        %{\n          \"name\" => \"age\",\n          \"type\" => %{\"kind\" => \"SCALAR\", \"name\" => \"Int\"}\n        },\n      ]\n    }\n  }}\n}\n```\n\nNote that you may have to nest several depths of `type`/`ofType`, as\ntype information includes any wrapping layers of [List](https://spec.graphql.org/October2021/#sec-List) and/or [NonNull](https://spec.graphql.org/October2021/#sec-Non-Null).","ref":"introspection.html#examples","title":"Examples - Schema Introspection","type":"extras"},{"doc":"The [GraphiQL project](https://github.com/graphql/graphiql) is\n\"an in-browser IDE for exploring GraphQL.\"\n\nAbsinthe provides GraphiQL via a plug in `absinthe_plug`. See the [Plug and Phoenix Guide](plug-phoenix.md)\nfor how to install that library. Once installed, usage is simple as:\n\n```elixir\nplug Absinthe.Plug.GraphiQL, schema: MyAppWeb.Schema\n```\n\nIf you want to use it at a particular path (in this case `graphiql` in your Phoenix\nrouter) simply do:\n\n```elixir\n# filename: router.ex\nforward \"/graphiql\", Absinthe.Plug.GraphiQL, schema: MyAppWeb.Schema\n```\n\nThis can be trivially reserved to just the `:dev` elixir environment by doing:\n\n```elixir\n# filename: router.ex\nif Mix.env == :dev do\n  forward \"/graphiql\", Absinthe.Plug.GraphiQL, schema: MyAppWeb.Schema\nend\n```\n\nIf you'd prefer to use a desktop application, we recommend using the pre-built\n[Electron](https://electron.atom.io)-based wrapper application,\n[GraphiQL.app](https://github.com/skevy/graphiql-app).","ref":"introspection.html#using-graphiql","title":"Using GraphiQL - Schema Introspection","type":"extras"},{"doc":"# Telemetry\n\nAbsinthe 1.5 and up uses `telemetry` to instrument its activity.\n\nCall `:telemetry.attach/4` or `:telemetry.attach_many/4` to attach your\nhandler function to any of the following event names:\n\n- `[:absinthe, :execute, :operation, :start]` when the operation starts\n- `[:absinthe, :execute, :operation, :stop]` when the operation finishes\n- `[:absinthe, :subscription, :publish, :start]` when a subscription starts\n- `[:absinthe, :subscription, :publish, :stop]` when a subscription finishes\n- `[:absinthe, :resolve, :field, :start]` when field resolution starts\n- `[:absinthe, :resolve, :field, :stop]` when field resolution finishes\n- `[:absinthe, :middleware, :batch, :start]` when the batch processing starts\n- `[:absinthe, :middleware, :batch, :stop]` when the batch processing finishes\n\nTelemetry handlers are called with `measurements` and `metadata`. For details on\nwhat is passed, checkout `Absinthe.Phase.Telemetry`, `Absinthe.Middleware.Telemetry`,\nand `Absinthe.Middleware.Batch`.\n\nFor async, batch, and dataloader fields, Absinthe sends the final event when\nit gets the results. That might be later than when the results are ready. If\nyou need to know how long the underlying operation took, you'll need to hook\ntelemetry up to that underlying operation. See, for example, the recommended\ntelemetry events in the documentation for `Ecto.Repo`.","ref":"telemetry.html","title":"Telemetry","type":"extras"},{"doc":"As an example, you could attach a handler in an `iex -S mix` shell. Paste in:\n\n```elixir\n:telemetry.attach_many(\n  :demo,\n  [\n    [:absinthe, :resolve, :field, :stop]\n  ],\n  fn event_name, measurements, metadata, _config ->\n    %{\n      event_name: event_name,\n      measurements: measurements,\n      metadata: metadata\n    }\n    |> IO.inspect()\n  end,\n  []\n)\n```\n\nAfter a query is executed, you'll see something like:\n\n```elixir\n%{\n  event_name: [:absinthe, :resolve, :field, :stop],\n  measurements: %{duration: 14000},\n  metadata: %{\n    id: -576460752303351647,\n    middleware: [\n      {{Absinthe.Resolution, :call}, &MyApp.Resolvers.resolve_field/3}\n    ],\n    resolution: :...,\n    start_time: 1565830447035742000\n  }\n}\n```","ref":"telemetry.html#interactive-telemetry","title":"Interactive Telemetry - Telemetry","type":"extras"},{"doc":"When using Opentelemetry, one usually wants to correlate spans that are created\nin spawned tasks with the main trace. For example, you might have a trace started\nin a Phoenix endpoint, and then have spans around database access.\n\nOne can correlate manually by attaching the OTel context the task function:\n\n```elixir\nctx = OpenTelemetry.Ctx.get_current()\n\nTask.async(fn ->\n  OpenTelemetry.Ctx.attach(ctx)\n\n  # do stuff that might create spans\nend)\n```\n\nWhen using the `async` and `batch` middleware, the tasks are spawned by Absinthe,\nso you can't attach the context manually.\n\nInstead, you can add the `:opentelemetry_process_propagator` package to your\ndependencies, which has a `Task.async/1` wrapper that will attach the context\nautomatically. If the package is installed, the middleware will use it in place\nof the default `Task.async/1`.","ref":"telemetry.html#opentelemetry","title":"Opentelemetry - Telemetry","type":"extras"},{"doc":"# Schema Deprecation\n\nUse the `deprecate` option when defining any field or enum value.\n\n- Provide a binary value to give a deprecation reason\n- Provide `true` to just mark it as deprecated\n\nAn example:\n\n```elixir\nquery do\n  field :old_item, :item, deprecate: true\n  field :another_old_item, :item, deprecate: \"still too old\"\nend\n```\n\nYou can also use the `deprecate` as a macro inside a block, for instance:\n\n```elixir\nfield :age, :integer do\n  deprecate\n  arg :user_id, non_null(:id)\nend\n```\n\nWith a reason:\n\n```elixir\nfield :ssn, :string do\n  deprecate \"Privacy concerns\"\nend\n```\n\n> ##","ref":"deprecation.html","title":"Schema Deprecation","type":"extras"},{"doc":">\n> Warning: Deprecated fields and enum values are not reported by default during [introspection](introspection.md).","ref":"deprecation.html#warning-warning","title":"Warning {: .warning} - Schema Deprecation","type":"extras"},{"doc":"# Document Adapters\n\nAbsinthe supports an adapter mechanism that allows developers to define their\nschema using one code convention (eg, `snake_cased` fields and arguments), but\naccept query documents and return results (including names in errors) in\nanother (eg, `camelCase`). This is useful in allowing both client and server to\nuse conventions most natural to them.\n\nAbsinthe ships with two adapters:\n\n* `Absinthe.Adapter.LanguageConventions`, which expects schemas to be defined\n  in `snake_case` (the standard Elixir convention), translating to/from `camelCase`\n  for incoming query documents and outgoing results. This is the default as of v0.3,\n  and it is _highly_ recommended that it's the adapter you use, as introspection\n  currently makes certain assumptions about how to return results.\n* `Absinthe.Adapter.Underscore`, which is similar to the `LanguageConventions`\n  adapter but converts all incoming identifiers to underscores and does not\n  modify outgoing identifiers (since those are already expected to be\n  underscores). Unlike `Absinthe.Adapter.Passthrough` this does not break\n  introspection.\n* `Absinthe.Adapter.Passthrough`, which is a no-op adapter and makes no\n  modifications.\n\nTo set the adapter, you provide it as an option to `Absinthe.run/3`:\n\n```elixir\nAbsinthe.run(query, MyAppWeb.Schema,\n             adapter: Absinthe.Adapter.TheAdapterName)\n```\n\nNotably, this means you're able to switch adapters on case-by-case basis.\nIn a Phoenix application, this means you could even support using different\nadapters for different clients.\n\nA custom adapter module must merely implement the `Absinthe.Adapter` behaviour,\nin many cases with `use Absinthe.Adapter` and only overriding the desired\nfunctions.\n\nNote that types that are defined external to your application (including\nthe introspection types) may not be compatible if you're using a different\nadapter.","ref":"adapters.html","title":"Document Adapters","type":"extras"},{"doc":"# Safety Limits\n\nA misbehaving client might send a very complex GraphQL query that would require\nconsiderable resources to handle. There are two variations of this problem:\n\n- Complex queries that overwhelm resolution resources.\n- Extremely long queries that could take considerable resources to parse. \n(For example, an attacker could craft a long query including thousands of \nundefined fields or directives.)\n\nEither of these could be a vector for a denial-of-service attack in any GraphQL \nservice. Absinthe includes mechanisms to protect services each of these.","ref":"complexity-analysis.html","title":"Safety Limits","type":"extras"},{"doc":"To protect against queries that could overwhelm available resources, the\ncomplexity of a query can be estimated before it is resolved and limited to a\nspecified maximum.\n\nFor example, to enable complexity analysis and limit the complexity to a value\nof `50` -- if we were using `Absinthe.run/3` directly -- we would do this:\n\n```elixir\nAbsinthe.run(doc, MyAppWeb.Schema, analyze_complexity: true, max_complexity: 50)\n```\n\nThat would translate to the following configuration when using\n[absinthe_plug](https://hex.pm/packages/absinthe_plug) (>= v1.2.3):\n\n```elixir\nplug Absinthe.Plug,\n  schema: MyAppWeb.Schema,\n  analyze_complexity: true,\n  max_complexity: 50\n```\n\nThe maximum value, `50`, is compared to complexity values calculated for each request.","ref":"complexity-analysis.html#complexity-analysis","title":"Complexity Analysis - Safety Limits","type":"extras"},{"doc":"Here's how the complexity value is calculated:\n\nBy default each field in a query will increase the complexity by 1. However, it\ncan be useful to customize how the complexity value for a field is calculated. This is done in your schema using the\n`complexity/1` macro, which can accept a function or an explicit integer value.\n\nAs an example, when a field is a list, the complexity is often correlated to the\nsize of the list. To prevent large selections, a field can use a limit argument\nwith a suitable default (think, for instance, of page sizes during pagination),\nand complexity can be calculated keeping that in mind. Here is a schema that\nsupports analyzing (and limiting) complexity using that approach:\n\n```elixir\ndefmodule MyAppWeb.Schema do\n\n  use Absinthe.Schema\n\n  query do\n    field :people, list_of(:person) do\n      arg :limit, :integer, default_value: 10\n      complexity fn %{limit: limit}, child_complexity ->\n        # set complexity based on maximum number of items in the list and\n        # complexity of a child.\n        limit * child_complexity\n      end\n    end\n  end\n\n  object :person do\n    field :name, :string\n    field :age, :integer\n  end\n\nend\n```\n\nFor a field, the first argument to the function you supply to `complexity/1` is the user arguments\n-- just as a field's resolver can use user arguments to resolve its value, the complexity\nfunction that you provide can use the same arguments to calculate the field's complexity.\n\nThe second argument passed to your complexity function is the sum of all the complexity scores\nof all the fields nested below the current field.\n\n(If a complexity function accepts three arguments, the third will be an\n`%Absinthe.Resolution{}` struct, just as with resolvers.)\n\nIf the value of a document's `:limit` argument was `10` and both `name` and `age` were queried for,\nthe complexity of the `:people` field would be calculated as `20`:\n\n* `10`, the value of `:limit`\n* times `2`, the sum of the complexity of the fields requested on the `:person`\n\nA field's complexity will default to `1` if it's not set explicitly.\n\nSo this would be okay:\n\n```graphql\n{\n  people(limit: 10) {\n    name\n  }\n}\n```\n\nBut this, at a complexity of `60`, wouldn't:\n\n```graphql\n{\n  people(limit: 30) {\n    name\n  }\n}\n```\n\n#","ref":"complexity-analysis.html#complexity-analysis","title":"Complexity Analysis - Safety Limits","type":"extras"},{"doc":"If a document's calculated complexity exceeds the configured limit, resolution\nwill be skipped and an error will be returned in the result detailing the\ncalculated and maximum complexities.","ref":"complexity-analysis.html#complexity-limiting","title":"Complexity limiting - Safety Limits","type":"extras"},{"doc":"To protect a service from invalid queries that could take considerable resources to parse, \nAbsinthe offers the option to configure a maximum limit on tokens in the GraphQL request document. \nIf the lexer encounters more tokens than this, it will stop the parse phase and return a phase error \nwith the message `\"Token limit exceeded\"`. This limit is `:infinity` by default (no limit) \nand can be overridden by providing an integer to the option `token_limit` to `Absinthe.run`.","ref":"complexity-analysis.html#token-limits","title":"Token Limits - Safety Limits","type":"extras"},{"doc":"# File Uploads\n\nAbsinthe provides a unique pattern to support file uploads in conjunction with normal GraphQL field arguments as part of the [absinthe_plug](https://hex.pm/packages/absinthe_plug) package.","ref":"file-uploads.html","title":"File Uploads","type":"extras"},{"doc":"The following schema includes a mutation field that accepts multiple uploaded files as arguments (`:users` and `:metadata`):\n\n```elixir\ndefmodule MyAppWeb.Schema do\n  use Absinthe.Schema\n\n  # Important: Needed to use the `:upload` type\n  import_types Absinthe.Plug.Types\n\n  mutation do\n    field :upload_file, :string do\n      arg :users, non_null(:upload)\n      arg :metadata, :upload\n\n      resolve fn args, _ ->\n        args.users # this is a `%Plug.Upload{}` struct.\n\n        {:ok, \"success\"}\n      end\n    end\n  end\nend\n```\n\nTo send a mutation that includes a file upload, you need to\nuse the `multipart/form-data` content type. For example, using `cURL`:\n\n```shell\n$ curl -X POST \\\n-F query=\"mutation { uploadFile(users: \\\"users_csv\\\", metadata: \\\"metadata_json\\\")}\" \\\n-F users_csv=@users.csv \\\n-F metadata_json=@metadata.json \\\nlocalhost:4000/graphql\n```\n\nNote how there is a correspondence between the value of the `:users` argument\nand the `-F` option indicating the associated file.\n\nBy treating uploads as regular arguments we get all the usual GraphQL argument\nbenefits (such as validation and documentation)---which we wouldn't get if\nwe were merely putting them in the context as in other implementations.","ref":"file-uploads.html#example","title":"Example - File Uploads","type":"extras"},{"doc":"* Apollo: [apollo-absinthe-upload-link](https://www.npmjs.com/package/apollo-absinthe-upload-link)\n* Apollo (v1): [apollo-absinthe-upload-client](https://www.npmjs.com/package/apollo-absinthe-upload-client) (Note: does not support Relay Native as of v1.0.1)\n* Relay: _(None known. Please submit a pull request updating this information.)_","ref":"file-uploads.html#integration-with-client-side-graphql-frameworks","title":"Integration with Client-side GraphQL Frameworks - File Uploads","type":"extras"},{"doc":"# Testing\n\nThere are three main approaches to testing GraphQL APIs built with Absinthe:\n\n1. Testing resolver functions, since they do most of the work.\n2. Testing GraphQL document execution directly via `Absinthe.run/3`, for the bigger picture.\n3. Outside-in, testing the full HTTP request/response cycle with [absinthe_plug](https://hexdocs.pm/absinthe_plug/Absinthe.Plug.html).\n\nThis guide focuses on the third approach, which we generally recommend since it exercises more\nof your application.","ref":"testing.html","title":"Testing","type":"extras"},{"doc":"GraphQL is transport independent, but it's most often served over HTTP. To test HTTP requests with `absinthe` you'll also need `absinthe_plug`. This guide will also assume you're using Phoenix, although \nit is possible to use Absinthe without it (see the [Plug and Phoenix Setup Guide](plug-phoenix.md)).","ref":"testing.html#testing-with-absinthe-plug","title":"Testing with Absinthe Plug - Testing","type":"extras"},{"doc":"Say we want to test the following schema:\n\n```elixir\ndefmodule MyAppWeb.Schema do\n  use Absinthe.Schema\n\n  @fakedb %{\n    \"1\" => %{name: \"Bob\", email: \"bubba@foo.com\"},\n    \"2\" => %{name: \"Fred\", email: \"fredmeister@foo.com\"}\n  }\n\n  query do\n    field :user, :user do\n      arg :id, non_null(:id)\n\n      resolve &find_user/2\n    end\n  end\n\n  object :user do\n    field :name, :string\n    field :email, :string\n  end\n\n  defp find_user(%{id: id}, _) do\n    {:ok, Map.get(@fakedb, id)}\n  end\nend\n```\n\nWhich we have exposed at the `/api` endpoint:\n\n```elixir\ndefmodule MyAppWeb.Router do\n  use Phoenix.Router\n\n  scope \"/api\" do\n    forward \"/\", Absinthe.Plug, schema: MyAppWeb.Schema\n  end\nend\n```\n\nThe test could look something like this:\n\n```elixir\ndefmodule MyAppWeb.SchemaTest do\n  use MyAppWeb.ConnCase\n\n  @user_query \"\"\"\n  query getUser($id: ID!) {\n    user(id: $id) {\n      name\n      email\n    }\n  }\n  \"\"\"\n\n  test \"query: user\", %{conn: conn} do\n    conn =\n      post(conn, \"/api\", %{\n        \"query\" => @user_query,\n        \"variables\" => %{id: 1}\n      })\n\n    assert json_response(conn, 200) == %{\n             \"data\" => %{\"user\" => %{\"email\" => \"bubba@foo.com\", \"name\" => \"Bob\"}}\n           }\n  end\nend\n\n```\n\nPhoenix generates the `MyAppWeb.ConnCase` test helper module. This supplies the\n`conn` variable containing the request and response. It also has helper functions \nsuch as [`post/3`](https://hexdocs.pm/phoenix/Phoenix.ConnTest.html#post/3)\nand [`json_response/2`](https://hexdocs.pm/phoenix/Phoenix.ConnTest.html#json_response/2).\n\nThe query is stored in the `@user_query` module attribute. We post this document to\nthe GraphQL endpoint at `/api`, along with a map of variables which will be \ntransformed to arguments for the `getUser` query.\n\nThe response to the query can then be directly asserted to be a JSON object of the right shape.","ref":"testing.html#example","title":"Example - Testing","type":"extras"},{"doc":"# Using with JavaScript\n\nYou can interact with an Absinthe GraphQL server via HTTP (thanks to\n`absinthe_plug`) and websockets (thanks to `absinthe_phoenix`):\n\nWe also have special support for configuring and working with specific\nJavaScript frameworks. You can see the guides here:\n\n- [Apollo Client](apollo.md)\n- [Relay](relay.md)","ref":"javascript.html","title":"Using with JavaScript","type":"extras"},{"doc":"To integrate a JavaScript application via HTTP, any standard GraphQL\nHTTP request (GET/POST) will do.\n\nHere's an example using\n[isomorphic-fetch](https://www.npmjs.com/package/isomorphic-fetch):\n\n``` javascript\nrequire('isomorphic-fetch');\n\nfetch('http://localhost:4000/graphql', {\n  method: 'POST',\n  headers: { 'Content-Type': 'application/json' },\n  body: JSON.stringify({ query: '{ posts { title } }' }),\n})\n  .then(res => res.json())\n  .then(res => console.log(res.data));\n```","ref":"javascript.html#over-http","title":"Over HTTP - Using with JavaScript","type":"extras"},{"doc":"See the [@absinthe/socket](https://github.com/absinthe-graphql/absinthe-socket/tree/master/packages/socket) NPM package\nfor special support for Absinthe's use of Phoenix channels for GraphQL over websockets, including support for\n[subscriptions](subscriptions.md).","ref":"javascript.html#over-websockets","title":"Over Websockets - Using with JavaScript","type":"extras"},{"doc":"# Using with Apollo Client\n\nAn Apollo client manages its connection to the GraphQL server using [links](https://www.apollographql.com/docs/link/) -- which are essentially middleware that tell Apollo how to resolve each query. You can configure Apollo to connect to your Absinthe server via HTTP, websockets, or both.","ref":"apollo.html","title":"Using with Apollo Client","type":"extras"},{"doc":"Using Apollo with an HTTP link does not require any Absinthe-specific configuration. You can create an HTTP link pointed at your Absinthe server as follows:\n\n```javascript\nimport { ApolloClient, InMemoryCache, createHttpLink } from \"@apollo/client\";\n\n// Create an HTTP link to the Absinthe server.\nconst link = createHttpLink({\n  uri: \"http://localhost:4000/graphql\"\n});\n\n// Apollo also requires you to provide a cache implementation\n// for caching query results. The InMemoryCache is suitable\n// for most use cases.\nconst cache = new InMemoryCache();\n\n// Create the client.\nconst client = new ApolloClient({\n  link,\n  cache\n});\n```\n\nYou may find that you need to modify the HTTP request that Apollo makes -- for example, if you wish to send the value of a particular cookie in the `Authorization` header. The `setContext` helper allows you to do this, and also demonstrates how links in Apollo can be chained.\n\n```javascript\nimport { ApolloClient, InMemoryCache, createHttpLink } from \"@apollo/client\";\nimport { setContext } from \"@apollo/client/link/context\";\nimport Cookies from \"js-cookie\";\n\n// Create an HTTP link to the Absinthe server.\nconst httpLink = createHttpLink({\n  uri: \"http://localhost:4000/graphql\"\n});\n\n// Use setContext to create a chainable link object that sets\n// the token cookie to the Authorization header.\nconst authLink = setContext((_, { headers }) => {\n  // Get the authentication token from the cookie if it exists.\n  const token = Cookies.get(\"token\");\n\n  // Add the new Authorization header.\n  return {\n    headers: {\n      ...headers,\n      authorization: token ? `Bearer ${token}` : \"\"\n    }\n  };\n});\n\n// Chain the HTTP link and the authorization link.\nconst link = authLink.concat(httpLink);\n\n// Apollo also requires you to provide a cache implementation\n// for caching query results. The InMemoryCache is suitable\n// for most use cases.\nconst cache = new InMemoryCache();\n\n// Create the client.\nconst client = new ApolloClient({\n  link,\n  cache\n});\n```","ref":"apollo.html#using-an-http-link","title":"Using an HTTP link - Using with Apollo Client","type":"extras"},{"doc":"An HTTP link is suitable for many basic use cases, but if you require two-way communication between the server and the client, you will need to use a websocket link. The most common use case is a client that needs to use GraphQL subscriptions to receive updates from the server when particular events occur. To implement a websocket link, we will need to use the [`@absinthe/socket`](https://www.npmjs.com/package/@absinthe/socket) and [`@absinthe/socket-apollo-link`](https://www.npmjs.com/package/@absinthe/socket-apollo-link) packages.\n\n```javascript\nimport { ApolloClient, InMemoryCache } from \"@apollo/client\";\nimport * as AbsintheSocket from \"@absinthe/socket\";\nimport { createAbsintheSocketLink } from \"@absinthe/socket-apollo-link\";\nimport { Socket as PhoenixSocket } from \"phoenix\";\nimport Cookies from \"js-cookie\";\n\n// Create a standard Phoenix websocket connection. If you need\n// to provide additional params, like an authentication token,\n// you can configure them in the `params` option.\nconst phoenixSocket = new PhoenixSocket(\"ws://localhost:4000/socket\", {\n  params: () => {\n    if (Cookies.get(\"token\")) {\n      return { token: Cookies.get(\"token\") };\n    } else {\n      return {};\n    }\n  }\n});\n\n// Wrap the Phoenix socket in an AbsintheSocket.\nconst absintheSocket = AbsintheSocket.create(phoenixSocket);\n\n// Create an Apollo link from the AbsintheSocket instance.\nconst link = createAbsintheSocketLink(absintheSocket);\n\n// Apollo also requires you to provide a cache implementation\n// for caching query results. The InMemoryCache is suitable\n// for most use cases.\nconst cache = new InMemoryCache();\n\n// Create the client.\nconst client = new ApolloClient({\n  link,\n  cache\n});\n```\n\n#","ref":"apollo.html#using-a-websocket-link","title":"Using a websocket link - Using with Apollo Client","type":"extras"},{"doc":"You may find that you periodically need to reconnect the websocket with different parameters. The most common case for this is when a user logs in or logs out; you will then want to refresh their subscriptions to reflect the new authentication state. You can accomplish this by invoking `phoenixSocket.conn.close();` from your application code whenever the reconnection needs to happen. Phoenix will notice the closed connection and automatically reconnect. It is important that you provide a function that returns the websocket parameters to the socket `params` option. If you provide the parameters directly as an object, the new parameters will not be picked up when the websocket reconnects, but if you provide a function, Phoenix invokes the function on each connection to obtain the parameters.\n\nNote that this solution (reconnecting with `phoenixSocket.conn.close();`) is somewhat unstable because it relies upon an implementation detail of the Phoenix socket. Ideally, a future version of the Phoenix package might add a public API method to reconnect the websocket with new parameters.\n\n```javascript\nimport { ApolloClient, InMemoryCache } from \"@apollo/client\";\nimport * as AbsintheSocket from \"@absinthe/socket\";\nimport { createAbsintheSocketLink } from \"@absinthe/socket-apollo-link\";\nimport { Socket as PhoenixSocket } from \"phoenix\";\nimport Cookies from \"js-cookie\";\n\n// Create a standard Phoenix websocket connection. If you need\n// to provide additional params, like an authentication token,\n// you can configure them in the `params` option.\n//\n// If you plan to reconnect the socket with updated parameters,\n// you must provide a function to the `params` option. If you\n// provide the parameters directly as an object, the updated\n// parameters will not be picked up when the socket reconnects.\nconst phoenixSocket = new PhoenixSocket(\"ws://localhost:4000/socket\", {\n  params: () => {\n    if (Cookies.get(\"token\")) {\n      return { token: Cookies.get(\"token\") };\n    } else {\n      return {};\n    }\n  }\n});\n\n// Wrap the Phoenix socket in an AbsintheSocket.\nconst absintheSocket = AbsintheSocket.create(phoenixSocket);\n\n// Create an Apollo link from the AbsintheSocket instance.\nconst link = createAbsintheSocketLink(absintheSocket);\n\n// Apollo also requires you to provide a cache implementation\n// for caching query results. The InMemoryCache is suitable\n// for most use cases.\nconst cache = new InMemoryCache();\n\n// Create the client.\nconst client = new ApolloClient({\n  link,\n  cache\n});\n\n// Later in your application code, when you need to reconnect\n// the socket.\nphoenixSocket.conn.close();\n```","ref":"apollo.html#reconnecting-the-websocket-link","title":"Reconnecting the websocket link - Using with Apollo Client","type":"extras"},{"doc":"A common configuration for Apollo client applications is to use both HTTP and websocket links -- HTTP for queries and mutations, and a websocket for subscriptions. We can implement this in our client using [directional composition with Apollo's `split` helper](https://www.apollographql.com/docs/react/api/link/introduction/#directional-composition).\n\n```javascript\nimport {\n  ApolloClient,\n  InMemoryCache,\n  createHttpLink,\n  split\n} from \"@apollo/client\";\nimport { getMainDefinition } from \"@apollo/client/utilities\";\nimport { setContext } from \"@apollo/client/link/context\";\nimport * as AbsintheSocket from \"@absinthe/socket\";\nimport { createAbsintheSocketLink } from \"@absinthe/socket-apollo-link\";\nimport { Socket as PhoenixSocket } from \"phoenix\";\nimport Cookies from \"js-cookie\";\n\n// Create an HTTP link to the Absinthe server.\nconst httpLink = createHttpLink({\n  uri: \"http://localhost:4000/graphql\"\n});\n\n// Use setContext to create a chainable link object that sets\n// the token cookie to the Authorization header.\nconst authLink = setContext((_, { headers }) => {\n  // Get the authentication token from the cookie if it exists.\n  const token = Cookies.get(\"token\");\n\n  // Add the new Authorization header.\n  return {\n    headers: {\n      ...headers,\n      authorization: token ? `Bearer ${token}` : \"\"\n    }\n  };\n});\n\n// Chain the HTTP link and the authorization link.\nconst authedHttpLink = authLink.concat(httpLink);\n\n// Create a standard Phoenix websocket connection. If you need\n// to provide additional params, like an authentication token,\n// you can configure them in the `params` option.\nconst phoenixSocket = new PhoenixSocket(\"ws://localhost:4000/socket\", {\n  params: () => {\n    if (Cookies.get(\"token\")) {\n      return { token: Cookies.get(\"token\") };\n    } else {\n      return {};\n    }\n  }\n});\n\n// Wrap the Phoenix socket in an AbsintheSocket.\nconst absintheSocket = AbsintheSocket.create(phoenixSocket);\n\n// Create an Apollo link from the AbsintheSocket instance.\nconst websocketLink = createAbsintheSocketLink(absintheSocket);\n\n// If the query contains a subscription, send it through the\n// websocket link. Otherwise, send it through the HTTP link.\nconst link = split(\n  (operation) => {\n    const definition = getMainDefinition(query);\n\n    return (\n      definition.kind === \"OperationDefinition\" &&\n      definition.operation === \"subscription\"\n    );\n  },\n  websocketLink,\n  authedHttpLink\n);\n\n// Apollo also requires you to provide a cache implementation\n// for caching query results. The InMemoryCache is suitable\n// for most use cases.\nconst cache = new InMemoryCache();\n\n// Create the client.\nconst client = new ApolloClient({\n  link,\n  cache\n});\n```","ref":"apollo.html#using-both-http-and-websocket-links","title":"Using both HTTP and websocket links - Using with Apollo Client","type":"extras"},{"doc":"# Using with Relay\n\nWhile GraphQL specifies what queries, mutations, and object types should look\nlike, Relay is a client-side implementation of an efficient data storage and\n(re-)fetching system that is designed to work with a GraphQL server.\n\nTo allow Relay to work its magic on the client side, all GraphQL queries and\nmutations need to follow certain conventions. `Absinthe.Relay` provides\nutilities to help you make your server-side schemas Relay-compatible while\nrequiring only minimal changes to your existing code.\n\n`Absinthe.Relay` supports three fundamental pieces of the Relay puzzle: *nodes*,\nwhich are normal GraphQL objects with a unique global ID scheme; *mutations*,\nwhich in Relay Classic conform to a certain input and output structure; and\n*connections*, which provide enhanced functionality around many-to-one lists\n(most notably pagination).","ref":"relay.html","title":"Using with Relay","type":"extras"},{"doc":"Make sure you have the [absinthe_relay](https://hex.pm/packages/absinthe_relay)\npackage [configured](https://github.com/absinthe-graphql/absinthe_relay#installation)\nas a dependency for your application.\n\nTo add Relay support schemas should start with `use Absinthe.Relay.Schema`, eg:\n\n```elixir\ndefmodule Schema do\n  use Absinthe.Schema\n  use Absinthe.Relay.Schema, :classic\n\n  # ...\n\nend\n```\n\n> Note that this schema is being prepared to support Relay Classic; if\n> you do not provide either a `:classic` or `:modern` option,\n> `:classic` is currently selected as the default, but a warning is\n> output; `:modern` will be the default option in v1.5.\n\nIf you're defining your types in a separate type module that you're using via\n`import_types` in your schema, use the `Notation` module instead:\n\n```elixir\ndefmodule Schema.Types do\n  use Absinthe.Schema.Notation\n  use Absinthe.Relay.Schema.Notation, :classic\n\n  # ...\n\nend\n```\n\nNow you're ready to implement the Relay features you need.","ref":"relay.html#using-absinthe-relay","title":"Using Absinthe.Relay - Using with Relay","type":"extras"},{"doc":"To enable Relay to be clever about caching and (re-)fetching data objects, your\nserver must assign a globally unique ID to each object before sending it down\nthe wire. Absinthe will take care of this for you if you provide some additional\ninformation in your schema.\n\nFirst of all, you must define a `:node` interface in your schema. Rather than\ndo this manually, `Absinthe.Relay` provides a macro so most of the configuration\nis handled for you.\n\nUse `node interface` in your schema:\n\n```elixir\nnode interface do\n  resolve_type fn\n    %YourApp.Model.Person{}, _ ->\n      :person\n    %YourApp.Model.Business{}, _ ->\n      :business\n    _, _ ->\n      nil\n  end\nend\n\n# ... mutations, queries ...\n```\n\nFor instance, if your query or mutation resolver returns:\n\n```elixir\n{:ok, %YourApp.Model.Business{id: 19, business_name: \"ACME Corp.\", employee_count: 52}}\n```\n\nAbsinthe will pattern-match the value to determine that the object type is\n`:business`. This becomes important when you configure your `:business` type as a `node`:\n\n```elixir\nnode object :business do  # <-- notice the macro prefix \"node\"\n  field :business_name, non_null(:string)\n  field :employee_count, :integer\nend\n```\n\n(Note the macro name, `node`, looks like a prefix to the `object` macro.)\n\nWhile it may appear that your `:business` object type only has two fields,\n`:business_name` and `:employee_count`, it actually has _three_. An `:id` field\nis configured for you because you used the `node object` macro, and because the\n`:node` interface knows how to identify the values returned from your resolvers,\nthat `:id` field is automatically set-up to convert internal (in this case,\nnumeric) IDs to the global ID scheme -- an opaque string (like `\"UWxf59AcjK=\"`)\nwill be returned instead.\n\n\n**Important:** the global ID is generated based on the object's\nunique identifier, which by default is **the value of its existing `:id`\nfield**. This is convenient, because if you are using Ecto, the\nprimary key `:id` database field is typically enough to uniquely identify an\nobject of a given type. It also means, however, that *the internal `:id` of a\nnode object will not be available to be queried as `:id`.*\n\n\n- If you wish to generate your global IDs based on something other than the\n  existing `:id` field (if, for instance, your internal IDs are returned as `_id`),\n  provide the `:id_fetcher` option (see the [documentation](https://hexdocs.pm/absinthe_relay/Absinthe.Relay.Node.html)).\n- If you wish to make your internal ID queryable, you must return it as a\n  different field (eg, you could define an `:internal_id` field whose resolver\n  extracts the raw, internal `:id` value from the source map/struct).\n\n#","ref":"relay.html#nodes","title":"Nodes - Using with Relay","type":"extras"},{"doc":"Ok, so your node objects provide a global `:id`. How does Relay use it?\n\nRelay expects you to provide a query field called `node` that accepts a global\nID (as arg `:id`) and returns the corresponding node object. Absinthe makes it\neasy to set this up -- use the `node field` macro inside your `query`.\n\n```elixir\nquery do\n  # ...\n  node field do\n    resolve fn\n      %{type: :person, id: id}, _ ->\n        # Get the person from the DB somehow, returning a tuple\n        YourApp.Resolver.Person.find(%{id: id}, %{})\n      %{type: :business, id: id}, _ ->\n        # Get the business from @businesses\n        {:ok, Map.get(@businesses, id)}\n      # etc.\n    end\n  end\n  # ... more queries ...\nend\n```\n\nNotice that the resolver for `node field` expects the first (args) argument to\ncontain a `:type` and `:id`. These are the node object type identifier and the\ninternal (non-global) ID, automatically parsed from the global ID. The resolver\nlooks up the correct value using the internal ID and returns a tuple, as usual.\n\nFor more information, see the [documentation](https://hexdocs.pm/absinthe_relay/Absinthe.Relay.Node.html).\n\n#","ref":"relay.html#node-query-field","title":"Node query field - Using with Relay","type":"extras"},{"doc":"If you need to parse a node (global) ID for use in a resolver, there is a\nhelpful utility, `parsing_node_ids/2` that is automatically imported for you.\nHere's an example of how it works.\n\nLet's assume we have a field, `:employees`, that returns a list of `:person`\nobjects for a given `:business_id` -- a node ID:\n\n```elixir\nquery do\n  field :employees, list_of(:people) do\n    arg :business_id, :id\n    resolve &resolve_employees/2\n  end\nend\n\ndef resolve_employees(%{business_id: global_id}, _) do\n  # But I need an internal ID to look-up the employees!\nend\n```\n\nIn `resolve_employees/2`, we could certainly parse out the internal ID manually.\nHere's how that would look:\n\n```elixir\ndef resolve_employees(%{business_id: global_id}, _) do\n  {:ok, %{type: :business, id: internal_id}} =\n    Absinthe.Relay.Node.from_global_id(global_id, YourApp.Schema)\n  # TODO: find employees using internal_id, return tuple\nend\n```\n\nObviously this can get a bit tedious if we have to do it often. Instead, we can\nuse `parsing_node_ids/2` to _wrap_ our resolver function to do the parsing for\nus, invoking our function with the internal ID instead. We just have to tell the\n`parsing_node_ids/2` what ID field arguments to parse and what the associated\ntypes should be:\n\n```elixir\nquery do\n  field :employees, list_of(:people) do\n    arg :business_id, :id\n    resolve parsing_node_ids(&resolve_employees/2, business_id: :business)\n  end\nend\n\ndef resolve_employees(%{business_id: internal_id}, _) do\n  # We have an internal ID!\nend\n```\n\nThis leaves our resolver function virtually unchanged, and keeps our code much\ncleaner.","ref":"relay.html#converting-node-ids-to-internal-ids-for-resolvers","title":"Converting node IDs to internal IDs for resolvers - Using with Relay","type":"extras"},{"doc":"Relay Classic sets some specific constraints around the way arguments and results for\nmutations are structured.\n\n> In Relay Modern (if you're using the `:modern` option when defining\n> the schema), you'll have access to a similar set of macros as\n> discussed here, but be aware that the constraints mentioned (on\n> `input`, on `clientMutationId`, etc) don't apply. See the\n> documentation for [Absinthe.Relay.Mutation.Notation.Modern](https://hexdocs.pm/absinthe_relay/Absinthe.Relay.Mutation.Notation.Modern.html) for more\n> specific instructions.\n\nRelay expects mutations to accept exactly one argument, `input`, an\n`InputObject`. On the JavaScript side, it automatically populates a field on the\ninput, `clientMutationId`, and expects to get it back, unchanged, as part of the\nresult. Thankfully `Absinthe.Relay` abstracts these details away from the schema\ndesigner, allowing them to focus on any _other_ arguments needed or results\nexpected.\n\n\n**Important:** Remember that input fields (and arguments in\ngeneral) cannot be of one of your `object` types. Use `input_object` to\nmodel complex argument types.\n\n\nIn this example, we accept a list of multiple `:person_input_object` values to\ninsert people into a database.\n\n```elixir\ndefmodule YourApp.Schema do\n  # ...\n\n  input_object :person_input_object do\n    field :first_name, non_null(:string)\n    field :last_name, non_null(:string)\n    field :age, :integer\n  end\n\n  mutation do\n\n    @desc \"A mutation that inserts a list of persons into the database\"\n    payload field :bulk_create_persons do\n      input do\n        field :persons, list_of(:person_input_object)\n      end\n      output do\n        # fields in the result\n      end\n      resolve &Resolver.Person.bulk_create/2\n    end\n\n    # ... more mutations ...\n  end\nend\n```\n\nNote the `payload` macro introduces a Relay mutation, `input` defines the fields\n(inside the `input` argument), and `output` defines the fields available as part\nof the result.\n\nSee the module documentation for more information:\n\n- [Absinthe.Relay.Mutation.Notation.Classic](https://hexdocs.pm/absinthe_relay/Absinthe.Relay.Mutation.Notation.Classic.html)\n- [Absinthe.Relay.Mutation.Notation.Modern](https://hexdocs.pm/absinthe_relay/Absinthe.Relay.Mutation.Notation.Modern.html)\n\n#","ref":"relay.html#mutations","title":"Mutations - Using with Relay","type":"extras"},{"doc":"Occasionally, your client may wish to make reference to an existing node in the\nmutation input (this happens particularly when manipulating the connection edges\nof a parent node).\n\nIncoming IDs for node types may have to be converted to their internal\nequivalents so you can persist changes to your backend. For this purpose, you\ncan use `Absinthe.Relay.Node.from_global_id/2` to parse node (global) IDs\nmanually.\n\n```elixir\ndef bulk_create(%{persons: new_persons, group: global_group_id}, _) do\n  {:ok, %{type: :group, id: internal_group_id}} = Absinthe.Relay.Node.from_global_id(global_group_id, YourApp.Schema)\n  # ... manipulate your DB using internal_group_id\nend\n```\n\nIf, of course, your client knows the internal IDs (in a peer field to `:id`, eg,\n`:internal_id`), you can depends on that ID -- but we recommend that you use\nnode IDs as they are opaque values and it's the more conventional practice.\n\n\n**Important:** When using `from_global_id`, remember to always\nmatch the `:type` value to ensure the internal ID is for the type you expect,\nand a global ID for the wrong type of node hasn't been mistakenly sent to the\nserver.","ref":"relay.html#referencing-existing-nodes-in-mutation-inputs","title":"Referencing existing nodes in mutation inputs - Using with Relay","type":"extras"},{"doc":"One of the more popular features of Relay is the rich pagination support provided by its\nconnections. [This medium post](https://dev-blog.apollodata.com/explaining-graphql-connections-c48b7c3d6976)\nhas a good explanation of the full feature set and nomenclature.\n\nFor example, you could define a connection for paginating `:location` objects with:\n\n`connection node_type: :location`\n\nThis will automatically define two new types: `:location_connection` and `:location_edge`.\n\nWe define a field that uses these types to paginate associated records by using\n`connection field`. Here, for instance, we support paginating a business’s locations:\n\n```elixir\nobject :business do\n  field :short_name, :string\n  connection field :locations, node_type: :location do\n    resolve fn\n      pagination_args, %{source: business} ->\n        Location\n        |> where(business_id: ^business.id)\n        |> order_by(:inserted_at)\n        |> Connection.from_query(&Repo.all/1, pagination_args)\n    end\n  end\nend\n```\n\nWe are piping a query for the associated locations into `from_query/3` along with the default\nrelay pagination arguments that allow for pagination. For example, to get just the first 10\nlocations, use the `first` argument:\n\n```graphql\nquery {\n  business(id:\"9ea6605e-e6c8-44ea-98d0-1fe6276e193d\") {\n    shortName\n    locations(first:10) {\n      edges\n        node {\n          address1\n          city\n        }\n      }\n    }\n  }\n}\n```\n\nCheck the [documentation](https://hexdocs.pm/absinthe_relay/Absinthe.Relay.Connection.html)\nfor more details on connections.\n\n\n**Note:** These features do not require using Relay on the client as Apollo\nand other client implementations generally support Relay connection configuration.","ref":"relay.html#connections","title":"Connections - Using with Relay","type":"extras"},{"doc":"# Upgrading to v1.4\n\n> This information is extracted and expanded from the CHANGELOG.\n\nThis version included [subscriptions](subscriptions.md), and also came packaged with a number of improvements that required breaking changes.\n\nThe breaking changes primarily affect middleware and plugin authors, but some changes (like `null` handling and changes to error messages) warrant review by all Absinthe users.","ref":"v1-4.html","title":"Upgrading to v1.4","type":"extras"},{"doc":"Default middleware are now applied eagerly. Although a small change, this will affect anyone who is currently changing the default middleware.\n\n#","ref":"v1-4.html#middleware-watch-out-for-eager-default","title":"Middleware: Watch Out for Eager Default - Upgrading to v1.4","type":"extras"},{"doc":"Before v1.4, the default middleware was applied \"lazily\". What this means is if you had a simple field like:\n\n```elixir\nobject :user do\n  field :name, :string\nend\n```\n\nthen when it is passed to the `middleware/3` callback on a schema, the middleware is an empty list:\n\n```elixir\ndef middleware(middleware, %{identifier: :name}, %{identifier: :user}) do\n  middleware |> IO.inspect\n  #=> []\nend\n```\n\nThe nice thing about this was that it made it easy to pattern match for the \"no middleware supplied\" case; you could just match against `[]`.\n\nThe problem, however, is that if you wanted to add a simple tracing middleware that runs on every field for example, the \"obvious\" way to do it seemed like it could be this:\n\n```elixir\ndef middleware(middleware, _field, _object) do\n  [YourTracer | middleware]\nend\n```\n\n... and we just broke our `field :name, :string` field, and all others like it. No longer did it come back from `def middleware` as `[]`, and thus our lazy default wasn't applied.\n\nThis has tripped up an unreasonable number of people, and it violated the common sense meaning of \"default value.\" It also made it hard for users to determine what default middleware might run on a field. Checking the value passed to `middleware/3` didn't help.\n\n#","ref":"v1-4.html#before-v1-4","title":"Before v1.4 - Upgrading to v1.4","type":"extras"},{"doc":"The default middleware is now _eager_:\n\n```elixir\ndef middleware(middleware, %{identifier: :name}, %{identifier: :user}) do\n  middleware |> IO.inspect\n  #=> [{Absinthe.Middleware.MapGet, :name}]\nend\n```\n\nEvery field has at least one middleware specified, and the `middleware/3` callback has full access to it. Conceptually, it's a lot simpler than the previous approach, in that there isn't some hidden action after the fact that you can't see.\n\nChanging the default is a bit more work now, because you have to explicitly match against the Absinthe default to put something else in its place. However it isn't very difficult (moreover a helper function could be added later to make it even easier).","ref":"v1-4.html#in-v1-4","title":"In v1.4 - Upgrading to v1.4","type":"extras"},{"doc":"Plugins now receive an `%Absinthe.Blueprint.Execution{}` struct instead of the bare accumulator. This makes it possible for plugins to set or operate on context values. Upgrade your plugins! Change this:\n\n```elixir\ndef before_resolution(acc) do\n  acc = # doing stuff to the acc here\nend\ndef after_resolution(acc) do\n  acc = # doing stuff to the acc here\nend\ndef pipeline(pipeline, acc) do\n  case acc do\n    # checking on the acc here\n  end\nend\n```\nto:\n\n```elixir\ndef before_resolution(%{acc: acc} = exec) do\n  acc = # doing stuff to the acc here\n  %{exec | acc: acc}\nend\ndef after_resolution(%{acc: acc} = exec) do\n  acc = # doing stuff to the acc here\n  %{exec | acc: acc}\nend\ndef pipeline(pipeline, exec) do\n  case exec.acc do\n    # checking on the acc here\n  end\nend\n```\n\nThe reason for this is that you can also access the `context` within the `exec` value. When using something like [Dataloader](dataloader.md), it's important to have easy to the context","ref":"v1-4.html#plugins-change-arguments","title":"Plugins: Change Arguments - Upgrading to v1.4","type":"extras"},{"doc":"Absinthe now supports [GraphQL `null` literals](https://spec.graphql.org/October2016/#sec-Null-Value).\n\n`null` values, when provided as arguments, are passed on to Absinthe resolvers as `nil` (provided they don't run afoul of a `non_null/1` argument constraint).\n\nThe concrete effect of this is that your resolvers may need to be updated to take into account that `nil` is a possible value; in the past arguments would _never_ be passed as `nil`.\n\nLet's take a look at this example schema snippet.\n\n#","ref":"v1-4.html#calling-all-resolvers-the-null-literal-has-arrived","title":"Calling All Resolvers: The Null Literal Has Arrived - Upgrading to v1.4","type":"extras"},{"doc":"``` elixir\nfield :avatar_url, :string do\n  arg :size, :integer, default_value: 64\n  # ... resolver here\nend\n```\n\nBefore v1.4, you could write a resolver that could _always_ assume a `:size` would be provided:\n\n``` elixir\nresolve fn _, %{size: size}, _ ->\n  {:ok, \"http://example.com/avatars/test_#{size}x#{size}.png\"}\nend\n```\n\nIf the user doesn't provide a value for `:size`, it would default to `64`:\n\n``` graphql\n{\n  avatarUrl(size: 32) # => http://example.com/avatars/test_32x32.png\n  avatarUrl           # => http://example.com/avatars/test_64x64.png\n  # This is invalid in v1.3, as there's no `null` literal:\n  avatarUrl(size: null)\n}\n```\n\n#","ref":"v1-4.html#before-v1-4","title":"Before v1.4 - Upgrading to v1.4","type":"extras"},{"doc":"In v1.4, users can freely send `null`, and _that value will override any default_:\n\n``` graphql\n{\n  # This didn't change:\n  avatarUrl(size: 32)   # => http://example.com/avatars/test_32x32.png\n  # Neither did this (default value applied as expected)\n  avatarUrl             # => http://example.com/avatars/test_64x64.png\n  # But `nil` gets interpolated here, as it overrode the default value of `64`:\n  avatarUrl(size: null) # => http://example.com/avatars/test_x.png\n}\n```\n\nYou may want to change your resolvers to use the default value if `nil` is received:\n\n``` elixir\nresolve fn _, %{size: size}, _ ->\n  px = size || 64\n  {:ok, \"http://example.com/avatars/test_#{px}x#{px}.png\"}\nend\n```\n\nOr handle it as an explicit match (especially if you want to assign a special semantic value to the now-available `null` value):\n\n``` elixir\nresolve fn\n  _, %{size: nil}, _ ->\n    {:ok, \"http://example.com/avatars/test_default.png\"}\n  _, %{size: size}, _ ->\n    {:ok, \"http://example.com/avatars/test_#{size}x#{size}.png\"}\nend\n```\n\nSome other options to think about:\n\n- Shore up your schema by adding more uses of `non_null/1` constraints where it makes sense.\n- Add  piece of middleware to strip out `nil` argument values, universally or focused on a specific set of fields.","ref":"v1-4.html#in-v1-4","title":"In v1.4 - Upgrading to v1.4","type":"extras"},{"doc":"Errors returned from resolvers no longer say `\"In field #{field_name}:\"`. The inclusion of the path information obviates the need for this data, and it makes error messages a lot easier to deal with on the frontend.\n\nIf you expect certain error messages in your clients, you may need to update your code.","ref":"v1-4.html#expect-shorter-error-messages","title":"Expect Shorter Error Messages - Upgrading to v1.4","type":"extras"},{"doc":"# Upgrading to v1.5\n\n> This information is extracted and expanded from the CHANGELOG.\n\nThis version includes:\n* Schema compilation phases\n* SDL based schema definitions\n* SDL rendering\n* Telemetry based instrumentation\n\nExisting macro-based schemas will work as-is, but make sure to note that the schema pipeline executes at compile time.","ref":"v1-5.html","title":"Upgrading to v1.5","type":"extras"},{"doc":"#","ref":"v1-5.html#breaking-changes","title":"Breaking changes - Upgrading to v1.5","type":"extras"},{"doc":"Default values are evaluated at compile time. For example `default_value: DateTime.utc_now()` will have its time set at compile time. You probably don't want this :)\n\n#","ref":"v1-5.html#default-values-evaluated-at-compile-time","title":"Default values evaluated at compile time - Upgrading to v1.5","type":"extras"},{"doc":"Scalar outputs are now type checked and will raise exceptions if the result tries to send the wrong data type in the result.\n\n#","ref":"v1-5.html#scalar-output-validation","title":"Scalar output validation - Upgrading to v1.5","type":"extras"},{"doc":"Variable types must now align exactly with the argument type. Previously Absinthe allowed variables of different types to be used by accident as long as the data parsed.\n\n#","ref":"v1-5.html#variable-types-validation","title":"Variable types validation - Upgrading to v1.5","type":"extras"},{"doc":"Added a schema phase to check the validity of field names according to GraphQL spec. Remove the `Absinthe.Phase.Schema.Validation.NamesMustBeValid` from the schema pipeline if you need to retain the previous behavior.\n\n### `Absinthe.Subscription.PubSub`\n\nAdded `node_name/0` callback to `Absinthe.Subscription.PubSub` behaviour. To retain old behaviour, implement this callback to return `Kernel.node/0`.\n\n### `Absinthe.Traversal`\n\nRemoved the un-used `Absinthe.Traversal` module.","ref":"v1-5.html#field-name-validation","title":"Field name validation - Upgrading to v1.5","type":"extras"},{"doc":"# Changelog","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"- Bugfix: [Handle non_null(list_of(:thing)) with null list elements properly](https://github.com/absinthe-graphql/absinthe/pull/1259)\n- Bugfix: [More non null result handling improvements](https://github.com/absinthe-graphql/absinthe/pull/1275)","ref":"changelog.html#unreleased","title":"Unreleased - Changelog","type":"extras"},{"doc":"- Feature: Support Dataloader 2.0","ref":"changelog.html#1-7-4","title":"1.7.4 - Changelog","type":"extras"},{"doc":"- Bug Fix: [Bugfix: multiple pushes per client for subscriptions that have a context_id](https://github.com/absinthe-graphql/absinthe/pull/1249)","ref":"changelog.html#1-7-4","title":"1.7.4 - Changelog","type":"extras"},{"doc":"- Bug Fix: [OTP 26 and Elixir 1.15 tweaks](https://github.com/absinthe-graphql/absinthe/pull/1253)\n- Bug Fix: [OTP 25 tweaks](https://github.com/absinthe-graphql/absinthe/pull/1253)\n- Bug Fix: [Place extra error attributes in error extensions field](https://github.com/absinthe-graphql/absinthe/pull/1215)","ref":"changelog.html#1-7-3","title":"1.7.3 - Changelog","type":"extras"},{"doc":"- Bug Fix: [Validate type references for invalid wrapped types](https://github.com/absinthe-graphql/absinthe/pull/1195)\n- Feature: [Add `specifiedBy` type system directive](https://github.com/absinthe-graphql/absinthe/pull/1193)\n- Bug Fix: [Object type extensions may be empty](https://github.com/absinthe-graphql/absinthe/pull/1228)\n- Bug Fix: [Validate input object not being an Enum](https://github.com/absinthe-graphql/absinthe/pull/1231)\n- Bug Fix: [Deduplicate directives when building schema](https://github.com/absinthe-graphql/absinthe/pull/1242)","ref":"changelog.html#1-7-2","title":"1.7.2 - Changelog","type":"extras"},{"doc":"- Breaking Bugfix: [Validate repeatable directives on schemas](https://github.com/absinthe-graphql/absinthe/pull/1179)\n- Breaking Bugfix: [Add \"Objects must define fields\" schema validation](https://github.com/absinthe-graphql/absinthe/pull/1167)\n- Bug Fix: [Validate field identifier uniqueness](https://github.com/absinthe-graphql/absinthe/pull/1200)\n- Bug Fix: [Validate type references for invalid wrapped types](https://github.com/absinthe-graphql/absinthe/pull/1195)\n- Bug Fix: Adds **optional fix** for non compliant built-in scalar Int type. `use Absinthe.Schema, use_spec_compliant_int_scalar: true` in your schema to use the fixed Int type. It is also advisable to upgrade for custom types if you are leveraging the use of integers outside the GraphQl standard. [#1131](https://github.com/absinthe-graphql/absinthe/pull/1131).\n- Feature: [Support error tuples when scalar parsing fails](https://github.com/absinthe-graphql/absinthe/pull/1187)\n- Feature: [Convert SDL Language.\\* structs to SDL notation](https://github.com/absinthe-graphql/absinthe/pull/1160)\n- Feature: [Support passing the resolution struct to dataloader helper callbacks](https://github.com/absinthe-graphql/absinthe/pull/1211)\n- Feature: [Add support for type extensions](https://github.com/absinthe-graphql/absinthe/pull/1157)\n- Bug Fix: [Add type system directives to introspection results](https://github.com/absinthe-graphql/absinthe/pull/1189)\n- Bug Fix: [Add `__private__` field to EnumValueDefinition](https://github.com/absinthe-graphql/absinthe/pull/1148)\n- Bug Fix: [Fix bug in Schema.**absinthe_types**(:all) for Persistent Term](https://github.com/absinthe-graphql/absinthe/pull/1161)\n- Bug Fix: [Fix default enum value check for SDL schema's](https://github.com/absinthe-graphql/absinthe/pull/1188)\n- Feature: [Add `import_directives` macro](https://github.com/absinthe-graphql/absinthe/pull/1158)\n- Feature: [Support type extensions on schema declarations](https://github.com/absinthe-graphql/absinthe/pull/1176)\n- Bug Fix: [Root objects are marked as referenced correctly](https://github.com/absinthe-graphql/absinthe/pull/1186)\n- Bug Fix: [Prevent DDOS attacks with long queries](https://github.com/absinthe-graphql/absinthe/pull/1220)\n- Feature: [pipeline_modifier option to Absinthe.run/3](https://github.com/absinthe-graphql/absinthe/pull/1221)\n- Bug Fix: [Add end_time_mono to telemetry :stop events](https://github.com/absinthe-graphql/absinthe/pull/1174)","ref":"changelog.html#1-7-1","title":"1.7.1 - Changelog","type":"extras"},{"doc":"- POTENTIALLY BREAKING Bug Fix: [Validate variable usage in according to spec](https://github.com/absinthe-graphql/absinthe/pull/1141). This could break incoming documents previously considered valid. Skip the `Absinthe.Phase.Document.Arguments.VariableTypesMatch` phase to avoid this check. See `Absinthe.Pipeline` on adjusting the document pipeline.\n\n- Feature: [Add GraphQL document formatter](https://github.com/absinthe-graphql/absinthe/pull/1114)\n- Bug Fix: [Fix Phase.Schema.Validation.InputOutputTypesCorrectlyPlaced not applied to SDL schema's](https://github.com/absinthe-graphql/absinthe/pull/1142/files)\n- Bug Fix: [Use inspect/1 to safely encode bad binary samples](https://github.com/absinthe-graphql/absinthe/pull/1121)\n- Bug Fix: [key :is_type_of not found on Interface ](https://github.com/absinthe-graphql/absinthe/issues/1077)\n- Bug Fix: [Validate object/interfaces implement all transitive interfaces](https://github.com/absinthe-graphql/absinthe/pull/1127)\n- Bug Fix: [Fix check unknown types to also cover wrapped types](https://github.com/absinthe-graphql/absinthe/pull/1138) This could break incoming documents previously considered valid. Skip the `Absinthe.Phase.Validation.KnownTypeNames` phase to avoid this check. See `Absinthe.Pipeline` on adjusting the document pipeline.\n- Bug Fix: [Validate field names are unique to an object, interface or an input object](https://github.com/absinthe-graphql/absinthe/pull/1135)","ref":"changelog.html#1-7-0","title":"1.7.0 - Changelog","type":"extras"},{"doc":"Originally included the items from 1.7.0, but the spec validation fix was considered\ntoo impactful for a patch release.","ref":"changelog.html#1-6-7-retired","title":"1.6.7 (Retired) - Changelog","type":"extras"},{"doc":"- Feature: [Update telemetry dependency to stable ~> 1.0](https://github.com/absinthe-graphql/absinthe/pull/1097)\n- Feature: [Use makeup_graphql to get GraphQL syntax highlighting in docs](https://github.com/absinthe-graphql/absinthe/pull/1099)\n- Bug Fix: [Fix exception when field name contains all invalid characters](https://github.com/absinthe-graphql/absinthe/pull/1096)","ref":"changelog.html#1-6-6","title":"1.6.6 - Changelog","type":"extras"},{"doc":"- Bug Fix: [Fix interface compilation behavior difference between SDL & DSL](https://github.com/absinthe-graphql/absinthe/pull/1091)\n- Bug Fix: [Allow SDL syntax to contain union without member types](https://github.com/absinthe-graphql/absinthe/pull/1085)\n- Bug Fix: [Account for prototype schema when rendering SDL via mix task](https://github.com/absinthe-graphql/absinthe/pull/1086)\n- Feature: Always inline functions when using persistent_term backend.\n- Feature: [Support optional open ended scalars](https://github.com/absinthe-graphql/absinthe/pull/1069)","ref":"changelog.html#1-6-5","title":"1.6.5 - Changelog","type":"extras"},{"doc":"- Feature: [Compress registry tables by default](https://github.com/absinthe-graphql/absinthe/pull/1058)\n- Bug Fix: [Fix compilation deadlocks on type imports](https://github.com/absinthe-graphql/absinthe/pull/1056)\n- Bug Fix: [Raise a better error when string serialization fails](https://github.com/absinthe-graphql/absinthe/pull/1062)","ref":"changelog.html#1-6-4","title":"1.6.4 - Changelog","type":"extras"},{"doc":"- Bug Fix: [Fix unicode bug when encoding parse error](https://github.com/absinthe-graphql/absinthe/pull/1044)","ref":"changelog.html#1-6-3","title":"1.6.3 - Changelog","type":"extras"},{"doc":"- Bug Fix: [Fix regression in SDL rendering for enum values](https://github.com/absinthe-graphql/absinthe/pull/1041)","ref":"changelog.html#1-6-2","title":"1.6.2 - Changelog","type":"extras"},{"doc":"- Feature: [Improved serialization failure messages](https://github.com/absinthe-graphql/absinthe/pull/1033)\n- Bug Fix: [Render null default values in SDL](https://github.com/absinthe-graphql/absinthe/pull/1032)\n- Bug Fix: [Reduce memory usage of Schema.Manager](https://github.com/absinthe-graphql/absinthe/pull/1037)","ref":"changelog.html#1-6-1","title":"1.6.1 - Changelog","type":"extras"},{"doc":"- Feature: [Interfaces can now implement Interfaces](https://github.com/absinthe-graphql/absinthe/pull/1012), matching the latest spec\n- Feature: Support for the [`repeatable` directive](https://github.com/absinthe-graphql/absinthe/pull/999)\n- Feature: Enable [rendering](https://github.com/absinthe-graphql/absinthe/pull/1010) of Type System Directives in SDL based schemas.\n- Feature: Correctly match [Introspection type specs](https://github.com/absinthe-graphql/absinthe/pull/1017)\n- Bug Fix: Restore dynamic [description support](https://github.com/absinthe-graphql/absinthe/pull/1005) (Note: the `description`s are evaluated once --- at compile time)\n- Bug Fix: Restore dynamic [default_value support](https://github.com/absinthe-graphql/absinthe/pull/1026) (Note: the `default_value`s evaluated once --- at compile time)\n- Bug Fix: Restore dynamic [Enum value support](https://github.com/absinthe-graphql/absinthe/pull/1023) (Note: the `value` is evaluated once --- at compile time)\n- Bug Fix: [Interface nullability](https://github.com/absinthe-graphql/absinthe/pull/1009) corrections\n- Bug Fix: Fix [field listing for Inputs](https://github.com/absinthe-graphql/absinthe/pull/1015) that import fields\n- Bug Fix: Properly [trim all descriptions](https://github.com/absinthe-graphql/absinthe/pull/1014) no matter the mechanism used to specify them\n- Bug Fix: Fix incorrect specification of [`__TypeKind`](https://github.com/absinthe-graphql/absinthe/pull/1019)\n- Bug Fix: Better match [introspection schema specification](https://github.com/absinthe-graphql/absinthe/pull/1029)\n- Bug Fix: Add missing value to [`__DirectiveLocation`](https://github.com/absinthe-graphql/absinthe/pull/1020)\n- Bug Fix: Fix [compilation problems with `import_types`](https://github.com/absinthe-graphql/absinthe/pull/1022)\n- Bug Fix: Reduce [memory consumption of Subscriptions](https://github.com/absinthe-graphql/absinthe/pull/1006)","ref":"changelog.html#1-6-0","title":"1.6.0 - Changelog","type":"extras"},{"doc":"- Bug Fix: Fix for `nil` in [`ArgumentsOfCorrectType` suggestions](https://github.com/absinthe-graphql/absinthe/pull/1000)","ref":"changelog.html#1-5-5","title":"1.5.5 - Changelog","type":"extras"},{"doc":"- Feature: Ensure [stable ordering in introspection results](https://github.com/absinthe-graphql/absinthe/pull/997).\n- Bug Fix: Fix [rendering of interfaces in SDL](https://github.com/absinthe-graphql/absinthe/pull/979)\n- Bug Fix: Properly [escape single line descriptions in SDL](https://github.com/absinthe-graphql/absinthe/pull/968)\n- Bug Fix: Fix [`:meta` on fields](https://github.com/absinthe-graphql/absinthe/pull/973)\n- Bug Fix: Validate that [DirectivesMustBeValid](https://github.com/absinthe-graphql/absinthe/pull/954)\n- Bug Fix: Handle [default value rendering with partial field set](https://github.com/absinthe-graphql/absinthe/pull/998)","ref":"changelog.html#1-5-4","title":"1.5.4 - Changelog","type":"extras"},{"doc":"- Bug Fix: Handle null propagation with `non_null(list_of(non_null(type)))` properly\n- Bug Fix: Fix [double escaping issue](https://github.com/absinthe-graphql/absinthe/pull/962) with string literal arguments.","ref":"changelog.html#1-5-3","title":"1.5.3 - Changelog","type":"extras"},{"doc":"- Bug Fix: Fix issue with persistent term backend.","ref":"changelog.html#1-5-2","title":"1.5.2 - Changelog","type":"extras"},{"doc":"- Bug Fix: Enable hydrating resolve_type on unions. #938\n- Bug Fix: #922","ref":"changelog.html#1-5-1","title":"1.5.1 - Changelog","type":"extras"},{"doc":"- Breaking Bug Fix: Variable types must align exactly with the argument type. Previously\n  Absinthe allowed variables of different types to be used by accident as long as the data parsed.\n- Feature (Experimental): `:persistent_term` based schema backend\n- Breaking Change: `telemetry` event keys [changed](https://github.com/absinthe-graphql/absinthe/pull/901) since the beta release.","ref":"changelog.html#v1-5-0-rc","title":"v1.5.0 (Rc) - Changelog","type":"extras"},{"doc":"- Feature: SDL directives, other improvements\n- Feature: Output rendered SDL for a schema\n- Feature: Substantially lower subscription memory usage.\n- Documentation: Testing guide, numerous fixes and updates\n- Breaking Change: Scalar outputs are now type checked and will raise exceptions if the result tries to send the wrong data type in the result.\n- Breaking Change: `telemetry` event names [changed](https://github.com/absinthe-graphql/absinthe/pull/782) from the `alpha` to match an emerging naming convention for tracing.\n- Breaking Change: Added phase to check validity of field names according to GraphQL spec. Might break existing schema's. Remove the `Absinthe.Phase.Schema.Validation.NamesMustBeValid` from the schema pipeline if you want to ignore this.\n- Breaking Change: To match the GraphQL spec, we [no longer](https://github.com/absinthe-graphql/absinthe/pull/816) add a non-null error when a resolver on a non-null field explicitly returns its own error.\n- Breaking Change: Removed `Absinthe.Traversal` module","ref":"changelog.html#v1-5-0-beta","title":"v1.5.0 (Beta) - Changelog","type":"extras"},{"doc":"Alpha 0 note: 1.5.0 alpha is safe to use on existing schemas. However, there are no schema validations at the moment, so when writing new ones you may get undefined behaviour if you write an invalid schema.\n\n- COLUMN NUMBERS! The Absinthe Lexer has been rewritten using `nimble_parsec` and now Absinthe includes column information.\n- Complete rewrite of schema internals. This fixes a number of long standing bugs, and provides a variety of new features\n- Feature: SDL support\n- Feature: Schema decorators\n- Feature: Customizable subscription de-duplification. See: https://github.com/absinthe-graphql/absinthe/blob/master/guides/subscriptions.md#de-duplicating-updates\n- Feature: Built-in `telemetry` instrumentation (https://github.com/beam-telemetry/telemetry)\n- Breaking Change: `default_value: DateTime.utc_now()` will have its time set at compile time. IE: DON'T DO THIS. It only worked by accident before anyway, and now it no longer works, which is correct.\n- Breaking change: added `node_name/0` callback to `Absinthe.Subscription.PubSub` behaviour. To retain old behaviour, implement this callback to return `Kernel.node/0`.","ref":"changelog.html#v1-5-0-alpha","title":"v1.5.0 (Alpha) - Changelog","type":"extras"},{"doc":"For changes pre-v1.5 see the [v1.4](https://github.com/absinthe-graphql/absinthe/blob/v1.4/CHANGELOG.md) branch.","ref":"changelog.html#v1-4","title":"v1.4 - Changelog","type":"extras"}]}